/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-edit {
  background-image: url('themes/wikimediaui/images/icons/edit.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/edit.svg');
}
.oo-ui-icon-editLock {
  background-image: url('themes/wikimediaui/images/icons/editLock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editLock.svg');
}
.oo-ui-icon-editUndo {
  background-image: url('themes/wikimediaui/images/icons/editUndo-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editUndo-ltr.svg');
}
.oo-ui-icon-link {
  background-image: url('themes/wikimediaui/images/icons/link.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/link.svg');
}
.oo-ui-icon-unLink {
  background-image: url('themes/wikimediaui/images/icons/unLink.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLink.svg');
}
.oo-ui-icon-linkExternal {
  background-image: url('themes/wikimediaui/images/icons/linkExternal-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkExternal-ltr.svg');
}
.oo-ui-icon-linkSecure {
  background-image: url('themes/wikimediaui/images/icons/linkSecure.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkSecure.svg');
}
.oo-ui-icon-redo {
  background-image: url('themes/wikimediaui/images/icons/redo-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/redo-ltr.svg');
}
.oo-ui-icon-undo {
  background-image: url('themes/wikimediaui/images/icons/undo-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/undo-ltr.svg');
}
