/*!
 * Styles for elements of the editing form, loaded only when JavaScript is enabled.
 */

.mw-toolbar-editbutton {
	width: 23px;
	height: 22px;
	cursor: pointer;
	vertical-align: middle;
	display: inline-block;
}
