/*!
 * VisualEditor stand-alone Initialization namespace.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

/**
 * Namespace for all VisualEditor stand-alone Initialization classes, static methods and static
 * properties.
 *
 * @class
 * @singleton
 */
ve.init.sa = {
};
