@import 'mediawiki.mixins.less';

.mw-category {
	.column-count(3);
	.column-width(24em);

	.mw-category-group {
		ul {
			margin-top: 0;
			margin-bottom: 0;
		}

		li {
			.column-break-inside-avoid;
		}
	}
}

/* stylelint-disable-next-line selector-class-pattern */
.redirect-in-category {
	font-style: italic;
}
