.mw-inputbox-centered {
	margin-left: auto;
	margin-right: auto;
	text-align: center;
}

.mw-inputbox-inline {
	display: inline;
}

.mw-inputbox-element {
	/* important is needed to overwrite display:table from mw-ui checkbox */
	display: inline-table !important;
	white-space: nowrap;
}

.mw-inputbox-input {
	margin-bottom: 0.5em;
}
