/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-userAdd {
  background-image: url('themes/wikimediaui/images/icons/userAdd-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAdd-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAdd {
  background-image: url('themes/wikimediaui/images/icons/userAdd-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAdd-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAdd {
  background-image: url('themes/wikimediaui/images/icons/userAdd-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAdd-ltr-progressive.svg');
}
.oo-ui-icon-userAnonymous {
  background-image: url('themes/wikimediaui/images/icons/userAnonymous.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAnonymous.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAnonymous {
  background-image: url('themes/wikimediaui/images/icons/userAnonymous-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAnonymous-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAnonymous {
  background-image: url('themes/wikimediaui/images/icons/userAnonymous-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAnonymous-progressive.svg');
}
.oo-ui-icon-userAvatar {
  background-image: url('themes/wikimediaui/images/icons/userAvatar.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatar.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAvatar {
  background-image: url('themes/wikimediaui/images/icons/userAvatar-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatar-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAvatar {
  background-image: url('themes/wikimediaui/images/icons/userAvatar-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatar-progressive.svg');
}
.oo-ui-icon-userAvatarOutline {
  background-image: url('themes/wikimediaui/images/icons/userAvatarOutline.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatarOutline.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAvatarOutline {
  background-image: url('themes/wikimediaui/images/icons/userAvatarOutline-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatarOutline-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAvatarOutline {
  background-image: url('themes/wikimediaui/images/icons/userAvatarOutline-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatarOutline-progressive.svg');
}
.oo-ui-icon-userContributions {
  background-image: url('themes/wikimediaui/images/icons/userContributions-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userContributions-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-userContributions {
  background-image: url('themes/wikimediaui/images/icons/userContributions-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userContributions-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userContributions {
  background-image: url('themes/wikimediaui/images/icons/userContributions-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userContributions-ltr-progressive.svg');
}
.oo-ui-icon-userGroup {
  background-image: url('themes/wikimediaui/images/icons/userGroup-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userGroup-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-userGroup {
  background-image: url('themes/wikimediaui/images/icons/userGroup-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userGroup-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userGroup {
  background-image: url('themes/wikimediaui/images/icons/userGroup-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userGroup-ltr-progressive.svg');
}
.oo-ui-icon-userTalk {
  background-image: url('themes/wikimediaui/images/icons/userTalk-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userTalk-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-userTalk {
  background-image: url('themes/wikimediaui/images/icons/userTalk-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userTalk-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userTalk {
  background-image: url('themes/wikimediaui/images/icons/userTalk-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userTalk-ltr-progressive.svg');
}
