/*!
 * VisualEditor MediaWiki UserInterface MWParameterPage styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-ui-mwParameterPage .ve-ui-mwParameter-wikitextFallbackInput {
	/* Support: Blink, Gecko, Webkit */
	/* Specify a valid second value to fix size, see T176636 */
	font-family: monospace, monospace;
	background-color: #c8ccd1;
}
