/*!
 * VisualEditor FormatTool styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-paragraph .oo-ui-tool-title {
	font-weight: normal;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-heading1 .oo-ui-tool-title {
	font-size: 190%;
	font-weight: normal;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-heading2 .oo-ui-tool-title {
	font-size: 150%;
	font-weight: normal;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-heading3 .oo-ui-tool-title {
	font-size: 130%;
	font-weight: bold;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-heading4 .oo-ui-tool-title {
	font-size: 115%;
	font-weight: bold;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-heading5 .oo-ui-tool-title {
	font-size: 100%;
	font-weight: bold;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-heading6 .oo-ui-tool-title {
	font-size: 80%;
	font-weight: bold;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-preformatted .oo-ui-tool-title {
	/* Support: Blink, Gecko, Webkit */
	/* Specify a valid second value to fix size, see T176636 */
	font-family: monospace, monospace;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-blockquote .oo-ui-tool-title {
	/* Indent in addition to padding */
	text-indent: 1.5em;
}

.oo-ui-menuToolGroup-tools .oo-ui-tool-name-tableCellHeader .oo-ui-tool-title {
	font-weight: bold;
}
