/*!
 * VisualEditor UserInterface FindAndReplaceDialog styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-findAndReplaceDialog-row {
	display: table;
	min-width: 30em;
	padding-bottom: 0.3em;
}

.ve-ui-findAndReplaceDialog-row:last-child {
	/* Extra pixel for button shadows */
	padding-bottom: 1px;
}

.ve-ui-findAndReplaceDialog-cell {
	display: table-cell;
	vertical-align: middle;
	white-space: nowrap;
	padding-right: 1em;
}

.ve-ui-findAndReplaceDialog-cell:last-child {
	padding-right: 0;
}

.ve-ui-findAndReplaceDialog-cell-input {
	width: 100%;
}

.ve-ui-findAndReplaceDialog-cell-input .oo-ui-textInputWidget {
	max-width: none;
}

/* @noflip */
.ve-ui-findAndReplaceDialog-findResults {
	position: absolute;
	top: 0;
	left: 0;
	pointer-events: none;
}

.ve-ui-findAndReplaceDialog-findResult {
	opacity: 0.2;
}

.ve-ui-findAndReplaceDialog-findResult > div {
	background: #28bb0b;
	position: absolute;
	margin-top: -0.15em;
	padding: 0.15em 0;
	border-radius: 2px;
}

.ve-ui-findAndReplaceDialog-findResult-focused {
	opacity: 0.4;
}

.ve-ui-findAndReplaceDialog-findResult-focused > div {
	background: #1f850b;
}

/* Support: IE, Edge */
/* Set opacity directly on the highlights, rather than their containers,
   otherwise the opacity isn't applied at all */
.ve-init-target-ie-or-edge .ve-ui-findAndReplaceDialog-findResult > div {
	opacity: 0.2;
}

.ve-init-target-ie-or-edge .ve-ui-findAndReplaceDialog-findResult-focused > div {
	opacity: 0.4;
}
