<!DOCTYPE html>
<html>
	<!--
	/*!
	 * VisualEditor rebase demo
	 *
	 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
	 */
	-->
	<head>
		<meta charset="utf-8">
		<meta name="viewport" content="initial-scale=1.0, user-scalable=no" />
		<title>VisualEditor Rebase Demo</title>

		<!-- oojs-ui-apex -->
		<link rel=stylesheet href="/lib/oojs-ui/oojs-ui-apex.css" class="stylesheet-ltr">
		<link rel=stylesheet href="/lib/oojs-ui/oojs-ui-apex-icons-editing-core.css" class="stylesheet-ltr">
		<link rel=stylesheet href="/lib/oojs-ui/oojs-ui-apex-icons-editing-advanced.css" class="stylesheet-ltr">
		<link rel=stylesheet href="/lib/oojs-ui/oojs-ui-apex-icons-editing-styling.css" class="stylesheet-ltr">
		<link rel=stylesheet href="/lib/oojs-ui/oojs-ui-apex-icons-editing-list.css" class="stylesheet-ltr">

		<!-- visualEditor.standalone.demo -->
		<link rel=stylesheet href="/demos/ve/demo.css" class="stylesheet-read">

		<!-- visualEditor.desktop.standalone.demo -->
		<link rel=stylesheet href="/demos/ve/demo.desktop.css" class="stylesheet-read">

		<!-- visualEditor.desktop.standalone.apex.demo -->
		<link rel=stylesheet href="/demos/ve/demo.apex.css" class="stylesheet-read">

		<link rel=stylesheet href="/rebaser/index.css" class="stylesheet-read">
	</head>
	<body>
		<div class="ve-pad-logo"></div>
		<div style="clear: both;"></div>
		<div class="ve-demo-index"></div>

		<!-- jquery -->
		<script src="/lib/jquery/jquery.js"></script>

		<!-- oojs -->
		<script src="/lib/oojs/oojs.jquery.js"></script>

		<!-- oojs-ui -->
		<script src="/lib/oojs-ui/oojs-ui-core.js"></script>
		<script src="/lib/oojs-ui/oojs-ui-widgets.js"></script>

		<!-- jquery.i18n.dist -->
		<script src="/dist/lib/jquery.i18n.js"></script>

		<!-- jquery.uls.data.dist -->
		<script src="/dist/lib/jquery.uls.data.js"></script>

		<!-- visualEditor.supportCheck -->
		<script src="/src/init/ve.init.SupportCheck.js"></script>

		<!-- oojs-ui-apex -->
		<script src="/lib/oojs-ui/oojs-ui-apex.js"></script>

		<!-- ve.init.sa.platform -->
		<script src="/src/ve.js"></script>
		<script src="/src/ve.utils.js"></script>
		<script src="/src/init/ve.init.js"></script>
		<script src="/src/init/sa/ve.init.sa.js"></script>
		<script src="/src/init/ve.init.SafeStorage.js"></script>
		<script src="/src/init/ve.init.ListStorage.js"></script>
		<script src="/src/init/ve.init.Platform.js"></script>
		<script src="/src/init/sa/ve.init.sa.SafeStorage.js"></script>
		<script src="/src/init/sa/ve.init.sa.Platform.js"></script>

		<script>
			ve.messagePaths = [
				'/dist/i18n/',
				'/lib/oojs-ui/i18n/'
			];
		</script>

		<script src="rebaser/index.js"></script>

	</body>
</html>
