@import 'mediawiki.mixins.less';

#mw-indicator-mw-helplink a {
	.background-image-svg('images/helpNotice.svg', 'images/helpNotice.png');
	background-position: left center;
	background-repeat: no-repeat;
	display: inline-block;
	height: 20px;
	padding-left: 25px;
	line-height: 20px;
}
