<?php
/** արեւմտահայերէն (Western Armenian)
 *
 * To improve a translation please visit https://translatewiki.net
 *
 * @ingroup Language
 * @file
 *
 */

$fallback = 'hy';

$magicWords['redirect'] = [ '0', '#ՎԵՐԱՅՂՈՒՄ', '#ՎԵՐԱՀՂՈՒՄ', '#REDIRECT' ];

$namespaceNames[NS_CATEGORY] = 'Ստորոգութիւն';
