/*!
 * VisualEditor ContentEditable CommentAnnotation styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-commentAnnotation {
	box-shadow: 0 0 0 1px rgba( 255, 221, 102, 0.5 ); /* #fd6 */
	border-radius: 2px;
	padding: 2px;
	margin: -2px;
	background: rgba( 255, 221, 102, 0.5 );
}

.ve-ce-commentAnnotation.ve-ce-annotation-active {
	box-shadow: 0 0 0 1px #fd6;
	background: #fd6;
}
