/*!
 * VisualEditor ContentEditable LanguageAnnotation styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-languageAnnotation {
	border-bottom: 1px dashed #ccc;
	background-color: #ebf3f5;
}

/* Support: IE */
/* Don't apply a background color to language annotations, otherwise the
   text disappears if the span has dir="rtl" */
.ve-init-target-ie .ve-ce-languageAnnotation {
	background: none;
}
