/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-bigger {
  background-image: url('themes/wikimediaui/images/icons/bigger.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bigger.svg');
}
.oo-ui-image-invert.oo-ui-icon-bigger {
  background-image: url('themes/wikimediaui/images/icons/bigger-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bigger-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bigger {
  background-image: url('themes/wikimediaui/images/icons/bigger-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bigger-progressive.svg');
}
.oo-ui-icon-smaller {
  background-image: url('themes/wikimediaui/images/icons/smaller.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smaller.svg');
}
.oo-ui-image-invert.oo-ui-icon-smaller {
  background-image: url('themes/wikimediaui/images/icons/smaller-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smaller-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-smaller {
  background-image: url('themes/wikimediaui/images/icons/smaller-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smaller-progressive.svg');
}
.oo-ui-icon-subscript {
  background-image: url('themes/wikimediaui/images/icons/subscript-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subscript-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-subscript {
  background-image: url('themes/wikimediaui/images/icons/subscript-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subscript-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-subscript {
  background-image: url('themes/wikimediaui/images/icons/subscript-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subscript-ltr-progressive.svg');
}
.oo-ui-icon-superscript {
  background-image: url('themes/wikimediaui/images/icons/superscript-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/superscript-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-superscript {
  background-image: url('themes/wikimediaui/images/icons/superscript-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/superscript-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-superscript {
  background-image: url('themes/wikimediaui/images/icons/superscript-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/superscript-ltr-progressive.svg');
}
.oo-ui-icon-bold {
  background-image: url('themes/wikimediaui/images/icons/bold-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-a.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-ain.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-ain.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-te.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-te.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-palochka.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-palochka.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/bold-n.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/bold-n.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/bold-n.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/bold-l.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/bold-l.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-dad.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-dad.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/bold-g.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/bold-g.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/bold-armn-to.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-armn-to.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/bold-geor-man.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-geor-man.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/bold-v.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-v.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-be.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-be.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-jeem.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-jeem.svg');
}
.oo-ui-image-invert.oo-ui-icon-bold {
  background-image: url('themes/wikimediaui/images/icons/bold-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-ain-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-ain-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-te-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-te-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-palochka-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-palochka-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-dad-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-dad-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/bold-armn-to-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-armn-to-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/bold-geor-man-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-geor-man-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/bold-v-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-v-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-be-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-be-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-jeem-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-jeem-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bold {
  background-image: url('themes/wikimediaui/images/icons/bold-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-ain-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-ain-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-te-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-te-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-palochka-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-palochka-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-dad-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-dad-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/bold-armn-to-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-armn-to-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/bold-geor-man-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-geor-man-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/bold-v-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-v-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-be-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-be-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-jeem-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-jeem-progressive.svg');
}
.oo-ui-icon-highlight {
  background-image: url('themes/wikimediaui/images/icons/highlight.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/highlight.svg');
}
.oo-ui-image-invert.oo-ui-icon-highlight {
  background-image: url('themes/wikimediaui/images/icons/highlight-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/highlight-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-highlight {
  background-image: url('themes/wikimediaui/images/icons/highlight-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/highlight-progressive.svg');
}
.oo-ui-icon-italic {
  background-image: url('themes/wikimediaui/images/icons/italic-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-a.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-meem.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-meem.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/italic-e.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-e.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/italic-d.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-d.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/italic-armn-sha.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-armn-sha.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/italic-s.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-s.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/italic-geor-kan.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-geor-kan.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-teh.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-teh.svg');
}
.oo-ui-image-invert.oo-ui-icon-italic {
  background-image: url('themes/wikimediaui/images/icons/italic-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-meem-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-meem-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/italic-e-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-e-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/italic-d-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-d-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/italic-armn-sha-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-armn-sha-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/italic-s-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-s-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/italic-geor-kan-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-geor-kan-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-teh-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-teh-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-italic {
  background-image: url('themes/wikimediaui/images/icons/italic-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-meem-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-meem-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/italic-e-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-e-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/italic-d-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-d-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/italic-armn-sha-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-armn-sha-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/italic-s-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-s-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/italic-geor-kan-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-geor-kan-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-teh-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-teh-progressive.svg');
}
.oo-ui-icon-strikethrough {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-a.svg');
}
/* @noflip */
.oo-ui-icon-strikethrough:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-s.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-s.svg');
}
/* @noflip */
.oo-ui-icon-strikethrough:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-y.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-y.svg');
}
.oo-ui-image-invert.oo-ui-icon-strikethrough {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-strikethrough:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-s-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-s-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-strikethrough:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-y-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-y-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-strikethrough {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-strikethrough:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-s-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-s-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-strikethrough:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-y-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-y-progressive.svg');
}
.oo-ui-icon-underline {
  background-image: url('themes/wikimediaui/images/icons/underline-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-a.svg');
}
/* @noflip */
.oo-ui-icon-underline:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/underline-u.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u.svg');
}
/* @noflip */
.oo-ui-icon-underline:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/underline-u.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u.svg');
}
.oo-ui-image-invert.oo-ui-icon-underline {
  background-image: url('themes/wikimediaui/images/icons/underline-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-underline:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-underline:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-underline {
  background-image: url('themes/wikimediaui/images/icons/underline-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-underline:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-underline:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-progressive.svg');
}
.oo-ui-icon-textLanguage {
  background-image: url('themes/wikimediaui/images/icons/language.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language.svg');
}
.oo-ui-image-invert.oo-ui-icon-textLanguage {
  background-image: url('themes/wikimediaui/images/icons/language-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textLanguage {
  background-image: url('themes/wikimediaui/images/icons/language-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-progressive.svg');
}
.oo-ui-icon-textDirLTR {
  background-image: url('themes/wikimediaui/images/icons/textDirLTR.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirLTR.svg');
}
.oo-ui-image-invert.oo-ui-icon-textDirLTR {
  background-image: url('themes/wikimediaui/images/icons/textDirLTR-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirLTR-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textDirLTR {
  background-image: url('themes/wikimediaui/images/icons/textDirLTR-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirLTR-progressive.svg');
}
.oo-ui-icon-textDirRTL {
  background-image: url('themes/wikimediaui/images/icons/textDirRTL.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirRTL.svg');
}
.oo-ui-image-invert.oo-ui-icon-textDirRTL {
  background-image: url('themes/wikimediaui/images/icons/textDirRTL-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirRTL-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textDirRTL {
  background-image: url('themes/wikimediaui/images/icons/textDirRTL-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirRTL-progressive.svg');
}
.oo-ui-icon-textStyle {
  background-image: url('themes/wikimediaui/images/icons/textStyle.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textStyle.svg');
}
.oo-ui-image-invert.oo-ui-icon-textStyle {
  background-image: url('themes/wikimediaui/images/icons/textStyle-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textStyle-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textStyle {
  background-image: url('themes/wikimediaui/images/icons/textStyle-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textStyle-progressive.svg');
}
