/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-bookmarkOutline {
  background-image: url('themes/wikimediaui/images/icons/bookmarkOutline.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmarkOutline.svg');
}
.oo-ui-icon-bookmark {
  background-image: url('themes/wikimediaui/images/icons/bookmark.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmark.svg');
}
.oo-ui-icon-block {
  background-image: url('themes/wikimediaui/images/icons/block.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/block.svg');
}
.oo-ui-icon-unBlock {
  background-image: url('themes/wikimediaui/images/icons/unBlock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unBlock.svg');
}
.oo-ui-icon-flag {
  background-image: url('themes/wikimediaui/images/icons/flag-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/flag-ltr.svg');
}
.oo-ui-icon-unFlag {
  background-image: url('themes/wikimediaui/images/icons/unFlag-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unFlag-ltr.svg');
}
.oo-ui-icon-lock {
  background-image: url('themes/wikimediaui/images/icons/lock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lock.svg');
}
.oo-ui-icon-unLock {
  background-image: url('themes/wikimediaui/images/icons/unLock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLock.svg');
}
.oo-ui-icon-restore {
  background-image: url('themes/wikimediaui/images/icons/restore.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/restore.svg');
}
.oo-ui-icon-star {
  background-image: url('themes/wikimediaui/images/icons/star.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/star.svg');
}
.oo-ui-icon-halfStar {
  background-image: url('themes/wikimediaui/images/icons/halfStar-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfStar-ltr.svg');
}
.oo-ui-icon-unStar {
  background-image: url('themes/wikimediaui/images/icons/unStar.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unStar.svg');
}
.oo-ui-icon-trash {
  background-image: url('themes/wikimediaui/images/icons/trash.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/trash.svg');
}
.oo-ui-icon-pushPin {
  background-image: url('themes/wikimediaui/images/icons/pushPin.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pushPin.svg');
}
.oo-ui-icon-ongoingConversation {
  background-image: url('themes/wikimediaui/images/icons/ongoingConversation-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ongoingConversation-ltr.svg');
}
