/*!
 * VisualEditor ContentEditable HorizontalRuleNode styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-horizontalRuleNode {
	/* Prevent margin collapse of child <hr> so the node has a reasonable height to click on */
	overflow: auto;
}
