/*!
 * VisualEditor UserInterface DimensionsWidget styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-dimensionsWidget,
.ve-ui-dimensionsWidget > .oo-ui-textInputWidget {
	display: inline-block;
}

.ve-ui-dimensionsWidget .oo-ui-textInputWidget {
	width: 3.5em;
}

.ve-ui-dimensionsWidget .oo-ui-labelWidget {
	padding-right: 0.5em;
	line-height: 2.35em;
	vertical-align: middle;
}

.ve-ui-dimensionsWidget .oo-ui-labelWidget:last-child {
	padding-right: 0;
}
