/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-menu {
  background-image: url('themes/wikimediaui/images/icons/menu.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/menu.svg');
}
.oo-ui-image-invert.oo-ui-icon-menu {
  background-image: url('themes/wikimediaui/images/icons/menu-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/menu-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-menu {
  background-image: url('themes/wikimediaui/images/icons/menu-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/menu-progressive.svg');
}
.oo-ui-icon-recentChanges {
  background-image: url('themes/wikimediaui/images/icons/recentChanges-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/recentChanges-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-recentChanges {
  background-image: url('themes/wikimediaui/images/icons/recentChanges-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/recentChanges-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-recentChanges {
  background-image: url('themes/wikimediaui/images/icons/recentChanges-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/recentChanges-rtl-progressive.svg');
}
.oo-ui-icon-textFlow {
  background-image: url('themes/wikimediaui/images/icons/textFlow-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textFlow-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-textFlow {
  background-image: url('themes/wikimediaui/images/icons/textFlow-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textFlow-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textFlow {
  background-image: url('themes/wikimediaui/images/icons/textFlow-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textFlow-rtl-progressive.svg');
}
.oo-ui-icon-textSummary {
  background-image: url('themes/wikimediaui/images/icons/textSummary-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textSummary-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-textSummary {
  background-image: url('themes/wikimediaui/images/icons/textSummary-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textSummary-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textSummary {
  background-image: url('themes/wikimediaui/images/icons/textSummary-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textSummary-rtl-progressive.svg');
}
.oo-ui-icon-viewCompact {
  background-image: url('themes/wikimediaui/images/icons/viewCompact.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewCompact.svg');
}
.oo-ui-image-invert.oo-ui-icon-viewCompact {
  background-image: url('themes/wikimediaui/images/icons/viewCompact-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewCompact-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-viewCompact {
  background-image: url('themes/wikimediaui/images/icons/viewCompact-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewCompact-progressive.svg');
}
.oo-ui-icon-viewDetails {
  background-image: url('themes/wikimediaui/images/icons/viewDetails-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewDetails-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-viewDetails {
  background-image: url('themes/wikimediaui/images/icons/viewDetails-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewDetails-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-viewDetails {
  background-image: url('themes/wikimediaui/images/icons/viewDetails-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewDetails-rtl-progressive.svg');
}
