/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-bright {
  background-image: url('themes/wikimediaui/images/icons/bright.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bright.svg');
}
.oo-ui-image-invert.oo-ui-icon-bright {
  background-image: url('themes/wikimediaui/images/icons/bright-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bright-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bright {
  background-image: url('themes/wikimediaui/images/icons/bright-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bright-progressive.svg');
}
.oo-ui-icon-halfBright {
  background-image: url('themes/wikimediaui/images/icons/halfBright-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfBright-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-halfBright {
  background-image: url('themes/wikimediaui/images/icons/halfBright-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfBright-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-halfBright {
  background-image: url('themes/wikimediaui/images/icons/halfBright-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfBright-rtl-progressive.svg');
}
.oo-ui-icon-notBright {
  background-image: url('themes/wikimediaui/images/icons/notBright.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notBright.svg');
}
.oo-ui-image-invert.oo-ui-icon-notBright {
  background-image: url('themes/wikimediaui/images/icons/notBright-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notBright-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-notBright {
  background-image: url('themes/wikimediaui/images/icons/notBright-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notBright-progressive.svg');
}
.oo-ui-icon-eye {
  background-image: url('themes/wikimediaui/images/icons/eye.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eye.svg');
}
.oo-ui-image-invert.oo-ui-icon-eye {
  background-image: url('themes/wikimediaui/images/icons/eye-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eye-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-eye {
  background-image: url('themes/wikimediaui/images/icons/eye-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eye-progressive.svg');
}
.oo-ui-icon-eyeClosed {
  background-image: url('themes/wikimediaui/images/icons/eyeClosed.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eyeClosed.svg');
}
.oo-ui-image-invert.oo-ui-icon-eyeClosed {
  background-image: url('themes/wikimediaui/images/icons/eyeClosed-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eyeClosed-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-eyeClosed {
  background-image: url('themes/wikimediaui/images/icons/eyeClosed-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eyeClosed-progressive.svg');
}
.oo-ui-icon-moon {
  background-image: url('themes/wikimediaui/images/icons/moon.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moon.svg');
}
.oo-ui-image-invert.oo-ui-icon-moon {
  background-image: url('themes/wikimediaui/images/icons/moon-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moon-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-moon {
  background-image: url('themes/wikimediaui/images/icons/moon-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moon-progressive.svg');
}
.oo-ui-icon-largerText {
  background-image: url('themes/wikimediaui/images/icons/largerText.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/largerText.svg');
}
.oo-ui-image-invert.oo-ui-icon-largerText {
  background-image: url('themes/wikimediaui/images/icons/largerText-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/largerText-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-largerText {
  background-image: url('themes/wikimediaui/images/icons/largerText-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/largerText-progressive.svg');
}
.oo-ui-icon-smallerText {
  background-image: url('themes/wikimediaui/images/icons/smallerText.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smallerText.svg');
}
.oo-ui-image-invert.oo-ui-icon-smallerText {
  background-image: url('themes/wikimediaui/images/icons/smallerText-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smallerText-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-smallerText {
  background-image: url('themes/wikimediaui/images/icons/smallerText-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smallerText-progressive.svg');
}
.oo-ui-icon-visionSimulator {
  background-image: url('themes/wikimediaui/images/icons/visionSimulator.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/visionSimulator.svg');
}
.oo-ui-image-invert.oo-ui-icon-visionSimulator {
  background-image: url('themes/wikimediaui/images/icons/visionSimulator-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/visionSimulator-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-visionSimulator {
  background-image: url('themes/wikimediaui/images/icons/visionSimulator-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/visionSimulator-progressive.svg');
}
