.mw-feedbackDialog-welcome-message,
.mw-feedbackDialog-feedback-terms {
	line-height: 1.4;
}

.mw-feedbackDialog-feedback-terms p:first-child {
	margin-top: 0;
}

.mw-feedbackDialog-welcome-message {
	margin-bottom: 1em;
}

/* Overwriting OOUI */
.mw-feedbackDialog-feedback-form .oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
	min-width: 4.2em;
	width: 20%;
}

.mw-feedbackDialog-feedback-form .oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
	width: 80%;
}

.mw-feedbackDialog-feedback-termsofuse {
	margin-left: 2em;
}
