<?php
/** Chinese (中文)
 *
 * To improve a translation please visit https://translatewiki.net
 *
 * @ingroup Language
 * @file
 *
 * @author Kuailong
 * @author PhiLiP
 * @author Shizhao
 * @author Wong128hk
 */

# Stub message file for converter code "zh"

$fallback = 'zh-hans';

$namespaceNames = [
	NS_MEDIA            => 'Media',
	NS_SPECIAL          => 'Special',
	NS_TALK             => 'Talk',
	NS_USER             => 'User',
	NS_USER_TALK        => 'User_talk',
	NS_PROJECT_TALK     => '$1_talk',
	NS_FILE             => 'File',
	NS_FILE_TALK        => 'File_talk',
	NS_MEDIAWIKI        => 'MediaWiki',
	NS_MEDIAWIKI_TALK   => 'MediaWiki_talk',
	NS_TEMPLATE         => 'Template',
	NS_TEMPLATE_TALK    => 'Template_talk',
	NS_HELP             => 'Help',
	NS_HELP_TALK        => 'Help_talk',
	NS_CATEGORY         => 'Category',
	NS_CATEGORY_TALK    => 'Category_talk',
];

$namespaceAliases = [
	'媒体'	=> NS_MEDIA,
	'媒體'	=> NS_MEDIA,
	'特殊'  => NS_SPECIAL,
	'对话'  => NS_TALK,
	'對話'  => NS_TALK,
	'讨论'	=> NS_TALK,
	'討論'	=> NS_TALK,
	'用户'  => NS_USER,
	'用戶'  => NS_USER,
	'用户对话' => NS_USER_TALK,
	'用戶對話' => NS_USER_TALK,
	'用户讨论' => NS_USER_TALK,
	'用戶討論' => NS_USER_TALK,
	# This has never worked so it's unlikely to annoy anyone if I disable it -- TS
	# '{{SITENAME}}_对话' => NS_PROJECT_TALK
	# "{{SITENAME}}_對話" => NS_PROJECT_TALK
	'图像' => NS_FILE,
	'圖像' => NS_FILE,
	'档案' => NS_FILE,
	'檔案' => NS_FILE,
	'文件' => NS_FILE,
	'图像对话' => NS_FILE_TALK,
	'圖像對話' => NS_FILE_TALK,
	'图像讨论' => NS_FILE_TALK,
	'圖像討論' => NS_FILE_TALK,
	'档案对话' => NS_FILE_TALK,
	'檔案對話' => NS_FILE_TALK,
	'档案讨论' => NS_FILE_TALK,
	'檔案討論' => NS_FILE_TALK,
	'文件对话' => NS_FILE_TALK,
	'文件對話' => NS_FILE_TALK,
	'文件讨论' => NS_FILE_TALK,
	'文件討論' => NS_FILE_TALK,
	'模板'	=> NS_TEMPLATE,
	'样板'  => NS_TEMPLATE,
	'樣板'  => NS_TEMPLATE,
	'模板对话' => NS_TEMPLATE_TALK,
	'模板對話' => NS_TEMPLATE_TALK,
	'模板讨论' => NS_TEMPLATE_TALK,
	'模板討論' => NS_TEMPLATE_TALK,
	'样板对话' => NS_TEMPLATE_TALK,
	'樣板對話' => NS_TEMPLATE_TALK,
	'样板讨论' => NS_TEMPLATE_TALK,
	'樣板討論' => NS_TEMPLATE_TALK,
	'帮助'	=> NS_HELP,
	'幫助'  => NS_HELP,
	'帮助对话' => NS_HELP_TALK,
	'幫助對話' => NS_HELP_TALK,
	'帮助讨论' => NS_HELP_TALK,
	'幫助討論' => NS_HELP_TALK,
	'分类'	=> NS_CATEGORY,
	'分類'  => NS_CATEGORY,
	'分类对话' => NS_CATEGORY_TALK,
	'分類對話' => NS_CATEGORY_TALK,
	'分类讨论' => NS_CATEGORY_TALK,
	'分類討論' => NS_CATEGORY_TALK,
];

$specialPageAliases = [
	'Allmessages'               => [ '所有消息', '所有訊息', '所有讯息' ],
	'Allpages'                  => [ '所有页面', '所有頁面' ],
	'Ancientpages'              => [ '最早页面', '最早頁面' ],
	'Badtitle'                  => [ '错误标题', '无效标题', '錯誤標題' ],
	'Blankpage'                 => [ '空白页面', '空白頁面' ],
	'Booksources'               => [ '网络书源', '網絡書源', '網路書源' ],
	'BrokenRedirects'           => [ '受损重定向' ],
	'Categories'                => [ '页面分类', '頁面分類' ],
	'ChangePassword'            => [ '修改密码', '修改密碼' ],
	'Confirmemail'              => [ '确认电子邮件', '確認電子郵件' ],
	'Contributions'             => [ '用户贡献', '用戶貢獻', '使用者貢獻', '使用者贡献' ],
	'CreateAccount'             => [ '创建账户', '創建賬戶', '建立帳戶', '建立帳號', '建立帐号' ],
	'Deadendpages'              => [ '断链页面', '斷鏈頁面', '斷連頁面' ],
	'DeletedContributions'      => [ '删除的贡献' ],
	'Diff'                      => [ '差异', '差異' ],
	'DoubleRedirects'           => [ '双重重定向', '雙重重新導向' ],
	'Emailuser'                 => [ '电邮联系', '電郵聯絡' ],
	'Export'                    => [ '导出页面', '導出頁面', '匯出頁面', '汇出页面' ],
	'Fewestrevisions'           => [ '最少修订页面', '最少修訂頁面' ],
	'Filepath'                  => [ '文件路径', '文件路徑', '檔案路徑', '档案路径' ],
	'Import'                    => [ '导入页面', '導入頁面', '匯入頁面', '汇入页面' ],
	'Invalidateemail'           => [ '不可识别的电邮地址', '不可識別的電郵位址' ],
	'BlockList'                 => [ '封禁列表' ],
	'LinkSearch'                => [ '链接搜索', '鏈接搜索', '連結搜尋', '连结搜寻' ],
	'Listadmins'                => [ '管理员列表', '管理員列表' ],
	'Listbots'                  => [ '机器人列表', '機器人清單', '機械人清單' ],
	'Listfiles'                 => [ '文件列表', '檔案列表', '档案列表' ],
	'Listgrouprights'           => [ '群组权限', '群組權限' ],
	'Listredirects'             => [ '重定向页列表', '重定向列表', '重新導向清單' ],
	'Listusers'                 => [ '用户列表', '用戶列表', '使用者清單' ],
	'Log'                       => [ '日志', '日誌' ],
	'Lonelypages'               => [ '孤立页面', '孤立頁面' ],
	'Longpages'                 => [ '长页面', '長頁面' ],
	'MergeHistory'              => [ '合并历史', '合併歷史' ],
	'MIMEsearch'                => [ 'MIME搜索', 'MIME搜尋', 'MIME搜寻' ],
	'Mostcategories'            => [ '最多分类页面', '最多分類頁面' ],
	'Mostimages'                => [ '最多链接文件', '最多鏈接文件', '最多連結檔案' ],
	'Mostlinked'                => [ '最多链接页面', '最多連結頁面' ],
	'Mostlinkedcategories'      => [ '最多链接分类', '最多連結分類' ],
	'Mostlinkedtemplates'       => [ '最多链接模板', '最多連結模板' ],
	'Mostrevisions'             => [ '最多修订页面', '最多修訂頁面' ],
	'Movepage'                  => [ '移动页面', '移動頁面' ],
	'Mycontributions'           => [ '我的贡献', '我的貢獻' ],
	'MyLanguage'                => [ '我的语言', '我的語言' ],
	'Mypage'                    => [ '我的用户页', '我的用戶頁', '我的使用者頁面' ],
	'Mytalk'                    => [ '我的讨论页', '我的討論頁' ],
	'Newimages'                 => [ '新建文件', '新建檔案' ],
	'Newpages'                  => [ '最新页面', '最新頁面' ],
	'PasswordReset'             => [ '重置密码', '重設密碼' ],
	'Preferences'               => [ '参数设置', '偏好設定', '參數設置' ],
	'Prefixindex'               => [ '前缀索引', '前綴索引', '字首索引' ],
	'Protectedpages'            => [ '已保护页面', '已保護頁面' ],
	'Protectedtitles'           => [ '已保护标题', '已保護標題' ],
	'Randompage'                => [ '随机页面', '隨機頁面' ],
	'Recentchanges'             => [ '最近更改', '近期變動' ],
	'Recentchangeslinked'       => [ '链出更改', '鏈出更改', '連出更改' ],
	'Redirect'                  => [ '重定向', '重新導向' ],
	'ResetTokens'               => [ '重置密钥', '重設金鑰' ],
	'Revisiondelete'            => [ '版本删除' ],
	'Search'                    => [ '搜索', '搜尋' ],
	'Shortpages'                => [ '短页面', '短頁面' ],
	'Specialpages'              => [ '特殊页面', '特殊頁面' ],
	'Statistics'                => [ '统计', '统计信息', '統計', '統計資訊' ],
	'Tags'                      => [ '标签', '標籤' ],
	'TrackingCategories'        => [ '追踪分类' ],
	'Uncategorizedcategories'   => [ '未归类分类', '未歸類分類' ],
	'Uncategorizedimages'       => [ '未归类文件', '未歸類文件', '未歸類檔案' ],
	'Uncategorizedpages'        => [ '未归类页面', '未歸類頁面' ],
	'Uncategorizedtemplates'    => [ '未归类模板', '未歸類模板' ],
	'Undelete'                  => [ '恢复被删页面', '恢復被刪頁面' ],
	'Unusedcategories'          => [ '未使用分类', '未使用分類' ],
	'Unusedimages'              => [ '未使用文件', '未使用檔案' ],
	'Unusedtemplates'           => [ '未使用模板' ],
	'Upload'                    => [ '上传文件', '上傳檔案', '上載檔案', '上载档案' ],
	'Userlogin'                 => [ '用户登录', '用戶登入', '使用者登入' ],
	'Userlogout'                => [ '用户退出', '用戶登出', '使用者登出' ],
	'Userrights'                => [ '用户权限', '用戶權限', '使用者權限' ],
	'Version'                   => [ '版本' ],
	'Wantedcategories'          => [ '需要的分类' ],
	'Wantedfiles'               => [ '需要的文件', '需要的檔案' ],
	'Wantedpages'               => [ '需要的页面' ],
	'Wantedtemplates'           => [ '需要的模板' ],
	'Watchlist'                 => [ '监视列表', '監視列表', '監視清单' ],
	'Whatlinkshere'             => [ '链入页面', '鏈入頁面', '連入頁面' ],
];
