/*!
 * VisualEditor MediaWiki Initialization DesktopArticleTarget styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

/* Toolbar */

.ve-init-mw-desktopArticleTarget-toolbar {
	overflow: hidden;
	transition: height 250ms ease;
}

.ve-init-mw-desktopArticleTarget-toolbar > .oo-ui-toolbar-bar {
	transform: translateY( -100% );
	transition: transform 250ms ease;
}

.ve-init-mw-desktopArticleTarget-toolbar-opened {
	overflow: visible;
}

.ve-init-mw-desktopArticleTarget-toolbar-open > .oo-ui-toolbar-bar {
	transform: translateY( 0 );
	opacity: 1;
}

.ve-init-mw-desktopArticleTarget-toolbar-opened > .oo-ui-toolbar-bar {
	/* Turn off transforms when done to avoid Safari rendering bugs (T202935) */
	transform: none;
}

.ve-tempSourceEditing .ve-init-mw-desktopArticleTarget-toolbar,
.ve-tempSourceEditing .ve-init-mw-desktopArticleTarget-toolbar > .oo-ui-toolbar-bar {
	transition: none;
}

.ve-tempSourceEditing .ve-init-mw-desktopArticleTarget-toolbar {
	margin-bottom: -3.125em;
	transition: opacity 2s ease;
}

/*!
 * ve-redirect-header is added just outside of the surface, which has its own
 * hiding style during activation.
 */
.ve-activating .ve-redirect-header {
	height: 0;
	overflow: hidden;
}

.ve-init-mw-desktopArticleTarget .ve-redirect-header {
	cursor: pointer;
}

/* stylelint-disable-next-line selector-class-pattern */
.ve-init-mw-desktopArticleTarget .redirectMsg a:hover {
	text-decoration: none;
}

/*!
 * Increase z-index of surface container so it appears above the debug bar
 */
.ve-init-mw-desktopArticleTarget #bodyContent { /* stylelint-disable-line selector-max-id */
	z-index: 1;
}

.ve-ui-init-desktopArticleTarget-sectionTitle {
	max-width: none;
	margin: 1em 0 0.5em 0;
}

/* stylelint-disable declaration-no-important */
.ve-ui-init-desktopArticleTarget-sectionTitle .oo-ui-inputWidget-input {
	padding: 0 !important;
	border: 0 !important;
	outline: 0 !important;
	box-shadow: none !important;
	line-height: inherit;
}
/* stylelint-enable declaration-no-important */
