/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-camera {
  background-image: url('themes/wikimediaui/images/icons/camera.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/camera.svg');
}
.oo-ui-image-invert.oo-ui-icon-camera {
  background-image: url('themes/wikimediaui/images/icons/camera-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/camera-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-camera {
  background-image: url('themes/wikimediaui/images/icons/camera-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/camera-progressive.svg');
}
.oo-ui-icon-chart {
  background-image: url('themes/wikimediaui/images/icons/chart.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/chart.svg');
}
.oo-ui-image-invert.oo-ui-icon-chart {
  background-image: url('themes/wikimediaui/images/icons/chart-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/chart-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-chart {
  background-image: url('themes/wikimediaui/images/icons/chart-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/chart-progressive.svg');
}
.oo-ui-icon-exitFullscreen {
  background-image: url('themes/wikimediaui/images/icons/exitFullscreen.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/exitFullscreen.svg');
}
.oo-ui-image-invert.oo-ui-icon-exitFullscreen {
  background-image: url('themes/wikimediaui/images/icons/exitFullscreen-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/exitFullscreen-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-exitFullscreen {
  background-image: url('themes/wikimediaui/images/icons/exitFullscreen-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/exitFullscreen-progressive.svg');
}
.oo-ui-icon-fullScreen {
  background-image: url('themes/wikimediaui/images/icons/fullScreen.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/fullScreen.svg');
}
.oo-ui-image-invert.oo-ui-icon-fullScreen {
  background-image: url('themes/wikimediaui/images/icons/fullScreen-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/fullScreen-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-fullScreen {
  background-image: url('themes/wikimediaui/images/icons/fullScreen-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/fullScreen-progressive.svg');
}
.oo-ui-icon-image {
  background-image: url('themes/wikimediaui/images/icons/image.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/image.svg');
}
.oo-ui-image-invert.oo-ui-icon-image {
  background-image: url('themes/wikimediaui/images/icons/image-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/image-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-image {
  background-image: url('themes/wikimediaui/images/icons/image-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/image-progressive.svg');
}
.oo-ui-icon-imageAdd {
  background-image: url('themes/wikimediaui/images/icons/imageAdd-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageAdd-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageAdd {
  background-image: url('themes/wikimediaui/images/icons/imageAdd-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageAdd-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageAdd {
  background-image: url('themes/wikimediaui/images/icons/imageAdd-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageAdd-rtl-progressive.svg');
}
.oo-ui-icon-imageBroken {
  background-image: url('themes/wikimediaui/images/icons/imageBroken.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageBroken.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageBroken {
  background-image: url('themes/wikimediaui/images/icons/imageBroken-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageBroken-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageBroken {
  background-image: url('themes/wikimediaui/images/icons/imageBroken-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageBroken-progressive.svg');
}
.oo-ui-icon-imageLock {
  background-image: url('themes/wikimediaui/images/icons/imageLock-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLock-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageLock {
  background-image: url('themes/wikimediaui/images/icons/imageLock-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLock-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageLock {
  background-image: url('themes/wikimediaui/images/icons/imageLock-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLock-rtl-progressive.svg');
}
.oo-ui-icon-imageGallery {
  background-image: url('themes/wikimediaui/images/icons/imageGallery.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageGallery.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageGallery {
  background-image: url('themes/wikimediaui/images/icons/imageGallery-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageGallery-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageGallery {
  background-image: url('themes/wikimediaui/images/icons/imageGallery-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageGallery-progressive.svg');
}
.oo-ui-icon-musicalScore {
  background-image: url('themes/wikimediaui/images/icons/musicalScore.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/musicalScore.svg');
}
.oo-ui-image-invert.oo-ui-icon-musicalScore {
  background-image: url('themes/wikimediaui/images/icons/musicalScore-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/musicalScore-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-musicalScore {
  background-image: url('themes/wikimediaui/images/icons/musicalScore-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/musicalScore-progressive.svg');
}
.oo-ui-icon-pause {
  background-image: url('themes/wikimediaui/images/icons/pause.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pause.svg');
}
.oo-ui-image-invert.oo-ui-icon-pause {
  background-image: url('themes/wikimediaui/images/icons/pause-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pause-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-pause {
  background-image: url('themes/wikimediaui/images/icons/pause-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pause-progressive.svg');
}
.oo-ui-icon-play {
  background-image: url('themes/wikimediaui/images/icons/play.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/play.svg');
}
.oo-ui-image-invert.oo-ui-icon-play {
  background-image: url('themes/wikimediaui/images/icons/play-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/play-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-play {
  background-image: url('themes/wikimediaui/images/icons/play-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/play-progressive.svg');
}
.oo-ui-icon-stop {
  background-image: url('themes/wikimediaui/images/icons/stop.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stop.svg');
}
.oo-ui-image-invert.oo-ui-icon-stop {
  background-image: url('themes/wikimediaui/images/icons/stop-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stop-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-stop {
  background-image: url('themes/wikimediaui/images/icons/stop-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stop-progressive.svg');
}
.oo-ui-icon-zoomIn {
  background-image: url('themes/wikimediaui/images/icons/zoomIn.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomIn.svg');
}
.oo-ui-image-invert.oo-ui-icon-zoomIn {
  background-image: url('themes/wikimediaui/images/icons/zoomIn-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomIn-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-zoomIn {
  background-image: url('themes/wikimediaui/images/icons/zoomIn-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomIn-progressive.svg');
}
.oo-ui-icon-zoomOut {
  background-image: url('themes/wikimediaui/images/icons/zoomOut.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomOut.svg');
}
.oo-ui-image-invert.oo-ui-icon-zoomOut {
  background-image: url('themes/wikimediaui/images/icons/zoomOut-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomOut-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-zoomOut {
  background-image: url('themes/wikimediaui/images/icons/zoomOut-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomOut-progressive.svg');
}
