/*!
 * VisualEditor ContentEditable GeneratedContentNode styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-generatedContentNode-generating {
	opacity: 0.5;
}
