/*!
 * VisualEditor UserInterface Table Context styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-tableLineContext {
	/* Ensure it is placed above the table context */
	z-index: 3;
}

.ve-ui-tableLineContext > .oo-ui-iconWidget.oo-ui-iconElement.oo-ui-iconElement-icon {
	position: absolute;
	display: block;
	background-color: #d3e5fd;
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;
	cursor: pointer;
	margin: 0;
}

.ve-ui-tableLineContext-col > .oo-ui-iconWidget.oo-ui-iconElement.oo-ui-iconElement-icon {
	width: 100%;
	height: 1em;
	min-height: auto;
	/* Context height + 0.2em gap */
	top: -1.2em;
	border-bottom: 0;
}

.ve-ui-tableLineContext-row > .oo-ui-iconWidget.oo-ui-iconElement.oo-ui-iconElement-icon {
	width: 1em;
	min-width: auto;
	/* Context width + 0.2em gap */
	left: -1.2em;
	border-right: 0;
}

.ve-ui-tableLineContext-table > .oo-ui-iconWidget.oo-ui-iconElement.oo-ui-iconElement-icon {
	height: 1em;
	width: 1em;
	min-width: auto;
	min-height: auto;
	/* Context width/height + 0.2em gap */
	top: -1.2em;
	left: -1.2em;
	border-bottom: 0;
	border-right: 0;
}

.ve-ce-tableNodeOverlay-deactivated .ve-ui-tableLineContext > .oo-ui-iconWidget.oo-ui-iconElement.oo-ui-iconElement-icon {
	background-color: #ddd;
}

.ve-ui-tableLineContext-menu {
	z-index: 2;
}
