/* All of the combinations should result in the same output in LTR and RTL mode. */

.selector { /*@embed*/ background-image: url(simple-ltr.gif); }

.selector { /*@embed*/ background-image: url(simple-ltr.gif); }

.selector { /*@embed*/ background-image: url(simple-ltr.gif); }

.selector { /*@embed*/ background-image: url(simple-ltr.gif); }

.selector { /*@embed*/ background-image: url(simple-ltr.gif); }
