/* All of the combinations should result in the same output in LTR and RTL mode. */

/*@noflip*/ .selector { /*@embed*/ background-image: url(simple-ltr.gif); }

/*@noflip*/ .selector { background-image: /*@embed*/ url(simple-ltr.gif); }

.selector { /*@noflip*/ /*@embed*/ background-image: url(simple-ltr.gif); }

.selector { /*@embed*/ /*@noflip*/ background-image: url(simple-ltr.gif); }

.selector { /*@noflip*/ background-image: /*@embed*/ url(simple-ltr.gif); }
