/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-article {
  background-image: url('themes/wikimediaui/images/icons/article-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/article-ltr.svg');
}
.oo-ui-icon-articles {
  background-image: url('themes/wikimediaui/images/icons/articles-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articles-ltr.svg');
}
.oo-ui-icon-articleAdd {
  background-image: url('themes/wikimediaui/images/icons/articleAdd.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleAdd.svg');
}
.oo-ui-icon-articleCheck {
  background-image: url('themes/wikimediaui/images/icons/articleCheck.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleCheck.svg');
}
.oo-ui-icon-articleDisambiguation {
  background-image: url('themes/wikimediaui/images/icons/articleDisambiguation-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleDisambiguation-ltr.svg');
}
.oo-ui-icon-articleNotFound {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr.svg');
}
/* @noflip */
.oo-ui-icon-articleNotFound:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr.svg');
}
/* @noflip */
.oo-ui-icon-articleNotFound:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr.svg');
}
.oo-ui-icon-articleSearch {
  background-image: url('themes/wikimediaui/images/icons/articleSearch.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleSearch.svg');
}
.oo-ui-icon-articlesSearch {
  background-image: url('themes/wikimediaui/images/icons/articlesSearch-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articlesSearch-ltr.svg');
}
.oo-ui-icon-articleRedirect {
  background-image: url('themes/wikimediaui/images/icons/articleRedirect-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleRedirect-ltr.svg');
}
.oo-ui-icon-die {
  background-image: url('themes/wikimediaui/images/icons/die.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/die.svg');
}
.oo-ui-icon-download {
  background-image: url('themes/wikimediaui/images/icons/download.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/download.svg');
}
.oo-ui-icon-folderPlaceholder {
  background-image: url('themes/wikimediaui/images/icons/folderPlaceholder-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/folderPlaceholder-ltr.svg');
}
.oo-ui-icon-history {
  background-image: url('themes/wikimediaui/images/icons/history.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/history.svg');
}
.oo-ui-icon-info {
  background-image: url('themes/wikimediaui/images/icons/info.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/info.svg');
}
/* @noflip */
.oo-ui-icon-info:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb.svg');
}
.oo-ui-icon-infoFilled {
  background-image: url('themes/wikimediaui/images/icons/infoFilled.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/infoFilled.svg');
}
/* @noflip */
.oo-ui-icon-infoFilled:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb.svg');
}
.oo-ui-icon-robot {
  background-image: url('themes/wikimediaui/images/icons/robot.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/robot.svg');
}
.oo-ui-icon-specialPages {
  background-image: url('themes/wikimediaui/images/icons/specialPages-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialPages-ltr.svg');
}
.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-ltr.svg');
}
.oo-ui-icon-upload {
  background-image: url('themes/wikimediaui/images/icons/upload.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upload.svg');
}
.oo-ui-icon-window {
  background-image: url('themes/wikimediaui/images/icons/window.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/window.svg');
}
