/*!
 * VisualEditor UserInterface SpecialCharacterDialog styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-specialCharacterDialog .oo-ui-outlineOptionWidget {
	padding: 0.25em 0.5em;
}

.ve-ui-specialCharacterDialog .oo-ui-bookletLayout {
	height: 150px;
}

/* When displaying inside another dialog, make the menu narrower */
.oo-ui-dialog .ve-ui-specialCharacterDialog .oo-ui-bookletLayout .oo-ui-menuLayout-menu {
	width: 10em;
}

.oo-ui-dialog .ve-ui-specialCharacterDialog .oo-ui-bookletLayout .oo-ui-menuLayout-content {
	left: 10em;
}

.oo-ui-bookletLayout-stackLayout > .ve-ui-specialCharacterPage {
	padding: 0.5em;
}

.ve-ui-specialCharacterPage h3 {
	color: #54595d;
	margin: 0 0 0.5em 0;
	font-weight: normal;
	font-size: 1em;
}

.ve-ui-specialCharacterPage-characters {
	white-space: normal;
}

.ve-ui-specialCharacterPage-character {
	cursor: pointer;
	font-size: 1.5em;
	line-height: 1.8em;
	min-width: 1.8em;
	height: 1.8em;
	text-align: center;
	display: inline-block;
	vertical-align: top;
	margin: 0 3px 3px 0;
	border: 1px solid #eaecf0;
	-webkit-transition: border-color 200ms;
	-moz-transition: border-color 200ms;
	transition: border-color 200ms;
}

.ve-ui-specialCharacterPage-character:hover {
	border-color: #c8ccd1;
}

.ve-ui-specialCharacterPage-character-source {
	/* Specify a valid second value to fix size in Chrome/FF */
	font-family: monospace, 'Courier';
	font-size: 1em;
	line-height: 2.7em;
	height: 2.7em;
	padding: 0 0.2em;
}
