/*!
 * VisualEditor UserInterface Context styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-desktopContext {
	position: absolute;
}

.ve-ui-desktopContext-menu {
	position: absolute;
	width: 400px;
}

.ve-ui-desktopContext-menu .oo-ui-toolbar-bar {
	white-space: nowrap;
	border: 0;
	background: none;
}

.ve-ui-desktopContext-menu .oo-ui-toolGroup {
	border: 0;
	margin: 0;
}

.ve-ui-desktopContext-menu .oo-ui-tool,
.ve-ui-desktopContext-menu .oo-ui-tool:hover {
	border: 0;
}

.ve-ui-desktopContext-menu .oo-ui-tool:active,
.ve-ui-desktopContext-menu .oo-ui-tool-active {
	background-image: none;
}

.ve-ui-desktopContext-floating {
	position: fixed;
}

.ve-ui-desktopContext-embedded > .oo-ui-popupWidget > .oo-ui-popupWidget-popup {
	margin-top: 0.25em;
	margin-right: 0.25em;
}
