/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-logoCC {
  background-image: url('themes/wikimediaui/images/icons/logo-CC.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-CC.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoCC {
  background-image: url('themes/wikimediaui/images/icons/logo-CC-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-CC-invert.svg');
}
.oo-ui-icon-logoWikidata {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikidata.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikidata.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikidata {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikidata-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikidata-invert.svg');
}
.oo-ui-icon-logoWikimedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikimedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-invert.svg');
}
.oo-ui-icon-logoWikimediaCommons {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikimediaCommons {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons-invert.svg');
}
.oo-ui-icon-logoWikimediaDiscovery {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikimediaDiscovery {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery-invert.svg');
}
.oo-ui-icon-logoWikipedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikipedia.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikipedia.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikipedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikipedia-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikipedia-invert.svg');
}
