/*!
 * VisualEditor MediaWiki UserInterface TableDialog styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-ui-tableDialog-panel {
	max-width: 20em;
	margin: 0 auto;
}

.ve-ui-tableDialog .oo-ui-fieldLayout.oo-ui-fieldLayout-align-left.oo-ui-labelElement > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
	width: 75%;
	margin-right: 0;
}

.ve-ui-tableDialog .oo-ui-fieldLayout.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
	width: 25%;
	text-align: right;
}
