/*!
 * VisualEditor UserInterface MobileContext styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-mobileContext {
	position: fixed;
	left: 0;
	right: 0;
	bottom: 0;
	background: #fff;
	/* Match toolbar border & shadow */
	border-top: 1px solid #c8ccd1;
	box-shadow: 0 -1px 1px 0 rgba( 0, 0, 0, 0.1 );
	/* Transition out faster, as keyboard may be coming up */
	transition: transform 100ms;
	transform: translateY( 0% );
}

.ve-ui-mobileContext.ve-ui-context-hidden {
	display: block;
	transition: transform 250ms;
	transform: translateY( 100% );
}
