/*!
 * VisualEditor MediaWiki Initialization DesktopArticleTarget Minerva skin styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-init-mw-desktopArticleTarget .ve-init-mw-target-surface .ve-ce-attachedRootNode,
.ve-init-mw-desktopArticleTarget .ve-init-mw-target-surface .ve-ui-surface-placeholder {
	padding: 0;
	/* Provide clickable gutter */
	padding-left: 1000px;
	margin-left: -1000px;
}
