/*!
 * VisualEditor UserInterface CommentAnnotationContextItem styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-commentAnnotationContextItem-comment {
	margin-bottom: 0.5em;
}
