/*!
 * VisualEditor MediaWiki UserInterface MWParameterResultWidget styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-ui-mwParameterResultWidget {
	padding-right: 0.25em;
}

.ve-ui-mwParameterResultWidget.oo-ui-optionWidget-highlighted {
	background-color: #e1f3ff;
}

.ve-ui-mwParameterResultWidget .oo-ui-optionWidget-icon {
	top: 0.25em;
	margin-top: 0;
}

.ve-ui-mwParameterResultWidget-label,
.ve-ui-mwParameterResultWidget-names,
.ve-ui-mwParameterResultWidget-description {
	white-space: nowrap;
	text-overflow: ellipsis;
	overflow: hidden;
	/* HACK: Based on the expected width of this widget, using % produces unreliable results */
	max-width: 33em;
}

.ve-ui-mwParameterResultWidget-label {
	font-weight: bold;
	float: left;
}

.ve-ui-mwParameterResultWidget-names {
	float: right;
	font-size: 0.9em;
	padding: 0.2em 0;
}

.ve-ui-mwParameterResultWidget-name {
	padding: 0.1em 0.25em;
	margin-left: 0.5em;
	color: #54595d;
	background-color: #eaecf0;
	border: 1px solid #c8ccd1;
	border-radius: 2px;
}

.ve-ui-mwParameterResultWidget-name:first-child {
	background-color: #fff;
}

.ve-ui-mwParameterResultWidget-description {
	clear: both;
	color: #54595d;
}
