/*!
 * VisualEditor Standalone Initialization styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

figure {
	display: table;
}

figcaption {
	display: table-caption;
	caption-side: bottom;
}

/* @noflip */
.ve-align-right {
	float: right;
	clear: right;
	margin: 0.5em 0 1em 1em;
}

/* @noflip */
.ve-align-left {
	float: left;
	clear: left;
	margin: 0.5em 1em 1em 0;
}

.ve-align-center {
	margin: 0.5em auto 1em auto;
	display: table;
}
