/*!
 * Styles for Special:ComparePages
 */
@import 'mediawiki.mixins.less';

.mw-special-ComparePages .mw-htmlform-ooui-wrapper {
	width: 100%;
}

.mw-special-ComparePages .oo-ui-layout.oo-ui-panelLayout.oo-ui-panelLayout-padded.oo-ui-panelLayout-framed {
	float: left;
	width: 49%;
	.box-sizing( border-box );
}

.mw-special-ComparePages .oo-ui-layout.oo-ui-panelLayout.oo-ui-panelLayout-padded.oo-ui-panelLayout-framed:nth-of-type( 2 ) {
	margin-left: 2%;
}

.mw-special-ComparePages .mw-htmlform-submit-buttons {
	clear: both;
}
