/*!
 * Styles for Special:Block
 */

// OOUIHTMLForm specifics
@ooui-font-size-browser: 16; // Assumed browser default of `16px`.
@ooui-font-size-base: 0.875em; // Equals `14px` at browser default of `16px`.

@ooui-spacing-radio-label: 26 / @ooui-font-size-browser / @ooui-font-size-base; // Equals `1.85714286em`≈`26px`.

.mw-block-editing-restriction.oo-ui-fieldLayout {
	margin-left: @ooui-spacing-radio-label;
	margin-top: 8px;
}

.mw-block-restriction.oo-ui-fieldLayout.oo-ui-labelElement {
	margin-left: 2 * @ooui-spacing-radio-label;
	margin-top: 8px;

	.oo-ui-tagMultiselectWidget,
	.oo-ui-textInputWidget {
		max-width: 50em - 2 * @ooui-spacing-radio-label;
	}
}

.mw-special-Block .mw-htmlform {
	> .oo-ui-panelLayout-framed {
		border: 0;
	}

	> .oo-ui-panelLayout-padded {
		padding: 0;
	}
}
