/*!
 * VisualEditor ContentEditable AlienNode styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-alienNode-highlights .ve-ce-focusableNode-highlight {
	background-color: #95d14f;
	background-image: -webkit-repeating-linear-gradient( -45deg, #fff 0, #fff 5px, #95d14f 5px, #95d14f 10px );
	background-image: -moz-repeating-linear-gradient( -45deg, #fff 0, #fff 5px, #95d14f 5px, #95d14f 10px );
	background-image: repeating-linear-gradient( -45deg, #fff 0, #fff 5px, #95d14f 5px, #95d14f 10px );
	background-size: 14px 14px;
	cursor: not-allowed;
}

.ve-ce-surface-highlights-focused .ve-ce-alienNode-highlights .ve-ce-focusableNode-highlight {
	background-image: -webkit-repeating-linear-gradient( -45deg, #6da9f7 0, #6da9f7 5px, #95d14f 5px, #95d14f 10px );
	background-image: -moz-repeating-linear-gradient( -45deg, #6da9f7 0, #6da9f7 5px, #95d14f 5px, #95d14f 10px );
	background-image: repeating-linear-gradient( -45deg, #6da9f7 0, #6da9f7 5px, #95d14f 5px, #95d14f 10px );
}

.ve-ce-surface-highlights-focused .ve-ce-alienNode-highlights.ve-ce-focusableNode-highlights-deactivated .ve-ce-focusableNode-highlight {
	background-image: -webkit-repeating-linear-gradient( -45deg, #000 0, #000 5px, #666 5px, #666 10px );
	background-image: -moz-repeating-linear-gradient( -45deg, #000 0, #000 5px, #666 5px, #666 10px );
	background-image: repeating-linear-gradient( -45deg, #000 0, #000 5px, #666 5px, #666 10px );
}

.ve-ce-surface-highlights-blurred .ve-ce-alienNode-highlights {
	opacity: 0.5;
}

/* Support: IE, Edge */
/* Match the hack in ve.ce.FocusableNode.css */
.ve-init-target-ie-or-edge .ve-ce-surface-highlights-blurred .ve-ce-alienNode-highlights .ve-ce-focusableNode-highlight {
	opacity: 0.5;
}
