/**
 * Basic styles for the edit revision history page 'HistoryAction.php'
 */

/* stylelint-disable selector-class-pattern */

#pagehistory .history-user {
	margin-left: 0.4em;
	margin-right: 0.2em;
}

#pagehistory li {
	padding: 1px;
}

#pagehistory li.selected {
	background-color: #f8f9fa;
	color: #202122;
	outline: 1px dashed #a2a9b1;
}

.mw-history-revisionactions {
	float: right;
}

.updatedmarker {
	background-color: #b7f430;
}
