/*!
 * VisualEditor ContentEditable SectionNode styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-focusableNode .ve-ce-activeNode {
	cursor: text;
}

.ve-ce-focusableNode .ve-ce-activeNode,
.ve-ce-focusableNode .ve-ce-activeNode * {
	/* T174774: We're using the vendor prefixed form of `text` here since it
	   has the semantics that it won't be overridden by ancestors having the
	   value `all` */
	-moz-user-select: -moz-text;
}
