/*!
 * VisualEditor UserInterface LanguageSearchWidget/LanguageResultWidget styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

/* ve.ui.LanguageResultWidget */

.ve-ui-languageResultWidget-otherMatch {
	color: #72777d;
	float: right;
}

.ve-ui-languageResultWidget .oo-ui-labelWidget {
	cursor: inherit;
	padding: 0.5em 0;
}
