/*!
 * VisualEditor ContentEditable general styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-bidi-isolate {
	unicode-bidi: isolate;
	unicode-bidi: -moz-isolate;
	unicode-bidi: -webkit-isolate;
}
