/**
 * @class jQuery
 * <http://api.jquery.com/>
 */

/**
 * @class jQuery.Event
 * <http://api.jquery.com/Types/#Event>
 */

/**
 * @class jQuery.Promise
 * <http://api.jquery.com/Types/#Promise>
 */

/**
 * @class ve.ui.Surface
 * <https://doc.wikimedia.org/VisualEditor/master/#!/api/ve.ui.Surface>
 */
