/*!
 * VisualEditor UserInterface targetWidget styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-targetWidget .ve-ui-surface .ve-ce-attachedRootNode,
.ve-ui-targetWidget .ve-ui-surface .ve-ui-surface-placeholder {
	/* 0.5/0.8, 1/0.8 */
	padding: 0.625em 1.25em;
}

.ve-ui-targetWidget .ve-ui-debugBar {
	padding: 1em;
}
