/*!
 * VisualEditor UserInterface FragmentInspector styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ui-fragmentInspector-container.oo-ui-panelLayout-padded,
.ve-ui-fragmentInspector-container .oo-ui-indexLayout-stackLayout > .oo-ui-panelLayout-padded {
	padding: 0.75em;
}

.ve-ui-fragmentInspector-content .oo-ui-processDialog-actions-other .oo-ui-buttonElement-framed {
	margin-left: 0;
	margin-bottom: 0;
}

/* Don't show scroll bars before ready for accurate measurement */
.oo-ui-window-content-setup:not( .oo-ui-window-content-ready ) .ve-ui-fragmentInspector-container {
	overflow: hidden;
}
