/**
 * Stylesheet for the CategoryTree extension.
 *
 * © 2006 Daniel Kinzler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @file
 * @ingroup Extensions
 * @author Daniel Kinzler, brightbyte.de
 */

.client-js #mw-subcategories ul {
	list-style: none none;
	margin-left: 0.25em;
}

#mw-panel .portal .body .CategoryTreeTag {
	margin: 0;
	font-size: 0.75em;
	line-height: 1.125em;
}

#mw-panel .portal .body .CategoryTreeItem {
	padding: 0.25em 0;
}

.CategoryTreeChildren {
	margin-left: 1.25em;
}

.CategoryTreeBullet,
.CategoryTreeEmptyBullet {
	cursor: default;
	-webkit-user-select: none;
	-moz-user-select: none;
	-ms-user-select: none;
	user-select: none;
}

.CategoryTreeEmptyBullet {
	color: #c0c0c0;
}

.CategoryTreeBullet a,
.CategoryTreeBullet a:link,
.CategoryTreeBullet a:active,
.CategoryTreeBullet a:visited {
	text-decoration: none;
	color: inherit;
	speak: none;
}

/* Hide toggle buttons for clients without JavaScript */
.client-nojs .CategoryTreeBullet,
.client-nojs .CategoryTreeEmptyBullet {
	display: none;
}

.CategoryTreeToggle {
	color: #0645ad;
}

.CategoryTreeLabelPage {
	font-style: italic;
}

.CategoryTreeParents {
	margin-top: 1.5ex;
	font-style: italic;
}

.CategoryTreeTag {
	margin-top: 0.5ex;
	margin-bottom: 0.5ex;
}

.CategoryTreeResult {
	margin-top: 1.5ex;
}

.CategoryTreeNotice {
	font-style: italic;
}

.CategoryTreeSection {
	direction: ltr;
}
