/*!
 * VisualEditor ContentEditable LanguageAnnotation styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see http://ve.mit-license.org
 */

.ve-ce-linkAnnotation.ve-ce-annotation-active {
	box-shadow: 0 0 0 1px #c6e0ff;
	border-radius: 2px;
	padding: 2px;
	margin: -2px;
	background-color: #e6f1ff;
}

.ve-init-mobileTarget .ve-ce-surface:not( .ve-ce-surface-deactivated ) .ve-ce-linkAnnotation.ve-ce-annotation-active {
	box-shadow: 0 0 0 1px #36c;
	background-color: transparent;
}

.ve-ce-nail:not( .ve-ce-nail-debug ) {
	width: 0;
	height: 0;
}

.ve-ce-nail-debug {
	vertical-align: middle;
}

.ve-init-target-ctrl-meta-down .ve-ce-linkAnnotation {
	cursor: pointer;
}
