<?php
/** Scottish Gaelic (Gàidhlig)
 *
 * To improve a translation please visit https://translatewiki.net
 *
 * @ingroup Language
 * @file
 *
 */

$namespaceNames = [
	NS_MEDIA            => 'Meadhan',
	NS_SPECIAL          => 'Sònraichte',
	NS_TALK             => 'Deasbaireachd',
	NS_USER             => 'Cleachdaiche',
	NS_USER_TALK        => 'Deasbaireachd_a\'_chleachdaiche',
	NS_PROJECT_TALK     => 'An_deasbaireachd_aig_$1',
	NS_FILE             => 'Faidhle',
	NS_FILE_TALK        => 'Deasbaireachd_an_fhaidhle',
	NS_MEDIAWIKI        => 'MediaWiki',
	NS_MEDIAWIKI_TALK   => 'Deasbaireachd_MediaWiki',
	NS_TEMPLATE         => 'Teamplaid',
	NS_TEMPLATE_TALK    => 'Deasbaireachd_na_teamplaid',
	NS_HELP             => 'Cobhair',
	NS_HELP_TALK        => 'Deasbaireachd_na_cobharach',
	NS_CATEGORY         => 'Roinn-seòrsa',
	NS_CATEGORY_TALK    => 'Deasbaireachd_na_roinn-seòrsa',
];
