.mw-widgets-rowWidget {
	clear: left;
	float: left;
	margin-bottom: -1px;
	width: 100%;
}

.mw-widgets-rowWidget-label {
	display: block;
	margin-right: 5%;
	padding-top: 0.5em;
	width: 35%;
}

.mw-widgets-rowWidget > .mw-widgets-rowWidget-label {
	float: left;
}

.mw-widgets-rowWidget > .mw-widgets-rowWidget-cells {
	float: left;
}

.mw-widgets-rowWidget > .mw-widgets-rowWidget-cells > .oo-ui-inputWidget {
	float: left;
	margin-right: -1px;
	width: 8em;
}

.mw-widgets-rowWidget > .mw-widgets-rowWidget-cells > .oo-ui-inputWidget > input,
.mw-widgets-rowWidget > .mw-widgets-rowWidget-delete-button > .oo-ui-buttonElement-button {
	margin: 0;
	border-radius: 0;
}
