/*!
 * VisualEditor MediaWiki UserInterface MWParameterPage styles.
 *
 * @copyright 2011-2020 VisualEditor Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.ve-ui-mwParameterPage.oo-ui-pageLayout {
	padding-top: 0;
	padding-bottom: 0;
	position: relative;
}

.ve-ui-mwParameterPage.oo-ui-pageLayout:last-of-type {
	margin-bottom: 1.5em;
}

.ve-ui-mwParameterPage.oo-ui-pageLayout-active .ve-ui-mwParameterPage {
	background-color: #f8f9fa;
}

.ve-ui-mwParameterPage-info {
	float: left;
	cursor: default;
	padding-left: 2.5em;
	width: 80%;
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;
}

.ve-ui-mwParameterPage-actions {
	cursor: default;
	position: absolute;
	top: -0.3125em;
	right: 4em; /* 1.5em for PanelLayout's padding + 2.5em */
	width: 25%;
	text-align: right;
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;
}

.ve-ui-mwParameterPage-label {
	display: inline;
	vertical-align: middle;
	font-size: 1.1em;
	font-weight: bold;
	line-height: 1.705em;
}

.ve-ui-mwParameterPage-statusIndicator {
	display: inline-block;
	vertical-align: middle;
	opacity: 0.5;
}

.ve-ui-mwParameterPage-description {
	font-size: 0.9em;
	line-height: 1.5em;
	padding: 0.25em 1em;
}

.ve-ui-mwParameterPage-description-required,
.ve-ui-mwParameterPage-description-deprecated {
	font-style: italic;
	color: #54595d;
}

.ve-ui-mwParameterPage-infoButton,
.ve-ui-mwParameterPage-removeButton {
	display: none;
}

.ve-ui-mwParameterPage-infoButton .oo-ui-popupWidget {
	z-index: 100;
	text-align: left;
}

.oo-ui-pageLayout-active .ve-ui-mwParameterPage-infoButton,
.oo-ui-pageLayout-active .ve-ui-mwParameterPage-removeButton {
	display: inline-block;
}

.ve-ui-mwParameterPage-field {
	position: relative;
	padding: 0.25em 2.5em 1em;
	clear: both;
}

.ve-ui-mwParameterPage .oo-ui-textInputWidget {
	display: inline-block;
	vertical-align: top;
	width: 100%;
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;
}

.ve-ui-mwParameterPage .oo-ui-textInputWidget textarea {
	height: 2.5em;
}

.ve-ui-mwParameterPage-more {
	display: none;
	margin: 1em 0 0.5em 0;
}

.ve-ui-mwParameterPage:last-child .ve-ui-mwParameterPage-more {
	display: block;
}
