/*!
 * MediaWiki Widgets – CopyTextLayout styles.
 *
 * @copyright 2011-2015 MediaWiki Widgets Team and others; see AUTHORS.txt
 * @license The MIT License (MIT); see LICENSE.txt
 */

.mw-widget-copyTextLayout > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
	/* TODO: This should be upstreamed to OOUI */
	max-width: 50em;
}

.mw-widget-copyTextLayout-multiline-button {
	display: block;
	max-width: 50em;
	margin-top: 0.5em;
	/* Float to right of inline help */
	float: right;
}
