<?php
/**
 * Aliases for Special:CiteThisPage
 *
 * @file
 * @ingroup Extensions
 */

$specialPageAliases = [];

/** English (English) */
$specialPageAliases['en'] = [
	'CiteThisPage' => [ 'CiteThisPage', 'Cite' ],
];

/** Arabic (العربية) */
$specialPageAliases['ar'] = [
	'CiteThisPage' => [ 'استشهاد' ],
];

/** Egyptian Arabic (مصرى) */
$specialPageAliases['arz'] = [
	'CiteThisPage' => [ 'استشهاد' ],
];

/** Assamese (অসমীয়া) */
$specialPageAliases['as'] = [
	'CiteThisPage' => [ 'উদ্ধৃতি' ],
];

/** Avaric (авар) */
$specialPageAliases['av'] = [
	'CiteThisPage' => [ 'Цитата' ],
];

/** Bikol Central (Bikol Central) */
$specialPageAliases['bcl'] = [
	'CiteThisPage' => [ 'Sambitón' ],
];

/** Bulgarian (български) */
$specialPageAliases['bg'] = [
	'CiteThisPage' => [ 'Цитиране' ],
];

/** Western Balochi (بلوچی رخشانی) */
$specialPageAliases['bgn'] = [
	'CiteThisPage' => [ 'یات_کورت' ],
];

/** Banjar (Bahasa Banjar) */
$specialPageAliases['bjn'] = [
	'CiteThisPage' => [ 'Juhut' ],
];

/** Breton (brezhoneg) */
$specialPageAliases['br'] = [
	'CiteThisPage' => [ 'Menegiñ' ],
];

/** Bosnian (bosanski) */
$specialPageAliases['bs'] = [
	'CiteThisPage' => [ 'Citiraj' ],
];

/** буряад (буряад) */
$specialPageAliases['bxr'] = [
	'CiteThisPage' => [ 'Сайт' ],
];

/** Catalan (català) */
$specialPageAliases['ca'] = [
	'CiteThisPage' => [ 'Citau', 'Citeu' ],
];

/** Min Dong Chinese (Mìng-dĕ̤ng-ngṳ̄) */
$specialPageAliases['cdo'] = [
	'CiteThisPage' => [ '註' ],
];

/** Chechen (нохчийн) */
$specialPageAliases['ce'] = [
	'CiteThisPage' => [ 'Дош' ],
];

/** Czech (čeština) */
$specialPageAliases['cs'] = [
	'CiteThisPage' => [ 'Citovat' ],
];

/** Danish (dansk) */
$specialPageAliases['da'] = [
	'CiteThisPage' => [ 'Citer' ],
];

/** German (Deutsch) */
$specialPageAliases['de'] = [
	'CiteThisPage' => [ 'Zitierhilfe', 'Zitieren' ],
];

/** Zazaki (Zazaki) */
$specialPageAliases['diq'] = [
	'CiteThisPage' => [ 'Sita' ],
];

/** Lower Sorbian (dolnoserbski) */
$specialPageAliases['dsb'] = [
	'CiteThisPage' => [ 'Citěrowańska_pomoc' ],
];

/** Greek (Ελληνικά) */
$specialPageAliases['el'] = [
	'CiteThisPage' => [ 'ΠαραπομπήΑυτήΤηΣελίδα', 'Παραπομπή' ],
];

/** Esperanto (Esperanto) */
$specialPageAliases['eo'] = [
	'CiteThisPage' => [ 'Citi' ],
];

/** Spanish (español) */
$specialPageAliases['es'] = [
	'CiteThisPage' => [ 'Citar' ],
];

/** Estonian (eesti) */
$specialPageAliases['et'] = [
	'CiteThisPage' => [ 'Sellele_leheküljele_viitamine', 'Tsiteerimine' ],
];

/** Persian (فارسی) */
$specialPageAliases['fa'] = [
	'CiteThisPage' => [ 'یادکرد' ],
];

/** Finnish (suomi) */
$specialPageAliases['fi'] = [
	'CiteThisPage' => [ 'Viittaus' ],
];

/** French (français) */
$specialPageAliases['fr'] = [
	'CiteThisPage' => [ 'Citer' ],
];

/** Arpitan (arpetan) */
$specialPageAliases['frp'] = [
	'CiteThisPage' => [ 'Citar' ],
];

/** Western Frisian (Frysk) */
$specialPageAliases['fy'] = [
	'CiteThisPage' => [ 'Sitearje' ],
];

/** Galician (galego) */
$specialPageAliases['gl'] = [
	'CiteThisPage' => [ 'Cita', 'Citar' ],
];

/** Swiss German (Alemannisch) */
$specialPageAliases['gsw'] = [
	'CiteThisPage' => [ 'Zitierhilf' ],
];

/** Hebrew (עברית) */
$specialPageAliases['he'] = [
	'CiteThisPage' => [ 'ציטוט_דף_זה', 'ציטוט' ],
];

/** Croatian (hrvatski) */
$specialPageAliases['hr'] = [
	'CiteThisPage' => [ 'Citiraj' ],
];

/** Upper Sorbian (hornjoserbsce) */
$specialPageAliases['hsb'] = [
	'CiteThisPage' => [ 'Citowanska_pomoc' ],
];

/** Xiang Chinese (湘语) */
$specialPageAliases['hsn'] = [
	'CiteThisPage' => [ '建脚注' ],
];

/** Haitian (Kreyòl ayisyen) */
$specialPageAliases['ht'] = [
	'CiteThisPage' => [ 'Site' ],
];

/** Hungarian (magyar) */
$specialPageAliases['hu'] = [
	'CiteThisPage' => [ 'Hivatkozás', 'Irodalomjegyzék' ],
];

/** Interlingua (interlingua) */
$specialPageAliases['ia'] = [
	'CiteThisPage' => [ 'Citation' ],
];

/** Indonesian (Bahasa Indonesia) */
$specialPageAliases['id'] = [
	'CiteThisPage' => [ 'Kutip' ],
];

/** Igbo (Igbo) */
$specialPageAliases['ig'] = [
	'CiteThisPage' => [ 'Dépùtà' ],
];

/** Ido (Ido) */
$specialPageAliases['io'] = [
	'CiteThisPage' => [ 'Citar' ],
];

/** Italian (italiano) */
$specialPageAliases['it'] = [
	'CiteThisPage' => [ 'Cita' ],
];

/** Japanese (日本語) */
$specialPageAliases['ja'] = [
	'CiteThisPage' => [ 'このページを引用', '引用' ],
];

/** Korean (한국어) */
$specialPageAliases['ko'] = [
	'CiteThisPage' => [ '이문서인용', '인용' ],
];

/** Colognian (Ripoarisch) */
$specialPageAliases['ksh'] = [
	'CiteThisPage' => [ 'Zitteere' ],
];

/** Cornish (kernowek) */
$specialPageAliases['kw'] = [
	'CiteThisPage' => [ 'Devynna' ],
];

/** Ladino (Ladino) */
$specialPageAliases['lad'] = [
	'CiteThisPage' => [ 'MostrarManaderos' ],
];

/** Luxembourgish (Lëtzebuergesch) */
$specialPageAliases['lb'] = [
	'CiteThisPage' => [ 'Zitéierhellëf' ],
];

/** Northern Luri (لۊری شومالی) */
$specialPageAliases['lrc'] = [
	'CiteThisPage' => [ 'تیار_کئردئن_ئی_بألگە', 'تیارگە' ],
];

/** Literary Chinese (文言) */
$specialPageAliases['lzh'] = [
	'CiteThisPage' => [ '引文' ],
];

/** Macedonian (македонски) */
$specialPageAliases['mk'] = [
	'CiteThisPage' => [ 'Навод' ],
];

/** Malayalam (മലയാളം) */
$specialPageAliases['ml'] = [
	'CiteThisPage' => [ 'അവലംബം' ],
];

/** Marathi (मराठी) */
$specialPageAliases['mr'] = [
	'CiteThisPage' => [ 'संदर्भद्या' ],
];

/** Malay (Bahasa Melayu) */
$specialPageAliases['ms'] = [
	'CiteThisPage' => [ 'Petik_laman_ini', 'Petik' ],
];

/** Maltese (Malti) */
$specialPageAliases['mt'] = [
	'CiteThisPage' => [ 'Iċċita' ],
];

/** Nāhuatl (Nāhuatl) */
$specialPageAliases['nah'] = [
	'CiteThisPage' => [ 'Tlahtoa', 'Citar' ],
];

/** Norwegian Bokmål (norsk bokmål) */
$specialPageAliases['nb'] = [
	'CiteThisPage' => [ 'Siteringshjelp' ],
];

/** Low German (Plattdüütsch) */
$specialPageAliases['nds'] = [
	'CiteThisPage' => [ 'Ziteerhelp' ],
];

/** Low Saxon (Netherlands) (Nedersaksies) */
$specialPageAliases['nds-nl'] = [
	'CiteThisPage' => [ 'Siteerhulpe' ],
];

/** Dutch (Nederlands) */
$specialPageAliases['nl'] = [
	'CiteThisPage' => [ 'Citeren' ],
];

/** Norwegian Nynorsk (norsk nynorsk) */
$specialPageAliases['nn'] = [
	'CiteThisPage' => [ 'Siter' ],
];

/** Occitan (occitan) */
$specialPageAliases['oc'] = [
	'CiteThisPage' => [ 'Citar' ],
];

/** Polish (polski) */
$specialPageAliases['pl'] = [
	'CiteThisPage' => [ 'Cytuj' ],
];

/** Pashto (پښتو) */
$specialPageAliases['ps'] = [
	'CiteThisPage' => [ 'درک' ],
];

/** Portuguese (português) */
$specialPageAliases['pt'] = [
	'CiteThisPage' => [ 'Citar' ],
];

/** Brazilian Portuguese (português do Brasil) */
$specialPageAliases['pt-br'] = [
	'CiteThisPage' => [ 'Citar' ],
];

/** Romanian (română) */
$specialPageAliases['ro'] = [
	'CiteThisPage' => [ 'Citează' ],
];

/** Russian (русский) */
$specialPageAliases['ru'] = [
	'CiteThisPage' => [ 'Цитата' ],
];

/** Sanskrit (संस्कृतम्) */
$specialPageAliases['sa'] = [
	'CiteThisPage' => [ 'उद्धृत' ],
];

/** Sicilian (sicilianu) */
$specialPageAliases['scn'] = [
	'CiteThisPage' => [ 'Cita' ],
];

/** Serbo-Croatian (srpsko-hrvatski) */
$specialPageAliases['sh'] = [
	'CiteThisPage' => [ 'Citiraj', 'Citiraj_ovu_stranicu', 'Citiraj_ovu_stranu' ],
];

/** Slovak (slovenčina) */
$specialPageAliases['sk'] = [
	'CiteThisPage' => [ 'Citovať' ],
];

/** Slovenian (slovenščina) */
$specialPageAliases['sl'] = [
	'CiteThisPage' => [ 'Navedi' ],
];

/** Albanian (shqip) */
$specialPageAliases['sq'] = [
	'CiteThisPage' => [ 'Citim' ],
];

/** Serbian (Cyrillic script) (српски (ћирилица)) */
$specialPageAliases['sr-ec'] = [
	'CiteThisPage' => [ 'Цитирај', 'Цитирај_ову_страницу', 'Цитирај_ову_страну' ],
];

/** Serbian (Latin script) (srpski (latinica)) */
$specialPageAliases['sr-el'] = [
	'CiteThisPage' => [ 'Citiraj', 'Citiraj_ovu_stranicu', 'Citiraj_ovu_stranu' ],
];

/** Swedish (svenska) */
$specialPageAliases['sv'] = [
	'CiteThisPage' => [ 'Citera' ],
];

/** Swahili (Kiswahili) */
$specialPageAliases['sw'] = [
	'CiteThisPage' => [ 'Taja', 'Hakikisha' ],
];

/** Tetum (tetun) */
$specialPageAliases['tet'] = [
	'CiteThisPage' => [ 'Sita' ],
];

/** Thai (ไทย) */
$specialPageAliases['th'] = [
	'CiteThisPage' => [ 'อ้างอิง' ],
];

/** Tagalog (Tagalog) */
$specialPageAliases['tl'] = [
	'CiteThisPage' => [ 'Sipiin' ],
];

/** Turkish (Türkçe) */
$specialPageAliases['tr'] = [
	'CiteThisPage' => [ 'KaynakGöster' ],
];

/** Ukrainian (українська) */
$specialPageAliases['uk'] = [
	'CiteThisPage' => [ 'Цитата' ],
];

/** Urdu (اردو) */
$specialPageAliases['ur'] = [
	'CiteThisPage' => [ 'حوالہ' ],
];

/** Venetian (vèneto) */
$specialPageAliases['vec'] = [
	'CiteThisPage' => [ 'Cita' ],
];

/** Vietnamese (Tiếng Việt) */
$specialPageAliases['vi'] = [
	'CiteThisPage' => [ 'Trích_dẫn' ],
];

/** Yiddish (ייִדיש) */
$specialPageAliases['yi'] = [
	'CiteThisPage' => [ 'ציטירן' ],
];

/** Cantonese (粵語) */
$specialPageAliases['yue'] = [
	'CiteThisPage' => [ '引用' ],
];

/** Simplified Chinese (中文（简体）‎) */
$specialPageAliases['zh-hans'] = [
	'CiteThisPage' => [ '引用此页面', '引用' ],
];

/** Traditional Chinese (中文（繁體）‎) */
$specialPageAliases['zh-hant'] = [
	'CiteThisPage' => [ '引用' ],
];
