/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-element-hidden {
  display: none !important;
}

.oo-ui-buttonElement {
  display: inline-block;
  line-height: normal;
  vertical-align: middle;
}
.oo-ui-buttonElement > .oo-ui-buttonElement-button {
  cursor: pointer;
  display: inline-block;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  vertical-align: middle;
  font-family: inherit;
  font-size: inherit;
  white-space: nowrap;
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.oo-ui-buttonElement > .oo-ui-buttonElement-button::-moz-focus-inner {
  border-color: transparent;
  padding: 0;
}
.oo-ui-buttonElement.oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  cursor: default;
}
.oo-ui-buttonElement-frameless {
  position: relative;
}
.oo-ui-buttonElement-framed > .oo-ui-buttonElement-button {
  vertical-align: top;
  text-align: center;
}
.oo-ui-buttonElement > .oo-ui-buttonElement-button {
  position: relative;
  min-height: 2.28571429em;
  border-radius: 2px;
  padding-top: 2.14285714em;
  font-weight: bold;
  text-decoration: none;
}
.oo-ui-buttonElement > .oo-ui-buttonElement-button:focus {
  outline: 0;
}
.oo-ui-buttonElement > input.oo-ui-buttonElement-button {
  -webkit-appearance: none;
}
.oo-ui-buttonElement.oo-ui-labelElement > .oo-ui-buttonElement-button {
  line-height: 1;
}
.oo-ui-buttonElement.oo-ui-labelElement > input.oo-ui-buttonElement-button,
.oo-ui-buttonElement.oo-ui-labelElement > .oo-ui-buttonElement-button > .oo-ui-labelElement-label {
  line-height: 1.28571429em;
}
.oo-ui-buttonElement.oo-ui-labelElement.oo-ui-indicatorElement > .oo-ui-buttonElement-button {
  padding-left: 2.14285714em;
}
.oo-ui-buttonElement.oo-ui-iconElement .oo-ui-iconElement-icon,
.oo-ui-buttonElement.oo-ui-indicatorElement .oo-ui-indicatorElement-indicator {
  -webkit-transform: translateZ(0);
  transform: translateZ(0);
}
.oo-ui-buttonElement.oo-ui-indicatorElement.oo-ui-labelElement > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator,
.oo-ui-buttonElement.oo-ui-indicatorElement.oo-ui-iconElement > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  left: 0.71428571em;
}
.oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button {
  -webkit-transition: background-color 100ms, color 100ms, border-color 100ms, box-shadow 100ms;
  -moz-transition: background-color 100ms, color 100ms, border-color 100ms, box-shadow 100ms;
  transition: background-color 100ms, color 100ms, border-color 100ms, box-shadow 100ms;
}
.oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon,
.oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  -webkit-transition: opacity 100ms;
  -moz-transition: opacity 100ms;
  transition: opacity 100ms;
}
.oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon:not( .oo-ui-image-invert ),
.oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator:not( .oo-ui-image-invert ) {
  opacity: 0.87;
}
.oo-ui-buttonElement.oo-ui-widget-enabled.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon,
.oo-ui-buttonElement.oo-ui-widget-enabled.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-buttonElement-frameless.oo-ui-iconElement:first-child {
  margin-right: -0.42857143em;
}
.oo-ui-buttonElement-frameless.oo-ui-iconElement > .oo-ui-buttonElement-button {
  min-width: 1.42857143em;
  min-height: 1.42857143em;
  border-color: transparent;
  border-style: solid;
  border-width: 1px;
  padding-top: 2.14285714em;
  padding-right: 2.14285714em;
}
.oo-ui-buttonElement-frameless.oo-ui-iconElement > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon {
  right: 0.35714286em;
}
.oo-ui-buttonElement-frameless.oo-ui-labelElement:first-child {
  margin-right: -6px;
}
.oo-ui-buttonElement-frameless.oo-ui-labelElement.oo-ui-iconElement:first-child {
  margin-right: -0.42857143em;
}
.oo-ui-buttonElement-frameless.oo-ui-labelElement > .oo-ui-buttonElement-button {
  border-color: transparent;
  border-style: solid;
  border-width: 1px;
  padding: 6px 6px;
}
.oo-ui-buttonElement-frameless.oo-ui-labelElement.oo-ui-iconElement > .oo-ui-buttonElement-button {
  padding-right: 2.14285714em;
}
.oo-ui-buttonElement-frameless.oo-ui-indicatorElement > .oo-ui-buttonElement-button {
  min-width: 12px;
  min-height: 12px;
}
.oo-ui-buttonElement-frameless.oo-ui-indicatorElement.oo-ui-iconElement > .oo-ui-buttonElement-button {
  padding-right: 3.85714286em;
}
.oo-ui-buttonElement-frameless.oo-ui-indicatorElement.oo-ui-labelElement > .oo-ui-buttonElement-button {
  padding-right: 6px;
  padding-top: 6px;
}
.oo-ui-buttonElement-frameless.oo-ui-indicatorElement.oo-ui-iconElement.oo-ui-labelElement > .oo-ui-buttonElement-button {
  padding-right: 2.14285714em;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled > .oo-ui-buttonElement-button {
  color: #202122;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled > .oo-ui-buttonElement-button:hover {
  background-color: rgba(0, 24, 73, 0.02745098);
  color: #000;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled > .oo-ui-buttonElement-button:active {
  background-color: rgba(0, 36, 73, 0.08235294);
  color: #404244;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-iconElement > .oo-ui-buttonElement-button:focus,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-labelElement > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-iconElement > .oo-ui-buttonElement-button:focus:active,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-labelElement > .oo-ui-buttonElement-button:focus:active {
  border-color: transparent;
  box-shadow: none;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-indicatorElement:not( .oo-ui-iconElement ):not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button {
  border-radius: 1px;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-indicatorElement:not( .oo-ui-iconElement ):not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button:focus {
  box-shadow: 0 0 0 2px #36c;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-indicatorElement:not( .oo-ui-iconElement ):not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button:focus:active {
  box-shadow: none;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-buttonElement-pressed > input.oo-ui-buttonElement-button,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled > .oo-ui-buttonElement-button:active {
  color: #000;
  border-color: transparent;
  box-shadow: none;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button {
  color: #36c;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:hover {
  color: #447ff5;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button {
  color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button {
  color: #d33;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:hover {
  color: #ff4242;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button {
  color: #b32424;
  box-shadow: none;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled[class*='oo-ui-flaggedElement'] > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon,
.oo-ui-buttonElement-frameless.oo-ui-widget-enabled[class*='oo-ui-flaggedElement'] > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  color: #72777d;
}
.oo-ui-buttonElement-frameless.oo-ui-widget-disabled > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon,
.oo-ui-buttonElement-frameless.oo-ui-widget-disabled > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  opacity: 0.51;
}
.oo-ui-buttonElement-framed > .oo-ui-buttonElement-button {
  border-style: solid;
  border-width: 1px;
  border-radius: 2px;
  padding-right: 12px;
  padding-left: 12px;
}
.oo-ui-buttonElement-framed.oo-ui-iconElement > .oo-ui-buttonElement-button {
  padding-top: 2.14285714em;
  padding-bottom: 0;
  padding-right: 2.14285714em;
}
.oo-ui-buttonElement-framed.oo-ui-iconElement > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon {
  right: 50%;
  margin-right: -0.71428571em;
}
.oo-ui-buttonElement-framed.oo-ui-iconElement.oo-ui-labelElement > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-iconElement.oo-ui-indicatorElement > .oo-ui-buttonElement-button {
  padding-right: 2.71428571em;
}
.oo-ui-buttonElement-framed.oo-ui-iconElement.oo-ui-labelElement > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon,
.oo-ui-buttonElement-framed.oo-ui-iconElement.oo-ui-indicatorElement > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon {
  right: 0.78571429em;
  margin-right: 0;
}
.oo-ui-buttonElement-framed.oo-ui-indicatorElement > .oo-ui-buttonElement-button {
  padding-top: 2.14285714em;
  padding-left: 1.71428571em;
  padding-bottom: 0;
}
.oo-ui-buttonElement-framed.oo-ui-labelElement > .oo-ui-buttonElement-button {
  padding-top: 6px;
  padding-bottom: 6px;
}
.oo-ui-buttonElement-framed.oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  background-color: #c8ccd1;
  color: #fff;
  border-color: #c8ccd1;
}
.oo-ui-buttonElement-framed.oo-ui-widget-disabled.oo-ui-buttonElement-active > .oo-ui-buttonElement-button {
  background-color: #919fb9;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button {
  background-color: #f8f9fa;
  color: #202122;
  border-color: #a2a9b1;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button:hover {
  background-color: #fff;
  color: #404244;
  border-color: #a2a9b1;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button:hover > .oo-ui-iconElement-icon:not( .oo-ui-image-invert ),
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button:hover > .oo-ui-indicatorElement-indicator:not( .oo-ui-image-invert ) {
  opacity: 0.74;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button {
  background-color: #c8ccd1;
  color: #000;
  border-color: #72777d;
  box-shadow: none;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-buttonElement-active > .oo-ui-buttonElement-button {
  background-color: #2a4b8d;
  color: #fff;
  border-color: #2a4b8d;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-buttonElement-active > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c, inset 0 0 0 2px #fff;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button {
  color: #36c;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:hover {
  background-color: #fff;
  border-color: #447ff5;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive.oo-ui-buttonElement-active > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive.oo-ui-popupToolGroup-active > .oo-ui-buttonElement-button {
  background-color: #eff3fa;
  color: #2a4b8d;
  border-color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button {
  color: #d73333;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:hover {
  background-color: #fff;
  border-color: #ff4242;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive.oo-ui-buttonElement-active > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive.oo-ui-popupToolGroup-active > .oo-ui-buttonElement-button {
  background-color: #ffffff;
  color: #b32424;
  border-color: #b32424;
  box-shadow: none;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:focus {
  border-color: #d33;
  box-shadow: inset 0 0 0 1px #d33;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button {
  color: #fff;
  background-color: #36c;
  border-color: #36c;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:hover {
  background-color: #447ff5;
  border-color: #447ff5;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-buttonElement-active > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-popupToolGroup-active > .oo-ui-buttonElement-button {
  color: #fff;
  background-color: #2a4b8d;
  border-color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c, inset 0 0 0 2px #fff;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button {
  color: #fff;
  background-color: #d33;
  border-color: #d33;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:hover {
  background-color: #ff4242;
  border-color: #ff4242;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:active,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:active:focus,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-buttonElement-pressed > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-buttonElement-active > .oo-ui-buttonElement-button,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-popupToolGroup-active > .oo-ui-buttonElement-button {
  color: #fff;
  background-color: #b32424;
  border-color: #b32424;
  box-shadow: none;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-buttonElement-button:focus {
  border-color: #d33;
  box-shadow: inset 0 0 0 1px #d33, inset 0 0 0 2px #fff;
  outline: 1px solid transparent;
}
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary > .oo-ui-buttonElement-button > .oo-ui-iconElement-icon,
.oo-ui-buttonElement-framed.oo-ui-widget-enabled.oo-ui-flaggedElement-primary > .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  opacity: 1;
}

.oo-ui-clippableElement-clippable {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  min-height: 40px;
  -webkit-overflow-scrolling: touch;
}

.oo-ui-floatableElement {
  position: absolute;
}



.oo-ui-labelElement .oo-ui-labelElement-label,
.oo-ui-labelElement.oo-ui-labelElement-label {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-labelElement-invisible {
  display: block;
  position: absolute;
  clip: rect(1px, 1px, 1px, 1px);
  width: 1px;
  height: 1px;
  margin: -1px;
  border: 0;
  padding: 0;
  overflow: hidden;
}
.oo-ui-labelElement .oo-ui-labelElement-label {
  line-height: 1.42857143em;
}
.oo-ui-labelElement .oo-ui-labelElement-label-highlight {
  font-weight: bold;
}

.oo-ui-iconElement-icon {
  background-size: contain;
  background-position: center center;
  background-repeat: no-repeat;
  position: absolute;
  top: 0;
  min-width: 20px;
  width: 1.42857143em;
  min-height: 20px;
  height: 100%;
}
.oo-ui-iconElement-noIcon {
  display: none;
}

.oo-ui-indicatorElement-indicator {
  background-size: contain;
  background-position: center center;
  background-repeat: no-repeat;
  position: absolute;
  top: 0;
  min-width: 12px;
  width: 0.85714286em;
  min-height: 12px;
  height: 100%;
}
.oo-ui-indicatorElement-noIndicator {
  display: none;
}

.oo-ui-pendingElement-pending {
  background-color: #eaecf0;
  background-image: -webkit-linear-gradient(135deg, #fff 25%, transparent 25%, transparent 50%, #fff 50%, #fff 75%, transparent 75%, transparent);
  background-image: -moz-linear-gradient(135deg, #fff 25%, transparent 25%, transparent 50%, #fff 50%, #fff 75%, transparent 75%, transparent);
  background-image: linear-gradient(135deg, #fff 25%, transparent 25%, transparent 50%, #fff 50%, #fff 75%, transparent 75%, transparent);
  background-size: 1.42857143em 1.42857143em;
  -webkit-animation: oo-ui-pendingElement-stripes 650ms linear infinite;
  -moz-animation: oo-ui-pendingElement-stripes 650ms linear infinite;
  animation: oo-ui-pendingElement-stripes 650ms linear infinite;
}
@-webkit-keyframes oo-ui-pendingElement-stripes {
  0% {
    background-position: -1.42857143em 0;
  }
  100% {
    background-position: 0 0;
  }
}
@-moz-keyframes oo-ui-pendingElement-stripes {
  0% {
    background-position: -1.42857143em 0;
  }
  100% {
    background-position: 0 0;
  }
}
@keyframes oo-ui-pendingElement-stripes {
  0% {
    background-position: -1.42857143em 0;
  }
  100% {
    background-position: 0 0;
  }
}





.oo-ui-fieldLayout {
  display: block;
  margin-top: 16px;
}
.oo-ui-fieldLayout:before,
.oo-ui-fieldLayout:after {
  content: ' ';
  display: table;
}
.oo-ui-fieldLayout:after {
  clear: both;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
  word-wrap: break-word;
  -webkit-hyphens: auto;
  -moz-hyphens: auto;
  -ms-hyphens: auto;
  hyphens: auto;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-help,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-help,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
  display: block;
  float: right;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
  text-align: left;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline {
  word-wrap: break-word;
  -webkit-hyphens: auto;
  -moz-hyphens: auto;
  -ms-hyphens: auto;
  hyphens: auto;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body {
  display: table;
  width: 100%;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
  display: table-cell;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
  vertical-align: middle;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
  width: 1px;
  vertical-align: top;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-top > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-top > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
  display: block;
}
.oo-ui-fieldLayout .oo-ui-fieldLayout-help {
  float: left;
}
.oo-ui-fieldLayout .oo-ui-fieldLayout-help:not( .oo-ui-popupButtonWidget ) > .oo-ui-buttonElement-button {
  cursor: help;
}
.oo-ui-fieldLayout.oo-ui-labelElement,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline {
  margin-top: 12px;
}
.oo-ui-fieldLayout:first-child,
.oo-ui-fieldLayout.oo-ui-labelElement:first-child,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline:first-child {
  margin-top: 0;
}
.oo-ui-fieldLayout.oo-ui-labelElement > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
  padding-bottom: 4px;
}
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-top > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body {
  max-width: 50em;
}
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header,
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  width: 40%;
  padding-left: 2.64285714em;
}
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header > .oo-ui-labelElement-label,
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header > .oo-ui-labelElement-label {
  display: block;
  padding-top: 4px;
}
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-help,
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-help {
  margin-left: 0;
  margin-right: -2.35714286em;
}
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-left > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field,
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-right > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-field {
  width: 60%;
}
.oo-ui-fieldLayout.oo-ui-labelElement.oo-ui-fieldLayout-align-inline > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header {
  padding-top: 0;
  padding-bottom: 0;
  padding-right: 6px;
}
.oo-ui-fieldLayout .oo-ui-fieldLayout-help {
  margin-left: 0;
}
.oo-ui-fieldLayout .oo-ui-fieldLayout-help .oo-ui-buttonElement-button {
  padding-top: 1.42857143em;
  padding-left: 0;
}
.oo-ui-fieldLayout .oo-ui-fieldLayout-help .oo-ui-buttonElement-button:hover,
.oo-ui-fieldLayout .oo-ui-fieldLayout-help .oo-ui-buttonElement-button:active {
  background-color: transparent;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-top > .oo-ui-fieldLayout-body > .oo-ui-inline-help {
  margin-top: 4px;
}
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-top .oo-ui-fieldLayout-help,
.oo-ui-fieldLayout.oo-ui-fieldLayout-align-inline .oo-ui-fieldLayout-help {
  margin-top: -6px;
  margin-left: -8px;
  margin-right: 0;
}
.oo-ui-fieldLayout-messages {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  max-width: 50em;
  padding: 4px 0;
}
.oo-ui-fieldLayout-messages > .oo-ui-messageWidget {
  margin-right: 12px;
  margin-left: 12px;
}
.oo-ui-fieldLayout-messages > .oo-ui-messageWidget:first-child {
  margin-top: 4px;
}
.oo-ui-fieldLayout-disabled > .oo-ui-fieldLayout-body > .oo-ui-fieldLayout-header > .oo-ui-labelElement-label {
  color: #72777d;
}

.oo-ui-actionFieldLayout-input,
.oo-ui-actionFieldLayout-button {
  display: table-cell;
  vertical-align: middle;
}
.oo-ui-actionFieldLayout-button {
  width: 1%;
  white-space: nowrap;
}
.oo-ui-actionFieldLayout.oo-ui-fieldLayout-align-top {
  max-width: 50em;
}
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input .oo-ui-widget:not( .oo-ui-textInputWidget ) {
  margin-left: 8px;
}
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input .oo-ui-widget.oo-ui-textInputWidget > .oo-ui-inputWidget-input {
  border-radius: 0 2px 2px 0;
  position: relative;
}
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-button .oo-ui-buttonElement-framed > .oo-ui-buttonElement-button {
  border-radius: 2px 0 0 2px;
  margin-right: -1px;
}
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-button .oo-ui-buttonElement-frameless {
  margin-right: 6px;
}
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input > .oo-ui-textInputWidget > .oo-ui-inputWidget-input:hover,
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input > .oo-ui-textInputWidget > .oo-ui-inputWidget-input:focus,
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input > .oo-ui-textInputWidget.oo-ui-flaggedElement-invalid > .oo-ui-inputWidget-input,
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input > .oo-ui-textInputWidget > .oo-ui-inputWidget-input:hover ~ *,
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input > .oo-ui-textInputWidget > .oo-ui-inputWidget-input:focus ~ *,
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-input > .oo-ui-textInputWidget.oo-ui-flaggedElement-invalid > .oo-ui-inputWidget-input ~ * {
  z-index: 1;
}
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-button > .oo-ui-buttonElement > .oo-ui-buttonElement-button:hover,
.oo-ui-actionFieldLayout .oo-ui-actionFieldLayout-button > .oo-ui-buttonElement > .oo-ui-buttonElement-button:focus {
  z-index: 1;
}

.oo-ui-fieldsetLayout {
  position: relative;
  min-width: 0;
  margin: 0;
  border: 0;
  padding: 0.01px 0 0 0;
}
body:not( :-moz-handler-blocked ) .oo-ui-fieldsetLayout {
  display: table-cell;
}
.oo-ui-fieldsetLayout > .oo-ui-fieldsetLayout-header {
  display: none;
}
.oo-ui-fieldsetLayout.oo-ui-iconElement > .oo-ui-fieldsetLayout-header,
.oo-ui-fieldsetLayout.oo-ui-labelElement > .oo-ui-fieldsetLayout-header {
  color: inherit;
  display: inline-table;
  box-sizing: border-box;
  padding: 0;
  white-space: normal;
  float: right;
  width: 100%;
}
.oo-ui-fieldsetLayout > .oo-ui-inline-help {
  clear: right;
}
.oo-ui-fieldsetLayout-group {
  clear: both;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help {
  float: left;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help:not( .oo-ui-popupButtonWidget ) > .oo-ui-buttonElement-button {
  cursor: help;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-header {
  max-width: 50em;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-header .oo-ui-iconElement-icon {
  height: 1.42857143em;
}
.oo-ui-fieldsetLayout.oo-ui-iconElement > .oo-ui-fieldsetLayout-header .oo-ui-iconElement-icon {
  display: block;
}
.oo-ui-fieldsetLayout + .oo-ui-fieldsetLayout,
.oo-ui-fieldsetLayout + .oo-ui-formLayout {
  margin-top: 24px;
}
.oo-ui-fieldsetLayout.oo-ui-labelElement > .oo-ui-fieldsetLayout-header > .oo-ui-labelElement-label {
  display: inline-block;
  margin-bottom: 8px;
  font-size: 1.14285714em;
  font-weight: bold;
}
.oo-ui-fieldsetLayout.oo-ui-iconElement > .oo-ui-fieldsetLayout-header > .oo-ui-labelElement-label {
  padding-right: 1.625em;
}
.oo-ui-fieldsetLayout > .oo-ui-inline-help {
  margin-bottom: 8px;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help,
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help:last-child {
  margin-left: -8px;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help .oo-ui-buttonElement-button {
  padding-top: 1.42857143em;
  padding-left: 0;
}
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help .oo-ui-buttonElement-button:hover,
.oo-ui-fieldsetLayout .oo-ui-fieldsetLayout-help .oo-ui-buttonElement-button:active {
  background-color: transparent;
}

.oo-ui-formLayout + .oo-ui-fieldsetLayout,
.oo-ui-formLayout + .oo-ui-formLayout {
  margin-top: 24px;
}

.oo-ui-panelLayout {
  position: relative;
}
.oo-ui-panelLayout-scrollable {
  overflow: auto;
  -webkit-overflow-scrolling: touch;
}
.oo-ui-panelLayout-expanded {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  bottom: 0;
}
.oo-ui-panelLayout-padded {
  padding: 12px 16px 16px;
}
.oo-ui-panelLayout-padded.oo-ui-formLayout > .oo-ui-fieldsetLayout .oo-ui-labelElement-label,
.oo-ui-panelLayout-padded.oo-ui-formLayout > .oo-ui-fieldsetLayout .oo-ui-iconElement-icon {
  margin-top: -6px;
}
.oo-ui-panelLayout-framed {
  border: 1px solid #a2a9b1;
  border-radius: 2px;
}
.oo-ui-panelLayout-padded.oo-ui-panelLayout-framed {
  margin: 12px 0;
}

.oo-ui-horizontalLayout > .oo-ui-widget {
  display: inline-block;
  vertical-align: middle;
}
.oo-ui-horizontalLayout > .oo-ui-layout {
  display: inline-block;
}
.oo-ui-horizontalLayout > .oo-ui-layout,
.oo-ui-horizontalLayout > .oo-ui-widget {
  margin-left: 8px;
}
.oo-ui-horizontalLayout > .oo-ui-layout:last-child,
.oo-ui-horizontalLayout > .oo-ui-widget:last-child {
  margin-left: 0;
}
.oo-ui-horizontalLayout > .oo-ui-layout {
  margin-top: 0;
}
.oo-ui-horizontalLayout > .oo-ui-widget {
  margin-bottom: 8px;
}



.oo-ui-optionWidget {
  position: relative;
  display: block;
}
.oo-ui-optionWidget.oo-ui-widget-enabled {
  cursor: pointer;
}
.oo-ui-optionWidget.oo-ui-widget-disabled {
  cursor: default;
}
.oo-ui-optionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  display: block;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
}
.oo-ui-optionWidget-selected .oo-ui-buttonElement-button > .oo-ui-iconElement-icon {
  opacity: 1;
}
.oo-ui-optionWidget.oo-ui-widget-disabled {
  color: #72777d;
}

.oo-ui-decoratedOptionWidget {
  padding: 7px 12px;
  line-height: 1;
}
.oo-ui-decoratedOptionWidget.oo-ui-iconElement {
  padding-right: 2.64285714em;
}
.oo-ui-decoratedOptionWidget .oo-ui-iconElement-icon {
  right: 0.78571429em;
}
.oo-ui-decoratedOptionWidget .oo-ui-labelElement-label {
  line-height: 1.28571429em;
}
.oo-ui-decoratedOptionWidget.oo-ui-indicatorElement {
  padding-left: 2.14285714em;
}
.oo-ui-decoratedOptionWidget .oo-ui-indicatorElement-indicator {
  left: 12px;
}
.oo-ui-decoratedOptionWidget.oo-ui-widget-enabled:hover .oo-ui-iconElement-icon,
.oo-ui-decoratedOptionWidget.oo-ui-widget-enabled:hover .oo-ui-indicatorElement-indicator {
  opacity: 0.74;
}
.oo-ui-decoratedOptionWidget.oo-ui-widget-enabled .oo-ui-iconElement-icon,
.oo-ui-decoratedOptionWidget.oo-ui-widget-enabled .oo-ui-indicatorElement-indicator {
  opacity: 0.87;
  -webkit-transition: opacity 100ms;
  -moz-transition: opacity 100ms;
  transition: opacity 100ms;
}
.oo-ui-decoratedOptionWidget.oo-ui-widget-disabled .oo-ui-iconElement-icon,
.oo-ui-decoratedOptionWidget.oo-ui-widget-disabled .oo-ui-indicatorElement-indicator {
  opacity: 0.51;
}

.oo-ui-radioSelectWidget:focus {
  outline: 0;
}
.oo-ui-radioSelectWidget:focus [type='radio']:checked + span:before {
  border-color: #fff;
}

.oo-ui-radioOptionWidget {
  display: table;
  padding: 4px 0;
}
.oo-ui-radioOptionWidget .oo-ui-radioInputWidget,
.oo-ui-radioOptionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  display: table-cell;
  vertical-align: top;
}
.oo-ui-radioOptionWidget .oo-ui-radioInputWidget {
  width: 1px;
}
.oo-ui-radioOptionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  white-space: normal;
}
.oo-ui-radioOptionWidget:first-child {
  padding-top: 0;
}
.oo-ui-radioOptionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  padding-right: 6px;
}
.oo-ui-radioOptionWidget .oo-ui-radioInputWidget {
  margin-left: 0;
}

.oo-ui-labelWidget {
  display: inline-block;
}
.oo-ui-labelWidget.oo-ui-inline-help {
  display: block;
  color: #54595d;
  font-size: 0.92857143em;
}

.oo-ui-messageWidget {
  position: relative;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  font-weight: bold;
}
.oo-ui-messageWidget > .oo-ui-labelElement-label {
  display: block;
}
.oo-ui-messageWidget > .oo-ui-iconElement-icon {
  background-position: 0 0;
}
.oo-ui-messageWidget > .oo-ui-labelElement-label {
  margin-right: 2em;
}
.oo-ui-messageWidget.oo-ui-messageWidget-block {
  border: 1px solid;
  padding: 16px 24px;
  font-weight: normal;
}
.oo-ui-messageWidget.oo-ui-messageWidget-block > .oo-ui-iconElement-icon {
  background-position: 0 16px;
}
.oo-ui-messageWidget.oo-ui-messageWidget-block.oo-ui-flaggedElement-error {
  background-color: #fee7e6;
  border-color: #d33;
}
.oo-ui-messageWidget.oo-ui-messageWidget-block.oo-ui-flaggedElement-warning {
  background-color: #fef6e7;
  border-color: #fc3;
}
.oo-ui-messageWidget.oo-ui-messageWidget-block.oo-ui-flaggedElement-success {
  background-color: #d5fdf4;
  border-color: #14866d;
}
.oo-ui-messageWidget.oo-ui-messageWidget-block.oo-ui-flaggedElement-notice {
  background-color: #eaecf0;
  border-color: #a2a9b1;
}
.oo-ui-messageWidget.oo-ui-flaggedElement-error:not( .oo-ui-messageWidget-block ) {
  color: #d33;
}
.oo-ui-messageWidget.oo-ui-flaggedElement-success:not( .oo-ui-messageWidget-block ) {
  color: #14866d;
}
.oo-ui-messageWidget + .oo-ui-messageWidget {
  margin-top: 8px;
}

.oo-ui-iconWidget {
  vertical-align: middle;
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  clip: auto;
  margin: 0;
  text-indent: -9999px;
  line-height: 2.5;
  display: inline-block;
  position: static;
  top: auto;
  height: 1.42857143em;
}
.oo-ui-iconWidget.oo-ui-widget-disabled {
  opacity: 0.51;
}

.oo-ui-indicatorWidget {
  vertical-align: middle;
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  clip: auto;
  margin: 0;
  text-indent: -9999px;
  line-height: 2.5;
  margin: 0.42857143em;
  display: inline-block;
  position: static;
  top: auto;
  height: 0.85714286em;
}
.oo-ui-indicatorWidget.oo-ui-widget-disabled {
  opacity: 0.51;
}

.oo-ui-buttonWidget {
  margin-left: 8px;
}
.oo-ui-buttonWidget:last-child {
  margin-left: 0;
}

.oo-ui-buttonGroupWidget {
  display: inline-block;
  border-radius: 2px;
  margin-left: 8px;
  z-index: 0;
  position: relative;
  padding-bottom: 1px;
}
.oo-ui-buttonGroupWidget .oo-ui-buttonWidget.oo-ui-buttonElement-active .oo-ui-buttonElement-button {
  cursor: default;
}
.oo-ui-buttonGroupWidget:last-child {
  margin-left: 0;
}
.oo-ui-buttonGroupWidget .oo-ui-buttonElement {
  margin-left: 0;
  z-index: 0;
}
.oo-ui-buttonGroupWidget .oo-ui-buttonElement-framed .oo-ui-buttonElement-button {
  margin-left: -1px;
  margin-bottom: -1px;
  border-radius: 0;
}
.oo-ui-buttonGroupWidget .oo-ui-buttonElement-framed:first-child .oo-ui-buttonElement-button {
  border-bottom-right-radius: 2px;
  border-top-right-radius: 2px;
}
.oo-ui-buttonGroupWidget .oo-ui-buttonElement-framed:last-child .oo-ui-buttonElement-button {
  margin-left: 0;
  border-bottom-left-radius: 2px;
  border-top-left-radius: 2px;
}
.oo-ui-buttonGroupWidget .oo-ui-buttonElement-framed.oo-ui-widget-disabled + .oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  border-right-color: #fff;
}
.oo-ui-buttonGroupWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button:active {
  z-index: 1;
}
.oo-ui-buttonGroupWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button:focus {
  z-index: 2;
}
.oo-ui-buttonGroupWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-buttonElement-active > .oo-ui-buttonElement-button {
  z-index: 3;
}
.oo-ui-buttonGroupWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  z-index: -1;
}
.oo-ui-buttonGroupWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-toggleWidget-on + .oo-ui-toggleWidget-on > .oo-ui-buttonElement-button,
.oo-ui-buttonGroupWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-toggleWidget-on + .oo-ui-toggleWidget-on > .oo-ui-buttonElement-button:active {
  border-right-color: #a2a9b1;
  z-index: 3;
}

.oo-ui-popupWidget {
  position: absolute;
  z-index: 1;
}
.oo-ui-popupWidget-popup {
  position: relative;
  overflow: hidden;
  word-wrap: break-word;
  overflow-wrap: break-word;
}
.oo-ui-popupWidget-anchor {
  display: none;
}
.oo-ui-popupWidget-anchored .oo-ui-popupWidget-anchor {
  display: block;
  position: absolute;
  background-repeat: no-repeat;
}
.oo-ui-popupWidget-anchored .oo-ui-popupWidget-anchor:before,
.oo-ui-popupWidget-anchored .oo-ui-popupWidget-anchor:after {
  content: '';
  position: absolute;
  width: 0;
  height: 0;
  border-style: solid;
  border-color: transparent;
}
.oo-ui-popupWidget-anchored-top .oo-ui-popupWidget-anchor {
  right: 0;
}
.oo-ui-popupWidget-anchored-top .oo-ui-popupWidget-anchor:before,
.oo-ui-popupWidget-anchored-top .oo-ui-popupWidget-anchor:after {
  border-top: 0;
}
.oo-ui-popupWidget-anchored-bottom .oo-ui-popupWidget-anchor {
  right: 0;
}
.oo-ui-popupWidget-anchored-bottom .oo-ui-popupWidget-anchor:before,
.oo-ui-popupWidget-anchored-bottom .oo-ui-popupWidget-anchor:after {
  border-bottom: 0;
}
.oo-ui-popupWidget-anchored-start .oo-ui-popupWidget-anchor {
  top: 0;
}
.oo-ui-popupWidget-anchored-start .oo-ui-popupWidget-anchor:before,
.oo-ui-popupWidget-anchored-start .oo-ui-popupWidget-anchor:after {
  border-right: 0;
}
.oo-ui-popupWidget-anchored-end .oo-ui-popupWidget-anchor {
  top: 0;
}
.oo-ui-popupWidget-anchored-end .oo-ui-popupWidget-anchor:before,
.oo-ui-popupWidget-anchored-end .oo-ui-popupWidget-anchor:after {
  border-left: 0;
}
.oo-ui-popupWidget-head {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.oo-ui-popupWidget-head > .oo-ui-buttonWidget {
  position: absolute;
}
.oo-ui-popupWidget-head > .oo-ui-iconElement-icon {
  float: right;
}
.oo-ui-popupWidget-head > .oo-ui-labelElement-label {
  float: right;
  cursor: default;
}
.oo-ui-popupWidget-body {
  clear: both;
}
.oo-ui-popupWidget-body.oo-ui-clippableElement-clippable {
  min-height: 1em;
}
.oo-ui-popupWidget-popup {
  background-color: #fff;
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.25);
}
@supports ( filter: drop-shadow( 0 0 0 ) ) {
  .oo-ui-popupWidget {
    filter: drop-shadow(0 2px 1px rgba(0, 0, 0, 0.3));
    -webkit-transform: translateZ(0);
    transform: translateZ(0);
  }
  .oo-ui-popupWidget-popup {
    box-shadow: none;
  }
}
.oo-ui-popupWidget-anchored-top {
  margin-top: 9px;
}
.oo-ui-popupWidget-anchored-top .oo-ui-popupWidget-anchor {
  top: -9px;
}
.oo-ui-popupWidget-anchored-top .oo-ui-popupWidget-anchor:before {
  bottom: -10px;
  right: -9px;
  border-bottom-color: #7b8590;
  border-width: 10px;
}
.oo-ui-popupWidget-anchored-top .oo-ui-popupWidget-anchor:after {
  bottom: -10px;
  right: -8px;
  border-bottom-color: #fff;
  border-width: 9px;
}
.oo-ui-popupWidget-anchored-bottom {
  margin-bottom: 9px;
}
.oo-ui-popupWidget-anchored-bottom .oo-ui-popupWidget-anchor {
  bottom: -9px;
}
.oo-ui-popupWidget-anchored-bottom .oo-ui-popupWidget-anchor:before {
  top: -10px;
  right: -9px;
  border-top-color: #a2a9b1;
  border-width: 10px;
}
.oo-ui-popupWidget-anchored-bottom .oo-ui-popupWidget-anchor:after {
  top: -10px;
  right: -8px;
  border-top-color: #fff;
  border-width: 9px;
}
.oo-ui-popupWidget-anchored-start {
  margin-right: 9px;
}
.oo-ui-popupWidget-anchored-start .oo-ui-popupWidget-anchor {
  right: -9px;
}
.oo-ui-popupWidget-anchored-start .oo-ui-popupWidget-anchor:before {
  left: -10px;
  top: -9px;
  border-left-color: #a2a9b1;
  border-width: 10px;
}
.oo-ui-popupWidget-anchored-start .oo-ui-popupWidget-anchor:after {
  left: -10px;
  top: -8px;
  border-left-color: #fff;
  border-width: 9px;
}
.oo-ui-popupWidget-anchored-end {
  margin-left: 9px;
}
.oo-ui-popupWidget-anchored-end .oo-ui-popupWidget-anchor {
  left: -9px;
}
.oo-ui-popupWidget-anchored-end .oo-ui-popupWidget-anchor:before {
  right: -10px;
  top: -9px;
  border-right-color: #a2a9b1;
  border-width: 10px;
}
.oo-ui-popupWidget-anchored-end .oo-ui-popupWidget-anchor:after {
  right: -10px;
  top: -8px;
  border-right-color: #fff;
  border-width: 9px;
}
.oo-ui-popupWidget-transitioning .oo-ui-popupWidget-popup {
  -webkit-transition: width 100ms, height 100ms, right 100ms;
  -moz-transition: width 100ms, height 100ms, right 100ms;
  transition: width 100ms, height 100ms, right 100ms;
}
.oo-ui-popupWidget-head {
  position: relative;
  overflow: auto;
}
.oo-ui-popupWidget-head > .oo-ui-iconElement-icon {
  right: 0.78571429em;
}
.oo-ui-popupWidget-head > .oo-ui-iconElement-noIcon + .oo-ui-labelElement-label {
  margin-right: 12px;
}
.oo-ui-popupWidget-head > .oo-ui-labelElement-label {
  margin: 9px 2.64285714em 9px 12px;
  line-height: 1.28571429em;
}
.oo-ui-popupWidget-head > .oo-ui-buttonWidget {
  left: 0;
}
.oo-ui-popupWidget-head > .oo-ui-buttonWidget .oo-ui-icon-close {
  background-size: 1.14285714em 1.14285714em;
}
.oo-ui-popupWidget-body {
  line-height: 1.42857143em;
}
.oo-ui-popupWidget-body-padded {
  margin: 0 12px 6px;
}
.oo-ui-popupWidget-body-padded > :first-child {
  margin-top: 0;
}
.oo-ui-popupWidget-footer {
  margin: 9px 12px;
}

.oo-ui-popupButtonWidget {
  position: relative;
}
.oo-ui-popupButtonWidget .oo-ui-popupWidget {
  cursor: auto;
}

.oo-ui-inputWidget {
  margin-left: 8px;
}
.oo-ui-inputWidget:last-child {
  margin-left: 0;
}

.oo-ui-buttonInputWidget > button,
.oo-ui-buttonInputWidget > input {
  background-color: transparent;
  margin: 0;
  border: 0;
  padding: 0;
}

.oo-ui-checkboxInputWidget {
  display: inline-block;
  z-index: 0;
  position: relative;
  line-height: 1.42857143em;
  white-space: nowrap;
}
.oo-ui-checkboxInputWidget * {
  font: inherit;
  vertical-align: middle;
}
.oo-ui-checkboxInputWidget [type='checkbox'] {
  position: relative;
  max-width: none;
  width: 1.42857143em;
  height: 1.42857143em;
  margin: 0;
  opacity: 0;
  z-index: 1;
}
.oo-ui-checkboxInputWidget [type='checkbox'] + span {
  background-color: #fff;
  background-size: 0 0;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  position: absolute;
  right: 0;
  width: 1.42857143em;
  height: 1.42857143em;
  border-color: #72777d;
  border-style: solid;
  border-radius: 2px;
  border-width: 0;
}
:root .oo-ui-checkboxInputWidget [type='checkbox'] + span {
  border-width: 1px;
}
.oo-ui-checkboxInputWidget [type='checkbox']:checked:not( :indeterminate ) + span {
  background-size: 1em 1em;
}
.oo-ui-checkboxInputWidget [type='checkbox']:indeterminate + span:before {
  content: ' ';
  background-color: #fff;
  position: absolute;
  top: 50%;
  right: 0.21428571em;
  left: 0.21428571em;
  height: 2px;
  margin-top: -1px;
}
.oo-ui-checkboxInputWidget [type='checkbox']:disabled + span {
  background-color: #c8ccd1;
  border-color: #c8ccd1;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox'] {
  cursor: pointer;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox'] + span {
  cursor: pointer;
  -webkit-transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
  -moz-transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
  transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:focus + span {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
  outline: 1px solid transparent;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:hover + span {
  border-color: #447ff5;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:active + span {
  background-color: #2a4b8d;
  border-color: #2a4b8d;
  box-shadow: inset 0 0 0 1px #2a4b8d;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:checked + span,
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:indeterminate + span {
  background-color: #36c;
  border-color: #36c;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:checked:focus + span,
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:indeterminate:focus + span {
  background-color: #36c;
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c, inset 0 0 0 2px #fff;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:checked:hover + span,
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:indeterminate:hover + span {
  background-color: #447ff5;
  border-color: #447ff5;
}
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:checked:active + span,
.oo-ui-checkboxInputWidget.oo-ui-widget-enabled [type='checkbox']:indeterminate:active + span {
  background-color: #2a4b8d;
  border-color: #2a4b8d;
  box-shadow: inset 0 0 0 1px #2a4b8d;
}

.oo-ui-checkboxMultiselectInputWidget .oo-ui-fieldLayout {
  margin-top: 0;
  padding: 4px 0;
}
.oo-ui-checkboxMultiselectInputWidget .oo-ui-fieldLayout:first-child {
  padding-top: 0;
}

.oo-ui-dropdownInputWidget {
  position: relative;
  vertical-align: middle;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  width: 100%;
  max-width: 50em;
}
.oo-ui-dropdownInputWidget .oo-ui-dropdownWidget,
.oo-ui-dropdownInputWidget.oo-ui-dropdownInputWidget-php select,
.oo-ui-dropdownInputWidget.oo-ui-isMobile select {
  display: block;
}
.oo-ui-dropdownInputWidget.oo-ui-isMobile .oo-ui-dropdownWidget {
  display: none;
}
.oo-ui-dropdownInputWidget select {
  display: none;
  background-position: -9999em 0;
  background-repeat: no-repeat;
  width: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled select {
  cursor: pointer;
}
.oo-ui-dropdownInputWidget select {
  -webkit-appearance: none;
  -moz-appearance: none;
  background-color: transparent;
  background-position: left 12px center;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  height: 2.28571429em;
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  padding-right: 12px;
  padding-left: 2.14285714em;
  font-size: inherit;
  font-family: inherit;
  vertical-align: middle;
  background-position: -9999em 0\9;
  padding: 0\9;
  /* Support IE 10: Move it in again and also add %60padding`. Because we talk to morons. */
}
@media screen and (-ms-high-contrast: active), (-ms-high-contrast: none) {
  .oo-ui-dropdownInputWidget select {
    background-position: left 12px center;
    padding-right: 12px;
    padding-left: 2.14285714em;
  }
}
.oo-ui-dropdownInputWidget select::-ms-expand {
  display: none;
}
.oo-ui-dropdownInputWidget option {
  background-color: transparent;
  font-size: inherit;
  font-family: inherit;
  height: 1.5em;
  padding: 6px 12px;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled {
  background-color: #f8f9fa;
  -webkit-transition: background-color 100ms;
  -moz-transition: background-color 100ms;
  transition: background-color 100ms;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled:hover {
  background-color: #fff;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled select {
  color: #202122;
  -webkit-transition: border-color 100ms, box-shadow 100ms;
  -moz-transition: border-color 100ms, box-shadow 100ms;
  transition: border-color 100ms, box-shadow 100ms;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled select:hover {
  color: #404244;
  border-color: #a2a9b1;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled select:active {
  color: #000;
  border-color: #72777d;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-enabled select:focus {
  border-color: #36c;
  outline: 0;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-disabled {
  background-color: #eaecf0;
}
.oo-ui-dropdownInputWidget.oo-ui-widget-disabled select {
  color: #72777d;
  border-color: #c8ccd1;
}

.oo-ui-radioInputWidget {
  display: inline-block;
  z-index: 0;
  position: relative;
  line-height: 1.42857143em;
  white-space: nowrap;
}
.oo-ui-radioInputWidget * {
  font: inherit;
  vertical-align: middle;
}
.oo-ui-radioInputWidget [type='radio'] {
  position: relative;
  max-width: none;
  width: 1.42857143em;
  height: 1.42857143em;
  margin: 0;
  opacity: 0;
  z-index: 1;
}
.oo-ui-radioInputWidget [type='radio'] + span {
  background-color: #fff;
  position: absolute;
  right: 0;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  width: 1.42857143em;
  height: 1.42857143em;
  border-color: #72777d;
  border-style: solid;
  border-radius: 100%;
  border-width: 0;
}
:root .oo-ui-radioInputWidget [type='radio'] + span {
  border-width: 1px;
}
.oo-ui-radioInputWidget [type='radio'] + span:before {
  content: ' ';
  position: absolute;
  top: -4px;
  right: -4px;
  left: -4px;
  bottom: -4px;
  border: 1px solid transparent;
  border-radius: 100%;
}
.oo-ui-radioInputWidget [type='radio']:checked + span,
.oo-ui-radioInputWidget [type='radio']:checked:hover + span,
.oo-ui-radioInputWidget [type='radio']:checked:focus:hover + span {
  border-width: 6px;
}
.oo-ui-radioInputWidget [type='radio']:disabled + span {
  background-color: #c8ccd1;
  border-color: #c8ccd1;
}
.oo-ui-radioInputWidget [type='radio']:disabled:checked + span {
  background-color: #fff;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio'] {
  cursor: pointer;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio'] + span {
  cursor: pointer;
  -webkit-transition: background-color 100ms, border-color 100ms, border-width 100ms;
  -moz-transition: background-color 100ms, border-color 100ms, border-width 100ms;
  transition: background-color 100ms, border-color 100ms, border-width 100ms;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:hover + span {
  border-color: #447ff5;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:active + span {
  background-color: #2a4b8d;
  border-color: #2a4b8d;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:checked + span {
  border-color: #36c;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:checked:focus + span:before {
  border-color: #fff;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:checked:hover + span {
  border-color: #447ff5;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:checked:active + span {
  border-color: #2a4b8d;
  box-shadow: inset 0 0 0 1px #2a4b8d;
}
.oo-ui-radioInputWidget.oo-ui-widget-enabled [type='radio']:checked:active + span:before {
  border-color: #2a4b8d;
}

.oo-ui-radioSelectInputWidget .oo-ui-fieldLayout {
  margin-top: 0;
  padding: 4px 0;
}
.oo-ui-radioSelectInputWidget .oo-ui-fieldLayout:first-child {
  padding-top: 0;
}

.oo-ui-textInputWidget {
  position: relative;
  vertical-align: middle;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  width: 100%;
  max-width: 50em;
}
.oo-ui-textInputWidget .oo-ui-inputWidget-input {
  -webkit-appearance: none;
  -moz-appearance: textfield;
  display: block;
  width: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-textInputWidget input::-ms-clear {
  display: none;
}
.oo-ui-textInputWidget textarea {
  overflow: auto;
}
.oo-ui-textInputWidget textarea.oo-ui-textInputWidget-autosized {
  resize: none;
}
.oo-ui-textInputWidget [type='number']::-webkit-outer-spin-button,
.oo-ui-textInputWidget [type='number']::-webkit-inner-spin-button {
  -webkit-appearance: none;
  margin: 0;
}
.oo-ui-textInputWidget [type='search']::-webkit-search-decoration,
.oo-ui-textInputWidget [type='search']::-webkit-search-cancel-button {
  display: none;
}
.oo-ui-textInputWidget > .oo-ui-iconElement-icon,
.oo-ui-textInputWidget-labelPosition-before > .oo-ui-labelElement-label {
  right: 0;
}
.oo-ui-textInputWidget > .oo-ui-indicatorElement-indicator,
.oo-ui-textInputWidget-labelPosition-after > .oo-ui-labelElement-label {
  left: 0;
}
.oo-ui-textInputWidget > .oo-ui-labelElement-label {
  position: absolute;
  top: 0;
}
.oo-ui-textInputWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  display: block;
}
.oo-ui-textInputWidget-labelPosition-after.oo-ui-labelElement ::-ms-clear {
  display: none;
}
.oo-ui-textInputWidget-php > .oo-ui-iconElement-icon,
.oo-ui-textInputWidget-php > .oo-ui-indicatorElement-indicator,
.oo-ui-textInputWidget-php > .oo-ui-labelElement-label {
  pointer-events: none;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled > .oo-ui-iconElement-icon,
.oo-ui-textInputWidget.oo-ui-widget-enabled > .oo-ui-indicatorElement-indicator {
  cursor: text;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled.oo-ui-textInputWidget-type-search > .oo-ui-indicatorElement-indicator {
  cursor: pointer;
}
.oo-ui-textInputWidget.oo-ui-widget-disabled > * {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.oo-ui-textInputWidget .oo-ui-inputWidget-input {
  background-color: #fff;
  color: #000;
  margin: 0;
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  padding: 6px 8px;
  font-size: inherit;
  font-family: inherit;
  line-height: 1.28571429em;
}
.oo-ui-textInputWidget input {
  height: 2.28571429em;
}
.oo-ui-textInputWidget .oo-ui-pendingElement-pending {
  background-color: #eaecf0;
}
.oo-ui-textInputWidget.oo-ui-iconElement .oo-ui-inputWidget-input {
  padding-right: 2.42857143em;
}
.oo-ui-textInputWidget.oo-ui-iconElement > .oo-ui-iconElement-icon {
  right: 9px;
}
.oo-ui-textInputWidget.oo-ui-iconElement textarea + .oo-ui-iconElement-icon {
  max-height: 2.28571429em;
}
.oo-ui-textInputWidget > .oo-ui-labelElement-label {
  color: #72777d;
  padding: 0 8px 0 12px;
  line-height: 2.28571429em;
}
.oo-ui-textInputWidget.oo-ui-indicatorElement .oo-ui-inputWidget-input {
  padding-left: 28px;
}
.oo-ui-textInputWidget.oo-ui-indicatorElement.oo-ui-textInputWidget-labelPosition-after > .oo-ui-labelElement-label {
  padding-left: 0;
}
.oo-ui-textInputWidget.oo-ui-indicatorElement > .oo-ui-indicatorElement-indicator {
  max-height: 2.28571429em;
  margin-left: 0.71428571em;
}
.oo-ui-textInputWidget-labelPosition-after.oo-ui-indicatorElement > .oo-ui-labelElement-label {
  margin-left: 2.28571429em;
}
.oo-ui-textInputWidget-labelPosition-before.oo-ui-iconElement > .oo-ui-labelElement-label {
  padding-right: 2.42857143em;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input {
  box-shadow: inset 0 0 0 1px transparent;
  -webkit-transition: border-color 250ms, box-shadow 250ms;
  -moz-transition: border-color 250ms, box-shadow 250ms;
  transition: border-color 250ms, box-shadow 250ms;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input::-webkit-input-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input:-ms-input-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input::-moz-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input:-moz-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input::placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input:focus {
  outline: 0;
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input:focus ~ .oo-ui-iconElement-icon,
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input:focus ~ .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled .oo-ui-inputWidget-input[readonly] {
  background-color: #f8f9fa;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled:hover .oo-ui-inputWidget-input {
  border-color: #72777d;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled:hover .oo-ui-inputWidget-input:focus {
  border-color: #36c;
}
@media screen and (min-width: 0) {
  .oo-ui-textInputWidget.oo-ui-widget-enabled textarea.oo-ui-inputWidget-input:focus {
    outline: 1px solid #36c;
    outline-offset: -2px;
  }
  .oo-ui-textInputWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid textarea.oo-ui-inputWidget-input:focus {
    outline-color: #d33;
  }
}
.oo-ui-textInputWidget.oo-ui-widget-enabled > .oo-ui-iconElement-icon {
  opacity: 0.67;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled > .oo-ui-indicatorElement-indicator {
  opacity: 0.87;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid .oo-ui-inputWidget-input {
  border-color: #d33;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid .oo-ui-inputWidget-input:hover {
  border-color: #d33;
}
.oo-ui-textInputWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid .oo-ui-inputWidget-input:focus {
  border-color: #d33;
  box-shadow: inset 0 0 0 1px #d33;
}
.oo-ui-textInputWidget.oo-ui-widget-disabled .oo-ui-inputWidget-input {
  background-color: #eaecf0;
  -webkit-text-fill-color: #72777d;
  color: #72777d;
  text-shadow: 0 1px 1px #fff;
  border-color: #c8ccd1;
}
.oo-ui-textInputWidget.oo-ui-widget-disabled > .oo-ui-iconElement-icon,
.oo-ui-textInputWidget.oo-ui-widget-disabled > .oo-ui-indicatorElement-indicator {
  opacity: 0.51;
}
.oo-ui-textInputWidget.oo-ui-widget-disabled > .oo-ui-labelElement-label {
  color: #72777d;
  text-shadow: 0 1px 1px #fff;
}

.oo-ui-menuSelectWidget {
  position: absolute;
  width: 100%;
  z-index: 4;
  background-color: #fff;
  margin-top: -1px;
  margin-bottom: -1px;
  border: 1px solid #a2a9b1;
  border-radius: 0 0 2px 2px;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.25);
}
.oo-ui-menuSelectWidget.oo-ui-clippableElement-clippable {
  min-height: 32px;
}
.oo-ui-menuSelectWidget-invisible {
  display: none;
}

.oo-ui-menuOptionWidget {
  -webkit-transition: background-color 100ms, color 100ms;
  -moz-transition: background-color 100ms, color 100ms;
  transition: background-color 100ms, color 100ms;
}
.oo-ui-menuOptionWidget-checkIcon {
  display: none;
}
.oo-ui-menuOptionWidget.oo-ui-optionWidget-highlighted {
  background-color: #eaecf0;
  color: #000;
}
.oo-ui-menuOptionWidget.oo-ui-optionWidget-selected {
  background-color: #eaf3ff;
  color: #36c;
}
.oo-ui-menuOptionWidget.oo-ui-optionWidget-selected.oo-ui-menuOptionWidget.oo-ui-optionWidget-highlighted,
.oo-ui-menuOptionWidget.oo-ui-optionWidget-pressed.oo-ui-menuOptionWidget.oo-ui-optionWidget-highlighted {
  background-color: rgba(41, 98, 204, 0.1);
  color: #36c;
}
.oo-ui-menuOptionWidget.oo-ui-widget-enabled.oo-ui-optionWidget {
  color: #202122;
}

.oo-ui-menuSectionOptionWidget {
  color: #72777d;
  padding: 6px 12px 4px;
  font-weight: bold;
}
.oo-ui-menuSectionOptionWidget.oo-ui-widget-enabled {
  cursor: default;
}
.oo-ui-menuSectionOptionWidget ~ .oo-ui-menuOptionWidget {
  padding-right: 24px;
}
.oo-ui-menuSectionOptionWidget ~ .oo-ui-menuOptionWidget.oo-ui-iconElement {
  padding-right: 3.5em;
}
.oo-ui-menuSectionOptionWidget ~ .oo-ui-menuOptionWidget.oo-ui-iconElement .oo-ui-iconElement-icon {
  right: 1.71428571em;
}

.oo-ui-dropdownWidget {
  display: inline-block;
  position: relative;
  vertical-align: middle;
  width: 100%;
  max-width: 50em;
  margin-left: 8px;
}
.oo-ui-dropdownWidget-handle {
  position: relative;
  width: 100%;
  display: block;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  cursor: default;
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-dropdownWidget-handle .oo-ui-labelElement-label {
  display: inline-block;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle {
  cursor: pointer;
}
.oo-ui-dropdownWidget:last-child {
  margin-left: 0;
}
.oo-ui-dropdownWidget-handle {
  min-height: 2.28571429em;
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  padding: 6px 12px 6px 2.14285714em;
  line-height: 1;
}
.oo-ui-dropdownWidget-handle .oo-ui-iconElement-icon {
  right: 12px;
}
.oo-ui-dropdownWidget-handle .oo-ui-indicatorElement-indicator {
  left: 11px;
}
.oo-ui-dropdownWidget-handle .oo-ui-labelElement-label {
  line-height: 1.28571429em;
}
.oo-ui-dropdownWidget.oo-ui-iconElement .oo-ui-dropdownWidget-handle {
  padding-right: 2.71428571em;
}
.oo-ui-dropdownWidget.oo-ui-indicatorElement .oo-ui-dropdownWidget-handle {
  padding-left: 2.57142857em;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle {
  background-color: #f8f9fa;
  color: #202122;
  -webkit-transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
  -moz-transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
  transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle:hover {
  background-color: #fff;
  color: #404244;
  border-color: #a2a9b1;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle:hover .oo-ui-iconElement-icon,
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle:hover .oo-ui-indicatorElement-indicator {
  opacity: 0.74;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle:active {
  color: #000;
  border-color: #72777d;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle:focus {
  border-color: #36c;
  outline: 0;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle .oo-ui-iconElement-icon,
.oo-ui-dropdownWidget.oo-ui-widget-enabled .oo-ui-dropdownWidget-handle .oo-ui-indicatorElement-indicator {
  opacity: 0.87;
  -webkit-transition: opacity 100ms;
  -moz-transition: opacity 100ms;
  transition: opacity 100ms;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled.oo-ui-dropdownWidget-open .oo-ui-dropdownWidget-handle {
  background-color: #fff;
}
.oo-ui-dropdownWidget.oo-ui-widget-enabled.oo-ui-dropdownWidget-open .oo-ui-dropdownWidget-handle .oo-ui-iconElement-icon,
.oo-ui-dropdownWidget.oo-ui-widget-enabled.oo-ui-dropdownWidget-open .oo-ui-dropdownWidget-handle .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-dropdownWidget.oo-ui-widget-disabled .oo-ui-dropdownWidget-handle {
  color: #72777d;
  text-shadow: 0 1px 1px #fff;
  border-color: #c8ccd1;
  background-color: #eaecf0;
}
.oo-ui-dropdownWidget.oo-ui-widget-disabled .oo-ui-dropdownWidget-handle:focus {
  outline: 0;
}
.oo-ui-dropdownWidget.oo-ui-widget-disabled .oo-ui-dropdownWidget-handle .oo-ui-indicatorElement-indicator {
  opacity: 0.15;
}

.oo-ui-comboBoxInputWidget {
  display: inline-block;
  position: relative;
}
.oo-ui-comboBoxInputWidget-field {
  display: table;
  width: 100%;
  table-layout: fixed;
}
.oo-ui-comboBoxInputWidget .oo-ui-inputWidget-input {
  display: table-cell;
  vertical-align: middle;
  position: relative;
  overflow: hidden;
}
.oo-ui-comboBoxInputWidget-dropdownButton {
  display: table-cell;
}
.oo-ui-comboBoxInputWidget-dropdownButton > .oo-ui-buttonElement-button {
  display: block;
  overflow: hidden;
}
.oo-ui-comboBoxInputWidget.oo-ui-comboBoxInputWidget-empty .oo-ui-comboBoxInputWidget-dropdownButton {
  display: none;
}
.oo-ui-comboBoxInputWidget-php ::-webkit-calendar-picker-indicator {
  opacity: 0;
  position: absolute;
  left: 0;
  top: 0;
  width: 2.5em;
  height: 2.5em;
  padding: 0;
}
.oo-ui-comboBoxInputWidget-php > .oo-ui-indicatorWidget {
  display: block;
  position: absolute;
  top: 0;
  height: 100%;
  pointer-events: none;
}
.oo-ui-comboBoxInputWidget .oo-ui-inputWidget-input {
  height: 2.28571429em;
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-left-width: 0;
}
.oo-ui-comboBoxInputWidget.oo-ui-comboBoxInputWidget-empty .oo-ui-inputWidget-input,
.oo-ui-comboBoxInputWidget-php .oo-ui-inputWidget-input {
  border-top-left-radius: 2px;
  border-bottom-left-radius: 2px;
  border-left-width: 1px;
}
.oo-ui-comboBoxInputWidget-dropdownButton.oo-ui-indicatorElement {
  width: 2.64285714em;
}
.oo-ui-comboBoxInputWidget-dropdownButton.oo-ui-indicatorElement .oo-ui-buttonElement-button {
  min-width: 37px;
  padding-right: 0;
}
.oo-ui-comboBoxInputWidget-dropdownButton.oo-ui-indicatorElement .oo-ui-buttonElement-button > .oo-ui-indicatorElement-indicator {
  left: 0.85714286em;
}
.oo-ui-comboBoxInputWidget-dropdownButton.oo-ui-indicatorElement .oo-ui-buttonElement-button,
.oo-ui-comboBoxInputWidget-dropdownButton.oo-ui-indicatorElement .oo-ui-buttonElement-button:focus {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.oo-ui-comboBoxInputWidget-php .oo-ui-indicatorWidget {
  left: 12px;
  margin: 0;
}
.oo-ui-comboBoxInputWidget-open .oo-ui-comboBoxInputWidget-dropdownButton > .oo-ui-buttonElement-button {
  background-color: #fff;
}
.oo-ui-comboBoxInputWidget-open .oo-ui-comboBoxInputWidget-dropdownButton > .oo-ui-buttonElement-button .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-comboBoxInputWidget.oo-ui-widget-disabled .oo-ui-indicatorElement-indicator {
  opacity: 1;
}


.oo-ui-multioptionWidget {
  position: relative;
  display: block;
}
.oo-ui-multioptionWidget.oo-ui-widget-enabled {
  cursor: pointer;
}
.oo-ui-multioptionWidget.oo-ui-widget-disabled {
  cursor: default;
}
.oo-ui-multioptionWidget.oo-ui-labelElement .oo-ui-labelElement-label {
  display: block;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
}
.oo-ui-multioptionWidget.oo-ui-widget-disabled {
  color: #72777d;
}


.oo-ui-checkboxMultioptionWidget {
  display: table;
  padding: 4px 0;
}
.oo-ui-checkboxMultioptionWidget .oo-ui-checkboxInputWidget,
.oo-ui-checkboxMultioptionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  display: table-cell;
  vertical-align: top;
}
.oo-ui-checkboxMultioptionWidget .oo-ui-checkboxInputWidget {
  width: 1px;
}
.oo-ui-checkboxMultioptionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  white-space: normal;
}
.oo-ui-checkboxMultioptionWidget:first-child {
  padding-top: 0;
}
.oo-ui-checkboxMultioptionWidget.oo-ui-labelElement > .oo-ui-labelElement-label {
  padding-right: 6px;
}
.oo-ui-checkboxMultioptionWidget .oo-ui-checkboxInputWidget {
  margin-left: 0;
}

.oo-ui-progressBarWidget {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  max-width: 50em;
  background-color: #fff;
  border: 1px solid #a2a9b1;
  border-radius: 1em;
  box-shadow: 0 1px 1px rgba(0, 0, 0, 0.15);
  overflow: hidden;
}
.oo-ui-progressBarWidget-bar {
  height: 1em;
  -webkit-transition: width 100ms;
  -moz-transition: width 100ms;
  transition: width 100ms;
}
.oo-ui-progressBarWidget-indeterminate .oo-ui-progressBarWidget-bar {
  -webkit-animation: oo-ui-progressBarWidget-slide 2s infinite linear;
  -moz-animation: oo-ui-progressBarWidget-slide 2s infinite linear;
  animation: oo-ui-progressBarWidget-slide 2s infinite linear;
  width: 40%;
  -webkit-transform: translate(25%);
  -moz-transform: translate(25%);
  -ms-transform: translate(25%);
  transform: translate(25%);
}
.oo-ui-progressBarWidget.oo-ui-widget-enabled .oo-ui-progressBarWidget-bar {
  background-color: #36c;
}
.oo-ui-progressBarWidget.oo-ui-widget-disabled .oo-ui-progressBarWidget-bar {
  background-color: #c8ccd1;
}
@-webkit-keyframes oo-ui-progressBarWidget-slide {
  from {
    -webkit-transform: translate(100%);
    -moz-transform: translate(100%);
    -ms-transform: translate(100%);
    transform: translate(100%);
  }
  to {
    -webkit-transform: translate(-350%);
    -moz-transform: translate(-350%);
    -ms-transform: translate(-350%);
    transform: translate(-350%);
  }
}
@-moz-keyframes oo-ui-progressBarWidget-slide {
  from {
    -webkit-transform: translate(100%);
    -moz-transform: translate(100%);
    -ms-transform: translate(100%);
    transform: translate(100%);
  }
  to {
    -webkit-transform: translate(-350%);
    -moz-transform: translate(-350%);
    -ms-transform: translate(-350%);
    transform: translate(-350%);
  }
}
@keyframes oo-ui-progressBarWidget-slide {
  from {
    -webkit-transform: translate(100%);
    -moz-transform: translate(100%);
    -ms-transform: translate(100%);
    transform: translate(100%);
  }
  to {
    -webkit-transform: translate(-350%);
    -moz-transform: translate(-350%);
    -ms-transform: translate(-350%);
    transform: translate(-350%);
  }
}

.oo-ui-numberInputWidget {
  display: inline-block;
  position: relative;
  max-width: 50em;
}
.oo-ui-numberInputWidget-buttoned .oo-ui-buttonWidget,
.oo-ui-numberInputWidget-buttoned .oo-ui-inputWidget-input {
  display: table-cell;
  height: 100%;
}
.oo-ui-numberInputWidget-field {
  display: table;
  table-layout: fixed;
  width: 100%;
}
.oo-ui-numberInputWidget-buttoned .oo-ui-buttonWidget {
  width: 2.64285714em;
}
.oo-ui-numberInputWidget-buttoned .oo-ui-buttonWidget .oo-ui-buttonElement-button {
  display: block;
  min-width: 37px;
  min-height: 2.28571429em;
  padding-right: 0;
  padding-left: 0;
}
.oo-ui-numberInputWidget-buttoned .oo-ui-inputWidget-input {
  border-radius: 0;
  height: 2.28571429em;
}
.oo-ui-numberInputWidget-minusButton > .oo-ui-buttonElement-button {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-left-width: 0;
}
.oo-ui-numberInputWidget-plusButton > .oo-ui-buttonElement-button {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
  border-right-width: 0;
}
.oo-ui-numberInputWidget.oo-ui-widget-disabled.oo-ui-numberInputWidget-buttoned .oo-ui-iconElement-icon {
  opacity: 1;
}

.oo-ui-selectFileInputWidget {
  width: 100%;
  max-width: 50em;
}
.oo-ui-selectFileInputWidget-selectButton > .oo-ui-buttonElement-button {
  position: relative;
  overflow: hidden;
}
.oo-ui-selectFileInputWidget-selectButton > .oo-ui-buttonElement-button > [type='file'] {
  position: absolute;
  top: 0;
  bottom: 0;
  right: 0;
  left: 0;
  width: 100%;
  height: 100%;
  opacity: 0;
  z-index: 1;
  cursor: pointer;
  padding-top: 100px;
}
.oo-ui-selectFileInputWidget-selectButton.oo-ui-widget-disabled > .oo-ui-buttonElement-button > [type='file'] {
  display: none;
}
.oo-ui-selectFileInputWidget-info > .oo-ui-inputWidget-input {
  pointer-events: none;
}
.oo-ui-selectFileInputWidget-empty.oo-ui-widget-enabled .oo-ui-selectFileInputWidget-label {
  cursor: default;
}
.oo-ui-selectFileInputWidget-info > .oo-ui-inputWidget-input {
  height: 2.28571429em;
}

.oo-ui-defaultOverlay {
  position: absolute;
  top: 0;
  /* @noflip */
  left: 0;
}

/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-draggableElement-handle:not( .oo-ui-draggableElement-undraggable ).oo-ui-widget {
  cursor: move;
  cursor: url(images/grab.cur );
  cursor: -webkit-grab;
  cursor: -moz-grab;
  cursor: grab;
}
.oo-ui-draggableElement-handle:not( .oo-ui-draggableElement-undraggable ):active {
  cursor: url(images/grabbing.cur );
  cursor: -webkit-grabbing;
  cursor: -moz-grabbing;
  cursor: grabbing;
}
.oo-ui-draggableElement-handle.oo-ui-widget-disabled,
.oo-ui-widget-disabled .oo-ui-draggableElement-handle {
  cursor: default;
}
.oo-ui-draggableElement-placeholder {
  opacity: 0.2;
}
.oo-ui-draggableGroupElement-horizontal .oo-ui-draggableElement {
  display: inline-block;
}



.oo-ui-bookletLayout-stackLayout.oo-ui-stackLayout-continuous > .oo-ui-panelLayout-scrollable {
  overflow: hidden;
}
.oo-ui-bookletLayout-stackLayout > .oo-ui-panelLayout {
  width: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-bookletLayout-outlinePanel-editable > .oo-ui-outlineSelectWidget {
  height: auto;
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  bottom: 2.5em;
  overflow: auto;
  -webkit-overflow-scrolling: touch;
}
.oo-ui-bookletLayout-outlinePanel > .oo-ui-outlineControlsWidget {
  position: absolute;
  bottom: 0;
  right: 0;
  left: 0;
  height: 2.5em;
}
.oo-ui-bookletLayout-stackLayout > .oo-ui-panelLayout {
  padding: 20px;
}
.oo-ui-bookletLayout-outlinePanel > .oo-ui-outlineControlsWidget {
  border-top: 1px solid #c8ccd1;
}
.oo-ui-bookletLayout > .oo-ui-menuLayout-menu {
  border-left: 1px solid #c8ccd1;
}

.oo-ui-indexLayout {
  /* Use two classes to ensure increased specificity over menuLayout */
}
.oo-ui-indexLayout.oo-ui-menuLayout > .oo-ui-menuLayout-menu {
  height: auto;
}
.oo-ui-indexLayout.oo-ui-menuLayout.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-menu {
  height: 3em;
  white-space: nowrap;
}
.oo-ui-indexLayout.oo-ui-menuLayout.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-content {
  top: 3em;
}
.oo-ui-indexLayout-stackLayout > .oo-ui-panelLayout {
  padding: 20px;
}
.oo-ui-indexLayout.oo-ui-menuLayout.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-menu {
  height: 2.64285714em;
}
.oo-ui-indexLayout.oo-ui-menuLayout.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-content {
  top: 2.64285714em;
}

.oo-ui-menuLayout {
  top: 0;
  right: 0;
  left: 0;
  bottom: 0;
}
.oo-ui-menuLayout-expanded,
.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-menu,
.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-content {
  position: absolute;
}
.oo-ui-menuLayout-expanded > .oo-ui-menuLayout-menu {
  z-index: 1;
}
.oo-ui-menuLayout-static.oo-ui-menuLayout-after,
.oo-ui-menuLayout-static.oo-ui-menuLayout-before {
  display: table;
  width: 100%;
}
.oo-ui-menuLayout-static.oo-ui-menuLayout-after > .oo-ui-menuLayout-menu,
.oo-ui-menuLayout-static.oo-ui-menuLayout-before > .oo-ui-menuLayout-menu,
.oo-ui-menuLayout-static.oo-ui-menuLayout-after > .oo-ui-menuLayout-content,
.oo-ui-menuLayout-static.oo-ui-menuLayout-before > .oo-ui-menuLayout-content {
  display: table-cell;
  vertical-align: top;
}
.oo-ui-menuLayout > .oo-ui-menuLayout-menu,
.oo-ui-menuLayout > .oo-ui-menuLayout-content {
  -webkit-transition: height ease-out 250ms, width ease-out 250ms, top ease-out 250ms, right ease-out 250ms, left ease-out 250ms, bottom ease-out 250ms;
  -moz-transition: height ease-out 250ms, width ease-out 250ms, top ease-out 250ms, right ease-out 250ms, left ease-out 250ms, bottom ease-out 250ms;
  transition: height ease-out 250ms, width ease-out 250ms, top ease-out 250ms, right ease-out 250ms, left ease-out 250ms, bottom ease-out 250ms;
}
.oo-ui-menuLayout > .oo-ui-menuLayout-menu {
  height: 14.28571429em;
  width: 14.28571429em;
}
.oo-ui-menuLayout > .oo-ui-menuLayout-content {
  top: 14.28571429em;
  right: 14.28571429em;
  left: 14.28571429em;
  bottom: 14.28571429em;
}
.oo-ui-menuLayout.oo-ui-menuLayout-hideMenu > .oo-ui-menuLayout-menu {
  width: 0 !important;
  height: 0 !important;
  overflow: hidden;
}
.oo-ui-menuLayout.oo-ui-menuLayout-hideMenu > .oo-ui-menuLayout-content {
  top: 0 !important;
  right: 0 !important;
  left: 0 !important;
  bottom: 0 !important;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-top > .oo-ui-menuLayout-menu {
  width: auto !important;
  right: 0;
  top: 0;
  left: 0;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-top > .oo-ui-menuLayout-content {
  left: 0 !important;
  bottom: 0 !important;
  right: 0 !important;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-after > .oo-ui-menuLayout-menu {
  height: auto !important;
  top: 0;
  left: 0;
  bottom: 0;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-after > .oo-ui-menuLayout-content {
  bottom: 0 !important;
  right: 0 !important;
  top: 0 !important;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-bottom > .oo-ui-menuLayout-menu {
  width: auto !important;
  left: 0;
  bottom: 0;
  right: 0;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-bottom > .oo-ui-menuLayout-content {
  right: 0 !important;
  top: 0 !important;
  left: 0 !important;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-before > .oo-ui-menuLayout-menu {
  height: auto !important;
  bottom: 0;
  right: 0;
  top: 0;
}
.oo-ui-menuLayout.oo-ui-menuLayout-showMenu.oo-ui-menuLayout-before > .oo-ui-menuLayout-content {
  top: 0 !important;
  left: 0 !important;
  bottom: 0 !important;
}



.oo-ui-stackLayout-continuous > .oo-ui-panelLayout {
  display: block;
  position: relative;
}

.oo-ui-buttonSelectWidget {
  display: inline-block;
  border-radius: 2px;
  margin-left: 8px;
  z-index: 0;
  position: relative;
  padding-bottom: 1px;
}
.oo-ui-buttonSelectWidget:last-child {
  margin-left: 0;
}
.oo-ui-buttonSelectWidget:focus {
  outline: 0;
}
.oo-ui-buttonSelectWidget .oo-ui-buttonOptionWidget .oo-ui-buttonElement-button {
  margin-left: -1px;
  margin-bottom: -1px;
  border-radius: 0;
}
.oo-ui-buttonSelectWidget .oo-ui-buttonOptionWidget:first-child .oo-ui-buttonElement-button {
  border-bottom-right-radius: 2px;
  border-top-right-radius: 2px;
}
.oo-ui-buttonSelectWidget .oo-ui-buttonOptionWidget:last-child .oo-ui-buttonElement-button {
  margin-left: 0;
  border-bottom-left-radius: 2px;
  border-top-left-radius: 2px;
}
.oo-ui-buttonSelectWidget .oo-ui-buttonOptionWidget.oo-ui-widget-disabled + .oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  border-right-color: #fff;
}
.oo-ui-buttonSelectWidget.oo-ui-widget-enabled:focus .oo-ui-buttonOptionWidget.oo-ui-optionWidget-selected .oo-ui-buttonElement-button {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c, inset 0 0 0 2px #fff;
  outline: 1px solid transparent;
}
.oo-ui-buttonSelectWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button:hover,
.oo-ui-buttonSelectWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-widget-enabled > .oo-ui-buttonElement-button:active {
  z-index: 1;
}
.oo-ui-buttonSelectWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-buttonElement-active > .oo-ui-buttonElement-button {
  z-index: 3;
}
.oo-ui-buttonSelectWidget.oo-ui-widget-enabled .oo-ui-buttonElement.oo-ui-widget-disabled > .oo-ui-buttonElement-button {
  z-index: -1;
}

.oo-ui-buttonOptionWidget {
  display: inline-block;
}
.oo-ui-buttonOptionWidget.oo-ui-buttonElement-active .oo-ui-buttonElement-button {
  cursor: default;
}


.oo-ui-toggleButtonWidget {
  margin-left: 8px;
}
.oo-ui-toggleButtonWidget:last-child {
  margin-left: 0;
}
.oo-ui-toggleButtonWidget.oo-ui-widget-enabled.oo-ui-buttonElement-frameless.oo-ui-toggleWidget-on .oo-ui-buttonElement-button {
  background-color: rgba(0, 36, 73, 0.08235294);
  color: #000;
}
.oo-ui-toggleButtonWidget.oo-ui-widget-enabled.oo-ui-buttonElement-frameless.oo-ui-toggleWidget-on .oo-ui-buttonElement-button > .oo-ui-iconElement-icon:not( .oo-ui-image-invert ) {
  opacity: 1;
}

.oo-ui-toggleSwitchWidget {
  position: relative;
  display: inline-block;
  vertical-align: middle;
  overflow: hidden;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  -webkit-transform: translateZ(0);
  transform: translateZ(0);
  background-color: #f8f9fa;
  width: 3.5em;
  min-height: 26px;
  height: 2em;
  border: 1px solid #72777d;
  border-radius: 1em;
  margin-left: 8px;
  -webkit-transition: background-color 250ms, border-color 250ms;
  -moz-transition: background-color 250ms, border-color 250ms;
  transition: background-color 250ms, border-color 250ms;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled {
  cursor: pointer;
}
.oo-ui-toggleSwitchWidget-grip {
  position: absolute;
  display: block;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-toggleSwitchWidget:last-child {
  margin-left: 0;
}
.oo-ui-toggleSwitchWidget:before {
  content: '';
  display: block;
  position: absolute;
  top: 1px;
  right: 1px;
  bottom: 1px;
  left: 1px;
  border: 1px solid transparent;
  border-radius: 1em;
  z-index: 1;
  -webkit-transition: border-color 250ms;
  -moz-transition: border-color 250ms;
  transition: border-color 250ms;
}
.oo-ui-toggleSwitchWidget-grip {
  top: 0.35714286em;
  min-width: 16px;
  width: 1.14285714em;
  min-height: 16px;
  height: 1.14285714em;
  border-radius: 1.14285714em;
  -webkit-transition: background-color 250ms, right 100ms, margin-right 100ms;
  -moz-transition: background-color 250ms, right 100ms, margin-right 100ms;
  transition: background-color 250ms, right 100ms, margin-right 100ms;
}
.oo-ui-toggleSwitchWidget-glow {
  display: none;
}
.oo-ui-toggleSwitchWidget.oo-ui-toggleWidget-off .oo-ui-toggleSwitchWidget-grip {
  right: 0.35714286em;
  margin-right: 0;
}
.oo-ui-toggleSwitchWidget.oo-ui-toggleWidget-on .oo-ui-toggleSwitchWidget-grip {
  right: 1.9em;
  margin-right: -2px;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled .oo-ui-toggleSwitchWidget-grip {
  background-color: #f8f9fa;
  border: 1px solid #72777d;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:hover {
  background-color: #fff;
  border-color: #447ff5;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:hover .oo-ui-toggleSwitchWidget-grip {
  background-color: #fff;
  border-color: #447ff5;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:active,
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:active:hover,
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:active:focus {
  background-color: #2a4b8d;
  border-color: #2a4b8d;
  box-shadow: inset 0 0 0 1px #2a4b8d;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:active .oo-ui-toggleSwitchWidget-grip,
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:active:hover .oo-ui-toggleSwitchWidget-grip,
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:active:focus .oo-ui-toggleSwitchWidget-grip {
  background-color: #fff;
  border-color: #fff;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.1);
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
  outline: 0;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled:focus .oo-ui-toggleSwitchWidget-grip {
  border-color: #36c;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on {
  background-color: #36c;
  border-color: #36c;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on .oo-ui-toggleSwitchWidget-grip {
  background-color: #fff;
  border-color: #fff;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.1);
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on:hover {
  background-color: #447ff5;
  border-color: #447ff5;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on:active,
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on:active:hover {
  background-color: #2a4b8d;
  border-color: #2a4b8d;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on:focus {
  border-color: #36c;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-enabled.oo-ui-toggleWidget-on:focus:before {
  border-color: #fff;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-disabled {
  background-color: #c8ccd1;
  border-color: #c8ccd1;
  outline: 0;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-disabled.oo-ui-toggleWidget-off .oo-ui-toggleSwitchWidget-grip {
  border: 1px solid #fff;
  box-shadow: inset 0 0 0 1px #fff;
}
.oo-ui-toggleSwitchWidget.oo-ui-widget-disabled.oo-ui-toggleWidget-on .oo-ui-toggleSwitchWidget-grip {
  background-color: #fff;
}

.oo-ui-selectFileWidget-dropTarget {
  position: relative;
  cursor: default;
  height: 8.815em;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileWidget-dropLabel,
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-selectButton {
  display: none;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileWidget-thumbnail {
  width: 7.815em;
  position: absolute;
  top: 0.5em;
  bottom: 0.5em;
  right: 0.5em;
  background-position: center center;
  background-repeat: no-repeat;
  background-size: contain;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileWidget-thumbnail > .oo-ui-selectFileWidget-noThumbnail-icon {
  opacity: 0.4;
  height: 7.815em;
  width: 7.815em;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-pendingElement-pending {
  background-repeat: repeat;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-info {
  display: block;
  height: 100%;
  width: auto;
  margin-right: 8.815em;
  margin-left: 0;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-info > .oo-ui-inputWidget-input {
  border: 0;
  box-shadow: none;
  background: transparent;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-label {
  display: block;
  position: relative;
  top: inherit;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-fileName {
  display: block;
}
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-selectFileWidget-dropTarget {
  text-align: center;
}
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileWidget-thumbnail,
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-info {
  display: none;
}
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-selectFileWidget-dropTarget .oo-ui-selectFileInputWidget-selectButton {
  display: block;
  margin: 2.2em 1em 1em;
}
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-widget-enabled .oo-ui-selectFileWidget-dropLabel {
  display: block;
}
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-widget-disabled.oo-ui-selectFileWidget-dropTarget {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-widget-disabled.oo-ui-selectFileWidget-dropTarget,
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-widget-disabled.oo-ui-selectFileWidget-dropTarget .oo-ui-buttonElement-button {
  cursor: no-drop;
}
.oo-ui-selectFileWidget-dropTarget {
  background-color: #fff;
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  vertical-align: middle;
  overflow: hidden;
}
.oo-ui-selectFileWidget-dropTarget.oo-ui-selectFileInputWidget-empty {
  border-style: dashed;
}
.oo-ui-selectFileWidget-dropTarget .oo-ui-pendingElement-pending {
  background-size: 1.42857143em 1.42857143em;
}
.oo-ui-selectFileWidget-buttonOnly {
  width: auto;
  max-width: none;
}
.oo-ui-selectFileWidget.oo-ui-widget-enabled.oo-ui-selectFileWidget-dropTarget {
  background-color: #fff;
  -webkit-transition: background-color 250ms, color 250ms, border-color 250ms;
  -moz-transition: background-color 250ms, color 250ms, border-color 250ms;
  transition: background-color 250ms, color 250ms, border-color 250ms;
}
.oo-ui-selectFileWidget.oo-ui-widget-enabled.oo-ui-selectFileWidget-dropTarget:hover {
  border-color: #72777d;
}
.oo-ui-selectFileWidget.oo-ui-widget-enabled .oo-ui-selectFileInputWidget-info > .oo-ui-inputWidget-input {
  -webkit-transition: background-color 250ms, color 250ms;
  -moz-transition: background-color 250ms, color 250ms;
  transition: background-color 250ms, color 250ms;
}
.oo-ui-selectFileWidget.oo-ui-widget-enabled.oo-ui-selectFileWidget-canDrop.oo-ui-selectFileWidget-dropTarget,
.oo-ui-selectFileWidget.oo-ui-widget-enabled.oo-ui-selectFileWidget-canDrop .oo-ui-selectFileInputWidget-info > .oo-ui-inputWidget-input {
  background-color: #eaf3ff;
  color: #36c;
}
.oo-ui-selectFileWidget.oo-ui-widget-disabled.oo-ui-selectFileWidget-dropTarget,
.oo-ui-selectFileWidget.oo-ui-selectFileInputWidget-empty.oo-ui-widget-disabled.oo-ui-selectFileWidget-dropTarget {
  background-color: #eaecf0;
  border-color: #c8ccd1;
}

.oo-ui-outlineSelectWidget {
  height: 100%;
}
.oo-ui-outlineSelectWidget:focus {
  box-shadow: inset 0 0 0 2px #36c;
}

.oo-ui-outlineOptionWidget {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  padding: 0.85714286em 12px;
  -webkit-transition: background-color 100ms, color 100ms;
  -moz-transition: background-color 100ms, color 100ms;
  transition: background-color 100ms, color 100ms;
}
.oo-ui-outlineOptionWidget.oo-ui-optionWidget-highlighted {
  background-color: #eaecf0;
  color: #000;
}
.oo-ui-outlineOptionWidget.oo-ui-optionWidget-selected {
  background-color: #eaf3ff;
  color: #36c;
}
.oo-ui-outlineOptionWidget.oo-ui-optionWidget-pressed {
  background-color: rgba(41, 98, 204, 0.1);
  color: #36c;
}
.oo-ui-outlineOptionWidget.oo-ui-indicatorElement .oo-ui-labelElement-label {
  padding-left: 1.5em;
}
.oo-ui-outlineOptionWidget.oo-ui-indicatorElement .oo-ui-indicatorElement-indicator {
  opacity: 0.5;
}
.oo-ui-outlineOptionWidget-level-1 {
  padding-right: 2.571em;
}
.oo-ui-outlineOptionWidget-level-1.oo-ui-iconElement {
  padding-right: 4.429em;
}
.oo-ui-outlineOptionWidget-level-1.oo-ui-iconElement .oo-ui-iconElement-icon {
  right: 2.571em;
}
.oo-ui-outlineOptionWidget-level-2 {
  padding-right: 5.142em;
}
.oo-ui-outlineOptionWidget-level-2.oo-ui-iconElement {
  padding-right: 6.857em;
}
.oo-ui-outlineOptionWidget-level-2.oo-ui-iconElement .oo-ui-iconElement-icon {
  right: 4.429em;
}

.oo-ui-outlineControlsWidget {
  background-color: #fff;
}
.oo-ui-outlineControlsWidget-items,
.oo-ui-outlineControlsWidget-movers {
  float: right;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-outlineControlsWidget-items {
  float: right;
}
.oo-ui-outlineControlsWidget-items .oo-ui-buttonWidget {
  float: right;
}
.oo-ui-outlineControlsWidget-movers {
  float: left;
}
.oo-ui-outlineControlsWidget-items > .oo-ui-buttonWidget,
.oo-ui-outlineControlsWidget-movers > .oo-ui-buttonWidget {
  height: 2.28571429em;
  width: 2.28571429em;
  margin: 0 !important;
  /* stylelint-disable-line declaration-no-important */
}

.oo-ui-tabSelectWidget {
  text-align: right;
  overflow-x: auto;
  overflow-y: hidden;
  -webkit-overflow-scrolling: auto;
}
.oo-ui-tabSelectWidget-mobile {
  white-space: nowrap;
}
.oo-ui-tabSelectWidget-framed {
  background-color: #eaecf0;
}
.oo-ui-tabSelectWidget-frameless {
  box-shadow: inset 0 -1px 0 0 #a2a9b1;
}
.oo-ui-tabSelectWidget-mobile:after {
  content: '';
  position: absolute;
  bottom: 0;
  left: 0;
  width: 18px;
  height: 2.28571429em;
  pointer-events: none;
}
.oo-ui-tabSelectWidget-mobile.oo-ui-tabSelectWidget-framed:after {
  background-color: #eaecf0;
  background-image: -webkit-gradient(linear, right top, left top, color-stop(0, rgba(234, 236, 240, 0)), color-stop(100%, #eaecf0));
  background-image: -webkit-linear-gradient(right, rgba(234, 236, 240, 0) 0, #eaecf0 100%);
  background-image: -moz-linear-gradient(right, rgba(234, 236, 240, 0) 0, #eaecf0 100%);
  background-image: linear-gradient(to left, rgba(234, 236, 240, 0) 0, #eaecf0 100%);
  background-color: transparent;
}
.oo-ui-tabSelectWidget-mobile.oo-ui-tabSelectWidget-frameless:after {
  background-color: #ffffff;
  background-image: -webkit-gradient(linear, right top, left top, color-stop(0, rgba(255, 255, 255, 0)), color-stop(100%, #fff));
  background-image: -webkit-linear-gradient(right, rgba(255, 255, 255, 0) 0, #fff 100%);
  background-image: -moz-linear-gradient(right, rgba(255, 255, 255, 0) 0, #fff 100%);
  background-image: linear-gradient(to left, rgba(255, 255, 255, 0) 0, #fff 100%);
  background-color: transparent;
  bottom: 0.14285714em;
}
.oo-ui-tabSelectWidget.oo-ui-widget-enabled:focus {
  outline: 0;
}

.oo-ui-tabOptionWidget {
  display: inline-block;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  vertical-align: bottom;
  color: #404244;
  margin: 6px 6px 0 0;
  padding: 0 8px;
  border-color: transparent;
  border-style: solid;
  font-weight: bold;
  line-height: 1;
  -webkit-transition: background-color 100ms, color 100ms, border-color 100ms;
  -moz-transition: background-color 100ms, color 100ms, border-color 100ms;
  transition: background-color 100ms, color 100ms, border-color 100ms;
}
.oo-ui-tabOptionWidget.oo-ui-optionWidget-selected {
  cursor: default;
}
.oo-ui-tabOptionWidget > .oo-ui-labelElement-label {
  color: inherit;
  padding: 6px 5px;
  line-height: 1.28571429em;
  text-decoration: inherit;
}
.oo-ui-tabSelectWidget-mobile .oo-ui-tabOptionWidget:last-child {
  margin-left: 18px;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget {
  border-width: 1px 1px 0 1px;
  border-top-right-radius: 2px;
  border-top-left-radius: 2px;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget .oo-ui-labelElement-label {
  border-bottom: 2px solid transparent;
  padding: 6px 5px 4px;
  -webkit-transition: border-color 100ms;
  -moz-transition: border-color 100ms;
  transition: border-color 100ms;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget.oo-ui-optionWidget-selected {
  background-color: #fff;
  color: #000;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget.oo-ui-optionWidget-selected .oo-ui-labelElement-label {
  border-bottom-color: #fff;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget.oo-ui-widget-enabled:hover {
  background-color: rgba(255, 255, 255, 0.3);
  color: #202122;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget.oo-ui-widget-enabled:active {
  background-color: rgba(255, 255, 255, 0.8);
  color: #000;
}
.oo-ui-tabSelectWidget-framed .oo-ui-tabOptionWidget.oo-ui-widget-enabled.oo-ui-optionWidget-selected:hover {
  background-color: #fff;
}
.oo-ui-tabSelectWidget-framed.oo-ui-widget-enabled:focus .oo-ui-tabOptionWidget.oo-ui-optionWidget-selected .oo-ui-labelElement-label {
  border-bottom-color: #36c;
}
.oo-ui-tabSelectWidget-frameless .oo-ui-tabOptionWidget {
  margin: 6px 0 -1px 6px;
  border-width: 1px 0;
  box-shadow: inset 0 0 0 0 #36c;
  -webkit-transition: color 100ms, box-shadow 100ms;
  -moz-transition: color 100ms, box-shadow 100ms;
  transition: color 100ms, box-shadow 100ms;
}
.oo-ui-tabSelectWidget-frameless .oo-ui-tabOptionWidget.oo-ui-optionWidget-selected {
  color: #36c;
  box-shadow: inset 0 -2px 0 0 #36c;
}
.oo-ui-tabSelectWidget-frameless .oo-ui-tabOptionWidget.oo-ui-widget-enabled:hover {
  color: #447ff5;
  box-shadow: inset 0 -2px 0 0 #447ff5;
}
.oo-ui-tabSelectWidget-frameless .oo-ui-tabOptionWidget.oo-ui-widget-enabled:active {
  color: #2a4b8d;
  box-shadow: inset 0 -2px 0 0 #2a4b8d;
}
.oo-ui-tabSelectWidget-frameless.oo-ui-widget-enabled:focus .oo-ui-tabOptionWidget.oo-ui-optionWidget-selected {
  border-radius: 2px;
  box-shadow: inset 0 0 0 2px #36c;
}

.oo-ui-tagMultiselectWidget {
  display: inline-block;
  position: relative;
  width: 100%;
  max-width: 50em;
  vertical-align: top;
}
.oo-ui-tagMultiselectWidget-handle {
  width: 100%;
  display: block;
  position: relative;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-iconElement-icon,
.oo-ui-tagMultiselectWidget-handle > .oo-ui-indicatorElement-indicator {
  position: absolute;
  top: 0;
  height: 100%;
}
.oo-ui-tagMultiselectWidget-content {
  position: relative;
}
.oo-ui-tagMultiselectWidget-group {
  display: inline;
}
.oo-ui-tagMultiselectWidget-outlined {
  width: 100%;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-inlined .oo-ui-tagMultiselectWidget-handle {
  cursor: text;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-disabled .oo-ui-tagMultiselectWidget-content > input {
  display: none;
}
.oo-ui-tagMultiselectWidget-focusTrap {
  display: inline-block;
  height: 1px;
  width: 1px;
}
.oo-ui-tagMultiselectWidget-focusTrap:focus {
  outline: 0;
}
.oo-ui-tagMultiselectWidget-handle {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  min-height: 2.28571429em;
  margin-left: 8px;
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  padding: 0 8px;
  line-height: 1;
}
.oo-ui-tagMultiselectWidget-handle:last-child {
  margin-left: 0;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input {
  background-color: transparent;
  color: #000;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  height: 2.14285714em;
  border: 0;
  padding: 6px 0.57142857em 6px 0;
  font-family: inherit;
  font-size: inherit;
  line-height: 1.28571429em;
  vertical-align: top;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input::-webkit-input-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input:-ms-input-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input::-moz-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input:-moz-placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input::placeholder {
  color: #72777d;
  opacity: 1;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content > input:focus {
  outline: 0;
}
.oo-ui-tagMultiselectWidget-handle > .oo-ui-tagMultiselectWidget-content .oo-ui-tagMultiselectWidget-group:empty + input {
  margin-right: 0;
}
.oo-ui-tagMultiselectWidget-handle .oo-ui-tagMultiselectWidget-group {
  display: inline-block;
  max-width: 100%;
  margin-top: 4px;
  margin-right: -0.14285714em;
}
.oo-ui-tagMultiselectWidget.oo-ui-iconElement .oo-ui-tagMultiselectWidget-handle {
  padding-right: 2.02857143em;
}
.oo-ui-tagMultiselectWidget.oo-ui-iconElement .oo-ui-tagMultiselectWidget-handle > .oo-ui-iconElement-icon {
  right: 0;
  margin: 0 0.3em;
}
.oo-ui-tagMultiselectWidget.oo-ui-indicatorElement .oo-ui-tagMultiselectWidget-handle {
  padding-left: 2.40714286em;
}
.oo-ui-tagMultiselectWidget.oo-ui-indicatorElement .oo-ui-tagMultiselectWidget-handle > .oo-ui-indicatorElement-indicator {
  left: 0;
  margin: 0 0.775em;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled {
  background-color: #fff;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled .oo-ui-tagMultiselectWidget-handle {
  -webkit-transition: border-color 250ms, box-shadow 250ms;
  -moz-transition: border-color 250ms, box-shadow 250ms;
  transition: border-color 250ms, box-shadow 250ms;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-inlined:hover .oo-ui-tagMultiselectWidget-handle {
  border-color: #72777d;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-inlined.oo-ui-tagMultiselectWidget-focus .oo-ui-tagMultiselectWidget-handle {
  border-color: #36c;
  outline: 0;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-inlined.oo-ui-flaggedElement-invalid .oo-ui-tagMultiselectWidget-handle,
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-inlined.oo-ui-flaggedElement-invalid:hover .oo-ui-tagMultiselectWidget-handle {
  border-color: #d33;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-outlined {
  background-color: #f8f9fa;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-outlined .oo-ui-tagMultiselectWidget-handle {
  border-bottom: 0;
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-outlined .oo-ui-tagItemWidget.oo-ui-widget-enabled {
  background-color: #fff;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-outlined .oo-ui-inputWidget-input {
  border-top-right-radius: 0;
  border-top-left-radius: 0;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-tagMultiselectWidget-outlined.oo-ui-flaggedElement-invalid .oo-ui-inputWidget-input {
  border-color: #d33;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid .oo-ui-inputWidget-input {
  color: #d33;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-disabled .oo-ui-tagMultiselectWidget-handle {
  background-color: #eaecf0;
  color: #72777d;
  text-shadow: 0 1px 1px #fff;
  border-color: #c8ccd1;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-disabled .oo-ui-tagMultiselectWidget-handle > .oo-ui-iconElement-icon {
  opacity: 0.51;
}
.oo-ui-tagMultiselectWidget.oo-ui-widget-disabled .oo-ui-tagMultiselectWidget-handle > .oo-ui-indicatorElement-indicator {
  opacity: 0.15;
}


.oo-ui-tagItemWidget {
  position: relative;
  display: inline-block;
  cursor: default;
  white-space: nowrap;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  width: auto;
  max-width: 100%;
  height: 1.57142857em;
  margin: 0 0 4px 4px;
  border: 1px solid #a2a9b1;
  border-radius: 1.57142857em;
  padding: 0 8px;
  line-height: 1.42857143em;
  vertical-align: middle;
}
.oo-ui-tagItemWidget.oo-ui-labelElement .oo-ui-labelElement-label {
  display: inline-block;
  max-width: 100%;
  text-overflow: ellipsis;
  overflow: hidden;
}
.oo-ui-tagItemWidget.oo-ui-labelElement:not( .oo-ui-tagItemWidget-fixed ) .oo-ui-labelElement-label {
  cursor: text;
}
.oo-ui-tagItemWidget.oo-ui-labelElement .oo-ui-labelElement-label {
  font-size: 0.92857143em;
  line-height: 1.61538462em;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled {
  color: #202122;
  -webkit-transition: background-color 100ms, color 100ms, border-color 100ms, box-shadow 100ms;
  -moz-transition: background-color 100ms, color 100ms, border-color 100ms, box-shadow 100ms;
  transition: background-color 100ms, color 100ms, border-color 100ms, box-shadow 100ms;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled:hover {
  background-color: #fff;
  color: #404244;
  border-color: #a2a9b1;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled:focus {
  border-color: #36c;
  box-shadow: inset 0 0 0 1px #36c;
  outline: 0;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled:not( .oo-ui-tagItemWidget-fixed ) {
  background-color: #f8f9fa;
  padding-left: 23px;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid {
  border-color: #d33;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid:hover {
  border-color: #d33;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-invalid:focus {
  border-color: #d33;
  box-shadow: inset 0 0 0 1px #d33;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled > .oo-ui-buttonElement {
  display: block;
  position: absolute;
  top: 0;
  left: 0;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled .oo-ui-buttonElement-button {
  display: block;
  border: 0;
  border-radius: 1.57142857em;
  padding-top: 1.42857143em;
  padding-right: 1.42857143em;
  -webkit-transition: background-color 100ms;
  -moz-transition: background-color 100ms;
  transition: background-color 100ms;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled .oo-ui-buttonElement-button:hover {
  background-color: #fff;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled .oo-ui-buttonElement-button:active {
  background-color: #c8ccd1;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled .oo-ui-buttonElement-button .oo-ui-icon-close {
  right: 3px;
  min-width: 14px;
  width: 1em;
}
.oo-ui-tagItemWidget.oo-ui-widget-enabled .oo-ui-buttonElement-button .oo-ui-labelElement-label,
.oo-ui-tagItemWidget.oo-ui-widget-enabled .oo-ui-buttonElement-button .oo-ui-indicatorElement-indicator {
  display: none;
}
.oo-ui-tagItemWidget.oo-ui-widget-disabled {
  background-color: #eaecf0;
  color: #72777d;
  border-color: #c8ccd1;
  text-shadow: 0 1px 1px #fff;
}
.oo-ui-tagItemWidget.oo-ui-widget-disabled:focus {
  outline: 0;
}
.oo-ui-tagItemWidget.oo-ui-widget-disabled > .oo-ui-buttonElement {
  display: none;
}

.oo-ui-searchWidget-query {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
}
.oo-ui-searchWidget-query .oo-ui-textInputWidget {
  width: 100%;
}
.oo-ui-searchWidget-results {
  position: absolute;
  bottom: 0;
  right: 0;
  left: 0;
  overflow-x: hidden;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
}
.oo-ui-searchWidget-query {
  height: 4em;
  padding: 0 16px;
  border-bottom: 1px solid #c8ccd1;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.1);
}
.oo-ui-searchWidget-query .oo-ui-textInputWidget {
  margin: 12px 0;
}
.oo-ui-searchWidget-results {
  top: 4em;
  padding: 12px 16px 16px;
  line-height: 0;
}

/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-tool {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-tool > .oo-ui-tool-link > .oo-ui-tool-checkIcon {
  display: none;
}
.oo-ui-tool .oo-ui-tool-link {
  position: relative;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  padding-top: 3em;
  padding-right: 3em;
}
.oo-ui-toolbar-narrow .oo-ui-tool .oo-ui-tool-link {
  padding-right: 2.85714286em;
}
.oo-ui-tool.oo-ui-tool-with-label .oo-ui-tool-link {
  padding: 0.85714286em 12px 0.85714286em 12px;
}
.oo-ui-tool.oo-ui-tool-with-icon.oo-ui-tool-with-label .oo-ui-tool-link {
  padding: 0.85714286em 2.64285714em 0.85714286em 12px;
}
.oo-ui-tool .oo-ui-iconElement-icon {
  display: block;
  right: 0.78571429em;
}
.oo-ui-toolbar-narrow .oo-ui-tool .oo-ui-iconElement-icon {
  right: 0.71428571em;
}
.oo-ui-tool .oo-ui-tool-title {
  line-height: 1.28571429em;
}
.oo-ui-tool.oo-ui-widget-enabled .oo-ui-tool-link {
  -webkit-transition: background-color 100ms;
  -moz-transition: background-color 100ms;
  transition: background-color 100ms;
}
.oo-ui-tool.oo-ui-widget-enabled .oo-ui-tool-link:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c;
  z-index: 1;
}
.oo-ui-tool.oo-ui-widget-enabled .oo-ui-iconElement-icon {
  opacity: 0.87;
  -webkit-transition: opacity 100ms;
  -moz-transition: opacity 100ms;
  transition: opacity 100ms;
  -webkit-transform: translateZ(0);
  transform: translateZ(0);
}
.oo-ui-tool.oo-ui-widget-enabled .oo-ui-tool-title {
  -webkit-transition: color 100ms;
  -moz-transition: color 100ms;
  transition: color 100ms;
}
.oo-ui-tool.oo-ui-widget-enabled:hover .oo-ui-iconElement-icon {
  opacity: 1;
}
.oo-ui-tool.oo-ui-widget-enabled[class*='oo-ui-flaggedElement'] .oo-ui-iconElement-icon {
  opacity: 1;
}
.oo-ui-tool.oo-ui-widget-enabled > .oo-ui-tool-link {
  color: inherit;
}
.oo-ui-tool.oo-ui-widget-enabled > .oo-ui-tool-link:hover {
  background-color: #eaecf0;
}
.oo-ui-tool.oo-ui-widget-enabled > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled > .oo-ui-tool-link:active:focus,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-tool-active > .oo-ui-tool-link,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-popupToolGroup-active > .oo-ui-tool-link {
  background-color: #eff3fa;
  color: #36c;
  box-shadow: none;
}
.oo-ui-tool.oo-ui-widget-enabled > .oo-ui-tool-link:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-tool-link {
  color: #36c;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:hover {
  background-color: #eaecf0;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:active:focus,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive.oo-ui-tool-active > .oo-ui-tool-link,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive.oo-ui-popupToolGroup-active > .oo-ui-tool-link {
  background-color: #eff3fa;
  color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-tool-link {
  color: #d73333;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:hover {
  background-color: #eaecf0;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:active:focus,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive.oo-ui-tool-active > .oo-ui-tool-link,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive.oo-ui-popupToolGroup-active > .oo-ui-tool-link {
  background-color: #ffffff;
  color: #b32424;
  box-shadow: none;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:focus {
  outline: 1px solid #d33;
  box-shadow: inset 0 0 0 1px #d33;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-tool-link {
  color: #fff;
  background-color: #36c;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:hover {
  background-color: #447ff5;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:active:focus,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-tool-active > .oo-ui-tool-link {
  color: #fff;
  background-color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-tool-link:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c, inset 0 0 0 2px #fff;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-widget-disabled > .oo-ui-tool-link,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-widget-disabled > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-widget-disabled > .oo-ui-tool-link:active:focus {
  background-color: #c8ccd1;
  color: #fff;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-tool-link {
  color: #fff;
  background-color: #d33;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:hover {
  background-color: #ff4242;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:active:focus,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-tool-active > .oo-ui-tool-link {
  color: #fff;
  background-color: #b32424;
  box-shadow: none;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-tool-link:focus {
  outline: 1px solid #d33;
  box-shadow: inset 0 0 0 1px #d33, inset 0 0 0 2px #fff;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-widget-disabled > .oo-ui-tool-link,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-widget-disabled > .oo-ui-tool-link:active,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-widget-disabled > .oo-ui-tool-link:active:focus {
  background-color: #c8ccd1;
  color: #fff;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary > .oo-ui-tool-link {
  font-weight: bold;
}
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary > .oo-ui-tool-link > .oo-ui-iconElement-icon,
.oo-ui-tool.oo-ui-widget-enabled.oo-ui-flaggedElement-primary > .oo-ui-tool-link > .oo-ui-indicatorElement-indicator {
  opacity: 1;
}

.oo-ui-popupTool-popup.oo-ui-popupWidget {
  z-index: 4;
}

.oo-ui-toolGroupTool > .oo-ui-toolGroup {
  border-left: 0;
}

.oo-ui-toolGroup {
  display: inline-block;
  vertical-align: middle;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-toolGroup-empty {
  display: none;
}
.oo-ui-toolGroup-tools .oo-ui-tool-link {
  color: inherit;
  text-decoration: none;
  cursor: pointer;
}
.oo-ui-toolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool-link,
.oo-ui-toolGroup-tools .oo-ui-widget-disabled > .oo-ui-tool-link {
  outline: 0;
  cursor: default;
}
.oo-ui-toolGroup.oo-ui-widget-disabled:not( .oo-ui-flaggedElement-primary ) .oo-ui-indicatorElement-indicator,
.oo-ui-toolGroup.oo-ui-widget-disabled:not( .oo-ui-flaggedElement-primary ) .oo-ui-iconElement-icon {
  opacity: 0.34 !important;
}

.oo-ui-barToolGroup-tools > .oo-ui-tool {
  display: inline-block;
  position: relative;
  vertical-align: top;
}
.oo-ui-barToolGroup-tools > .oo-ui-tool > .oo-ui-tool-link {
  display: block;
}
.oo-ui-barToolGroup-tools > .oo-ui-tool > .oo-ui-tool-link .oo-ui-tool-accel {
  display: none;
}
.oo-ui-barToolGroup-tools > .oo-ui-tool.oo-ui-iconElement > .oo-ui-tool-link .oo-ui-tool-title {
  display: none;
}
.oo-ui-barToolGroup-tools > .oo-ui-tool.oo-ui-iconElement.oo-ui-tool-with-label > .oo-ui-tool-link .oo-ui-tool-title {
  display: inline-block;
}
.oo-ui-barToolGroup-tools > .oo-ui-tool + .oo-ui-tool {
  margin-right: -2px;
}
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-enabled-tools .oo-ui-tool.oo-ui-widget-disabled .oo-ui-iconElement-icon,
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool .oo-ui-iconElement-icon {
  opacity: 0.34;
}
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-enabled-tools .oo-ui-tool.oo-ui-widget-disabled.oo-ui-flaggedElement-primary > .oo-ui-tool-link,
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool.oo-ui-flaggedElement-primary > .oo-ui-tool-link {
  background-color: #c8ccd1;
  border-color: #c8ccd1;
}
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-enabled-tools .oo-ui-tool.oo-ui-widget-disabled.oo-ui-flaggedElement-primary > .oo-ui-tool-link .oo-ui-tool-title,
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool.oo-ui-flaggedElement-primary > .oo-ui-tool-link .oo-ui-tool-title {
  color: #fff;
}
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-enabled-tools .oo-ui-tool.oo-ui-widget-disabled.oo-ui-flaggedElement-primary.oo-ui-tool-active > .oo-ui-tool-link,
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool.oo-ui-flaggedElement-primary.oo-ui-tool-active > .oo-ui-tool-link {
  background-color: #919fb9;
}
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-enabled-tools .oo-ui-tool.oo-ui-widget-disabled.oo-ui-flaggedElement-primary .oo-ui-iconElement-icon,
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool.oo-ui-flaggedElement-primary .oo-ui-iconElement-icon {
  opacity: 1;
}
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-enabled-tools .oo-ui-tool.oo-ui-widget-disabled > .oo-ui-tool-link .oo-ui-tool-title,
.oo-ui-barToolGroup-tools.oo-ui-toolGroup-disabled-tools .oo-ui-tool > .oo-ui-tool-link .oo-ui-tool-title {
  color: #72777d;
}

.oo-ui-popupToolGroup {
  position: relative;
  min-width: 3em;
}
.oo-ui-popupToolGroup-handle {
  display: block;
  cursor: pointer;
}
.oo-ui-popupToolGroup-handle .oo-ui-labelElement-label:not( :empty ) {
  display: inline-block;
}
.oo-ui-popupToolGroup.oo-ui-widget-disabled .oo-ui-popupToolGroup-handle {
  outline: 0;
  cursor: default;
}
.oo-ui-popupToolGroup-tools {
  display: none;
  position: absolute;
  z-index: 4;
}
.oo-ui-popupToolGroup-tools.oo-ui-popupToolGroup-active-tools {
  display: block;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-link {
  display: table;
  width: 100%;
  vertical-align: middle;
  white-space: nowrap;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-link .oo-ui-tool-accel,
.oo-ui-popupToolGroup-tools .oo-ui-tool-link .oo-ui-tool-title {
  display: table-cell;
  vertical-align: middle;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-link .oo-ui-tool-accel {
  text-align: left;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup {
  min-width: 2.85714286em;
}
.oo-ui-popupToolGroup.oo-ui-indicatorElement:not( .oo-ui-labelElement ):not( .oo-ui-iconElement ) {
  min-width: 1.85714286em;
}
.oo-ui-popupToolGroup-handle {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-popupToolGroup-handle .oo-ui-iconElement-icon {
  right: 0.78571429em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup-handle .oo-ui-iconElement-icon {
  right: 0.64285714em;
}
.oo-ui-popupToolGroup-handle .oo-ui-indicatorElement-indicator {
  background-position: center 48%;
  left: 0.57142857em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup-handle .oo-ui-indicatorElement-indicator {
  left: 0.28571429em;
}
.oo-ui-popupToolGroup.oo-ui-iconElement .oo-ui-popupToolGroup-handle,
.oo-ui-popupToolGroup.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-top: 3em;
}
.oo-ui-popupToolGroup.oo-ui-iconElement.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-right: 3em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup.oo-ui-iconElement.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-right: 2.85714286em;
}
.oo-ui-popupToolGroup.oo-ui-labelElement .oo-ui-popupToolGroup-handle {
  padding: 0.85714286em 12px;
}
.oo-ui-popupToolGroup.oo-ui-labelElement .oo-ui-popupToolGroup-handle .oo-ui-labelElement-label {
  line-height: 1.28571429em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup.oo-ui-labelElement .oo-ui-popupToolGroup-handle {
  padding-right: 0.71428571em;
  padding-left: 0.71428571em;
}
.oo-ui-popupToolGroup.oo-ui-iconElement.oo-ui-labelElement .oo-ui-popupToolGroup-handle {
  padding-right: 2.64285714em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup.oo-ui-iconElement.oo-ui-labelElement .oo-ui-popupToolGroup-handle {
  padding-right: 2.5em;
}
.oo-ui-popupToolGroup.oo-ui-labelElement.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-left: 1.85714286em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup.oo-ui-labelElement.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-left: 1.28571429em;
}
.oo-ui-popupToolGroup.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-left: 1.14285714em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup.oo-ui-indicatorElement .oo-ui-popupToolGroup-handle {
  padding-left: 0.85714286em;
}
.oo-ui-popupToolGroup:not( .oo-ui-labelElement ):not( .oo-ui-iconElement ) .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-popupToolGroup-header {
  color: #72777d;
  padding: 0 12px;
  font-weight: bold;
  line-height: 2.28571429em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup-header {
  padding: 0 0.71428571em;
}
.oo-ui-popupToolGroup-tools {
  background-color: #fff;
  min-width: 16em;
  margin: 0 -1px;
  border: 1px solid #c8ccd1;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.25);
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-link {
  padding: 0.85714286em 12px;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup-tools .oo-ui-tool-link {
  padding-right: 0.71428571em;
  padding-left: 0.71428571em;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-link .oo-ui-tool-accel {
  color: #72777d;
  line-height: 1.28571429em;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-link .oo-ui-tool-accel:not( :empty ) {
  padding-right: 1.28571429em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup-tools .oo-ui-tool-link .oo-ui-tool-accel:not( :empty ) {
  padding-right: 1.14285714em;
}
.oo-ui-popupToolGroup-tools .oo-ui-iconElement .oo-ui-tool-link {
  padding-right: 2.64285714em;
}
.oo-ui-toolbar-narrow .oo-ui-popupToolGroup-tools .oo-ui-iconElement .oo-ui-tool-link {
  padding-right: 2.5em;
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle {
  -webkit-transition: background-color 100ms, box-shadow 100ms;
  -moz-transition: background-color 100ms, box-shadow 100ms;
  transition: background-color 100ms, box-shadow 100ms;
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:hover {
  background-color: #eaecf0;
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:active {
  background-color: #eaf3ff;
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle .oo-ui-iconElement-icon,
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle .oo-ui-indicatorElement-indicator {
  opacity: 0.87;
  -webkit-transition: opacity 100ms;
  -moz-transition: opacity 100ms;
  transition: opacity 100ms;
  -webkit-transform: translateZ(0);
  transform: translateZ(0);
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:hover .oo-ui-iconElement-icon,
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:active .oo-ui-iconElement-icon,
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:hover .oo-ui-indicatorElement-indicator,
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:active .oo-ui-indicatorElement-indicator {
  opacity: 1;
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-popupToolGroup.oo-ui-widget-enabled[class*='oo-ui-flaggedElement'] > .oo-ui-popupToolGroup-handle .oo-ui-iconElement-icon,
.oo-ui-popupToolGroup.oo-ui-widget-enabled[class*='oo-ui-flaggedElement'] > .oo-ui-popupToolGroup-handle:hover .oo-ui-iconElement-icon {
  opacity: 1;
}
.oo-ui-toolbar-actions .oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle {
  border-left: 1px solid transparent;
  -webkit-transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
  -moz-transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
  transition: background-color 100ms, border-color 100ms, box-shadow 100ms;
}
.oo-ui-toolbar-actions .oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:hover {
  border-left-color: #eaecf0;
}
.oo-ui-toolbar-actions .oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:active {
  border-left-color: #eaf3ff;
}
.oo-ui-toolbar-actions .oo-ui-popupToolGroup.oo-ui-widget-enabled > .oo-ui-popupToolGroup-handle:focus {
  border-left-color: #36c;
}
.oo-ui-popupToolGroup.oo-ui-popupToolGroup-active > .oo-ui-popupToolGroup-handle {
  background-color: #eaf3ff;
  color: #36c;
}
.oo-ui-popupToolGroup.oo-ui-popupToolGroup-active > .oo-ui-popupToolGroup-handle:hover {
  background-color: rgba(41, 98, 204, 0.1);
  color: #36c;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool-active.oo-ui-widget-enabled .oo-ui-tool-link .oo-ui-tool-title {
  color: #36c;
}
.oo-ui-popupToolGroup-tools .oo-ui-tool.oo-ui-widget-enabled .oo-ui-tool-link:focus {
  outline: 0;
  box-shadow: inset 0 0 0 2px #36c;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle {
  color: #36c;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:hover {
  background-color: #fff;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:active,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:active:focus,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive.oo-ui-tool-active > .oo-ui-popupToolGroup-handle,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive.oo-ui-popupToolGroup-active > .oo-ui-popupToolGroup-handle {
  background-color: #eff3fa;
  color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle {
  color: #d73333;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:hover {
  background-color: #fff;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:active,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:active:focus,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive.oo-ui-tool-active > .oo-ui-popupToolGroup-handle,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive.oo-ui-popupToolGroup-active > .oo-ui-popupToolGroup-handle {
  background-color: #ffffff;
  color: #b32424;
  box-shadow: none;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:focus {
  outline: 1px solid #d33;
  box-shadow: inset 0 0 0 1px #d33;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle {
  color: #fff;
  background-color: #36c;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:hover {
  background-color: #447ff5;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:active,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:active:focus,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-tool-active > .oo-ui-popupToolGroup-handle {
  color: #fff;
  background-color: #2a4b8d;
  box-shadow: none;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive > .oo-ui-popupToolGroup-handle:focus {
  outline: 1px solid #36c;
  box-shadow: inset 0 0 0 1px #36c, inset 0 0 0 2px #fff;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-widget-disabled > .oo-ui-popupToolGroup-handle,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-widget-disabled > .oo-ui-popupToolGroup-handle:active,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-progressive.oo-ui-widget-disabled > .oo-ui-popupToolGroup-handle:active:focus {
  background-color: #c8ccd1;
  color: #fff;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle {
  color: #fff;
  background-color: #d33;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:hover {
  background-color: #ff4242;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:active,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:active:focus,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-tool-active > .oo-ui-popupToolGroup-handle {
  color: #fff;
  background-color: #b32424;
  box-shadow: none;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive > .oo-ui-popupToolGroup-handle:focus {
  outline: 1px solid #d33;
  box-shadow: inset 0 0 0 1px #d33, inset 0 0 0 2px #fff;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-widget-disabled > .oo-ui-popupToolGroup-handle,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-widget-disabled > .oo-ui-popupToolGroup-handle:active,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary.oo-ui-flaggedElement-destructive.oo-ui-widget-disabled > .oo-ui-popupToolGroup-handle:active:focus {
  background-color: #c8ccd1;
  color: #fff;
}
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary > .oo-ui-popupToolGroup-handle > .oo-ui-iconElement-icon,
.oo-ui-popupToolGroup.oo-ui-flaggedElement-primary > .oo-ui-popupToolGroup-handle > .oo-ui-indicatorElement-indicator {
  opacity: 1;
}

.oo-ui-listToolGroup-tools .oo-ui-tool {
  display: block;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-listToolGroup-tools .oo-ui-tool.oo-ui-widget-enabled:hover {
  background-color: #eaecf0;
  color: #000;
}
.oo-ui-listToolGroup-tools .oo-ui-tool-active.oo-ui-widget-enabled {
  background-color: #eaf3ff;
}
.oo-ui-listToolGroup-tools .oo-ui-tool-active.oo-ui-widget-enabled:hover {
  background-color: rgba(41, 98, 204, 0.1);
}
.oo-ui-listToolGroup-tools .oo-ui-tool-active.oo-ui-widget-enabled .oo-ui-tool-link .oo-ui-tool-title {
  color: #36c;
}
.oo-ui-listToolGroup.oo-ui-widget-disabled,
.oo-ui-listToolGroup-tools .oo-ui-tool.oo-ui-widget-disabled .oo-ui-tool-title {
  color: #72777d;
}

.oo-ui-menuToolGroup {
  border-right: 1px solid #eaecf0;
  border-left: 1px solid #eaecf0;
}
.oo-ui-menuToolGroup-tools .oo-ui-tool {
  display: block;
}
.oo-ui-menuToolGroup + .oo-ui-menuToolGroup {
  border-right: 0;
}
.oo-ui-menuToolGroup .oo-ui-popupToolGroup-handle {
  min-width: 140px;
}
.oo-ui-toolbar-narrow .oo-ui-menuToolGroup .oo-ui-popupToolGroup-handle {
  min-width: 100px;
}
.oo-ui-menuToolGroup-tools .oo-ui-tool-link .oo-ui-iconElement-icon {
  right: 0.78571429em;
}
.oo-ui-toolbar-narrow .oo-ui-menuToolGroup-tools .oo-ui-tool-link .oo-ui-iconElement-icon {
  right: 0.71428571em;
}
.oo-ui-menuToolGroup-tools .oo-ui-tool.oo-ui-widget-enabled:hover {
  background-color: rgba(41, 98, 204, 0.1);
}
.oo-ui-menuToolGroup-tools .oo-ui-tool.oo-ui-tool-active {
  background-color: #eaf3ff;
}
.oo-ui-menuToolGroup.oo-ui-widget-disabled,
.oo-ui-menuToolGroup-tools .oo-ui-tool.oo-ui-widget-disabled .oo-ui-tool-title {
  color: #72777d;
}

.oo-ui-toolbar {
  clear: both;
}
.oo-ui-toolbar-bar {
  line-height: 1;
  position: relative;
}
.oo-ui-toolbar-tools,
.oo-ui-toolbar-actions {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.oo-ui-toolbar-tools {
  display: inline;
}
.oo-ui-toolbar-popups {
  position: absolute;
}
.oo-ui-toolbar-tools,
.oo-ui-toolbar-popups {
  white-space: nowrap;
}
.oo-ui-toolbar-tools .oo-ui-tool,
.oo-ui-toolbar-popups .oo-ui-tool,
.oo-ui-toolbar-tools .oo-ui-popupTool-popup,
.oo-ui-toolbar-popups .oo-ui-popupTool-popup {
  white-space: normal;
}
.oo-ui-toolbar-narrow .oo-ui-toolbar-tools,
.oo-ui-toolbar-narrow.oo-ui-toolbar-popups {
  white-space: normal;
}
.oo-ui-toolbar-actions {
  float: left;
}
.oo-ui-toolbar-actions .oo-ui-toolbar,
.oo-ui-toolbar-actions .oo-ui-buttonElement.oo-ui-labelElement > input.oo-ui-buttonElement-button,
.oo-ui-toolbar-actions .oo-ui-buttonElement.oo-ui-labelElement > .oo-ui-buttonElement-button > .oo-ui-labelElement-label {
  display: inline-block;
}
.oo-ui-toolbar-actions .oo-ui-popupWidget {
  -webkit-touch-callout: default;
  -webkit-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  user-select: text;
}
.oo-ui-toolbar-bar {
  background-color: #fff;
  color: #202122;
}
.oo-ui-toolbar-position-top > .oo-ui-toolbar-bar {
  border-bottom: 1px solid #c8ccd1;
  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.1);
}
.oo-ui-toolbar-position-bottom > .oo-ui-toolbar-bar {
  border-top: 1px solid #c8ccd1;
  box-shadow: 0 -1px 1px 0 rgba(0, 0, 0, 0.1);
}
.oo-ui-toolbar-bar .oo-ui-toolbar-bar {
  background-color: transparent;
  border: 0;
  box-shadow: none;
}
.oo-ui-toolbar-narrow-bar:after {
  content: '';
  display: block;
  position: absolute;
  top: 3em;
  right: 0;
  width: 100%;
  height: 0;
}
.oo-ui-toolbar-narrow.oo-ui-toolbar-position-top .oo-ui-toolbar-bar:after {
  border-bottom: 1px solid #c8ccd1;
}
.oo-ui-toolbar-narrow.oo-ui-toolbar-position-bottom .oo-ui-toolbar-bar:after {
  border-top: 1px solid #c8ccd1;
}

/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */

.oo-ui-window {
  background: transparent;
}
.oo-ui-window-frame {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-window-content {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  bottom: 0;
  overflow: hidden;
}
.oo-ui-window-content:focus {
  outline: 0;
}
.oo-ui-window-head,
.oo-ui-window-foot {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.oo-ui-window-body {
  margin: 0;
  padding: 0;
  background: none;
}
.oo-ui-window-overlay {
  position: absolute;
  top: 0;
  /* @noflip */
  left: 0;
}

.oo-ui-dialog-content > .oo-ui-window-head,
.oo-ui-dialog-content > .oo-ui-window-body,
.oo-ui-dialog-content > .oo-ui-window-foot {
  position: absolute;
  right: 0;
  left: 0;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.oo-ui-dialog-content > .oo-ui-window-head {
  overflow: hidden;
  z-index: 3;
  top: 0;
}
.oo-ui-dialog-content > .oo-ui-window-body {
  overflow: auto;
  -webkit-overflow-scrolling: touch;
  z-index: 2;
  top: 0;
  bottom: 0;
}
.oo-ui-dialog-content > .oo-ui-window-foot {
  z-index: 3;
  bottom: 0;
}

.oo-ui-messageDialog-actions .oo-ui-actionWidget {
  position: relative;
  text-align: center;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget .oo-ui-buttonElement-button {
  display: block;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget .oo-ui-labelElement-label {
  position: relative;
  top: auto;
  bottom: auto;
}
.oo-ui-messageDialog-actions-horizontal {
  display: table;
  table-layout: fixed;
  width: 100%;
}
.oo-ui-messageDialog-actions-horizontal .oo-ui-actionWidget {
  display: table-cell;
  width: 1%;
}
.oo-ui-messageDialog-actions-vertical {
  display: block;
}
.oo-ui-messageDialog-actions-vertical .oo-ui-actionWidget {
  display: block;
  overflow: hidden;
  text-overflow: ellipsis;
}
.oo-ui-messageDialog-content > .oo-ui-window-foot {
  outline: 1px solid #a2a9b1;
}
.oo-ui-messageDialog-title,
.oo-ui-messageDialog-message {
  display: block;
  line-height: 1.42857143em;
  text-align: center;
}
.oo-ui-messageDialog-title {
  font-size: 1.5em;
  color: #000;
}
.oo-ui-messageDialog-message {
  font-size: 1.1em;
  color: #202122;
  text-align: right;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget {
  min-height: 2.85714286em;
  margin-left: 0;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget:last-child {
  margin-left: 0;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget:first-child {
  margin-right: 0;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget .oo-ui-buttonElement-button {
  border: 0;
  border-radius: 0;
  padding-top: 0;
  padding-bottom: 0;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget.oo-ui-labelElement .oo-ui-labelElement-label {
  line-height: 2.85714286em;
  text-align: center;
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-progressive .oo-ui-buttonElement-button:active {
  background-color: rgba(8, 126, 204, 0.1);
}
.oo-ui-messageDialog-actions .oo-ui-actionWidget.oo-ui-widget-enabled.oo-ui-flaggedElement-destructive .oo-ui-buttonElement-button:active {
  background-color: rgba(212, 83, 83, 0.1);
}
.oo-ui-messageDialog-actions-horizontal .oo-ui-actionWidget {
  border-left: 1px solid #a2a9b1;
  margin: 0;
}
.oo-ui-messageDialog-actions-horizontal .oo-ui-actionWidget:first-child > .oo-ui-buttonElement-button {
  border-radius: 0 0 2px 0;
}
.oo-ui-messageDialog-actions-horizontal .oo-ui-actionWidget:last-child {
  border-left-width: 0;
}
.oo-ui-messageDialog-actions-horizontal .oo-ui-actionWidget:last-child > .oo-ui-buttonElement-button {
  border-radius: 0 0 0 2px;
}
.oo-ui-messageDialog-actions-horizontal .oo-ui-actionWidget:only-child > .oo-ui-buttonElement-button {
  border-radius: 0 0 2px 2px;
}
.oo-ui-messageDialog-actions-vertical .oo-ui-actionWidget {
  border-bottom: 1px solid #a2a9b1;
  margin: 0;
}
.oo-ui-messageDialog-actions-vertical .oo-ui-actionWidget:last-child {
  border-bottom-width: 0;
}
.oo-ui-messageDialog-actions-vertical .oo-ui-actionWidget:last-child > .oo-ui-buttonElement-button {
  border-radius: 0 0 2px 2px;
}

.oo-ui-processDialog-location {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}
.oo-ui-processDialog-title {
  display: inline;
  padding: 0;
}
.oo-ui-processDialog-actions-safe .oo-ui-actionWidget,
.oo-ui-processDialog-actions-primary .oo-ui-actionWidget,
.oo-ui-processDialog-actions-other .oo-ui-actionWidget {
  white-space: nowrap;
}
.oo-ui-processDialog-actions-safe,
.oo-ui-processDialog-actions-primary {
  position: absolute;
  top: 0;
  bottom: 0;
}
.oo-ui-processDialog-actions-safe {
  right: 0;
}
.oo-ui-processDialog-actions-primary {
  left: 0;
}
.oo-ui-processDialog-errors {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  bottom: 0;
  z-index: 4;
  overflow-x: hidden;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
}
.oo-ui-processDialog-content .oo-ui-window-head,
.oo-ui-processDialog-content .oo-ui-window-foot {
  outline: 1px solid #c8ccd1;
}
.oo-ui-processDialog-content .oo-ui-window-head {
  min-height: 2.85714286em;
}
.oo-ui-processDialog-content .oo-ui-window-body {
  top: 2.85714286em;
}
.oo-ui-processDialog-navigation {
  position: relative;
  min-height: 2.85714286em;
}
.oo-ui-processDialog-location {
  min-height: 2.85714286em;
  text-align: center;
  cursor: default;
}
.oo-ui-processDialog-title {
  font-size: 1.1em;
  font-weight: bold;
  line-height: 2.5974026em;
}
.oo-ui-processDialog .oo-ui-actionWidget > .oo-ui-buttonElement-button {
  border-radius: 0;
  border-top-color: transparent;
  border-bottom-color: transparent;
}
.oo-ui-processDialog .oo-ui-actionWidget > .oo-ui-buttonElement-button:hover,
.oo-ui-processDialog .oo-ui-actionWidget > .oo-ui-buttonElement-button:active {
  border-top-color: transparent;
  border-bottom-color: transparent;
}
.oo-ui-processDialog .oo-ui-actionWidget:not( .oo-ui-flaggedElement-progressive ):not( .oo-ui-flaggedElement-destructive ) > .oo-ui-buttonElement-button:hover,
.oo-ui-processDialog .oo-ui-actionWidget:not( .oo-ui-flaggedElement-progressive ):not( .oo-ui-flaggedElement-destructive ) > .oo-ui-buttonElement-button:active {
  border-left-color: #c8ccd1;
  border-right-color: transparent;
}
.oo-ui-processDialog .oo-ui-actionWidget:not( .oo-ui-flaggedElement-progressive ):not( .oo-ui-flaggedElement-destructive ) > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
}
.oo-ui-processDialog .oo-ui-actionWidget.oo-ui-iconElement > .oo-ui-buttonElement-button {
  min-height: 2.85714286em;
  padding-top: 0;
  padding-right: 3.14285714em;
  padding-left: 0;
}
.oo-ui-processDialog .oo-ui-actionWidget.oo-ui-labelElement:first-child {
  margin-right: 0;
}
.oo-ui-processDialog .oo-ui-actionWidget.oo-ui-labelElement > .oo-ui-buttonElement-button {
  padding: 0.71428571em 12px;
}
.oo-ui-processDialog .oo-ui-actionWidget.oo-ui-labelElement.oo-ui-iconElement > .oo-ui-buttonElement-button {
  padding-right: 2.42857143em;
}
.oo-ui-processDialog-actions-primary .oo-ui-actionWidget > .oo-ui-buttonElement-button {
  border-right-color: #c8ccd1;
  border-left-color: transparent;
}
.oo-ui-processDialog-actions-primary .oo-ui-actionWidget:not( .oo-ui-flaggedElement-progressive ):not( .oo-ui-flaggedElement-destructive ) > .oo-ui-buttonElement-button:hover,
.oo-ui-processDialog-actions-primary .oo-ui-actionWidget:not( .oo-ui-flaggedElement-progressive ):not( .oo-ui-flaggedElement-destructive ) > .oo-ui-buttonElement-button:active {
  border-right-color: #c8ccd1;
  border-left-color: transparent;
}
.oo-ui-processDialog-actions-primary .oo-ui-actionWidget:not( .oo-ui-flaggedElement-progressive ):not( .oo-ui-flaggedElement-destructive ) > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
}
.oo-ui-processDialog-actions-safe .oo-ui-actionWidget > .oo-ui-buttonElement-button,
.oo-ui-processDialog-actions-other .oo-ui-actionWidget > .oo-ui-buttonElement-button {
  border-left-color: #c8ccd1;
  border-right-color: transparent;
}
.oo-ui-processDialog-actions-safe .oo-ui-widget-enabled.oo-ui-iconElement:not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button {
  background-color: transparent;
  border-left-color: transparent;
}
.oo-ui-processDialog-actions-safe .oo-ui-widget-enabled.oo-ui-iconElement:not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button:hover {
  background-color: #f8f9fa;
  border-left-color: #c8ccd1;
}
.oo-ui-processDialog-actions-safe .oo-ui-widget-enabled.oo-ui-iconElement:not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button:active {
  background-color: #c8ccd1;
  border-left-color: #c8ccd1;
}
.oo-ui-processDialog-actions-safe .oo-ui-widget-enabled.oo-ui-iconElement:not( .oo-ui-labelElement ) > .oo-ui-buttonElement-button:focus {
  border-color: #36c;
}
.oo-ui-processDialog-actions-other .oo-ui-actionWidget.oo-ui-buttonElement {
  margin-left: 0;
}
.oo-ui-processDialog > .oo-ui-window-frame {
  min-height: 5em;
}
.oo-ui-processDialog-errors {
  background-color: rgba(255, 255, 255, 0.9);
  padding: 3em 3em 0 3em;
}
.oo-ui-processDialog-errors-title {
  font-size: 1.5em;
  color: #000;
  margin-bottom: 1.5em;
  text-align: center;
}
.oo-ui-processDialog-errors > .oo-ui-messageWidget {
  margin: 1em 0 0;
}
.oo-ui-processDialog-errors-actions {
  text-align: center;
}
.oo-ui-processDialog-errors-actions > .oo-ui-buttonWidget {
  margin: 2em 1em;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-processDialog-content .oo-ui-window-head,
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-processDialog-navigation,
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-processDialog-location {
  min-height: 3.14285714em;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-processDialog-location {
  margin-right: 1em;
  text-align: right;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-processDialog-title {
  line-height: 2.85714286em;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-processDialog-content .oo-ui-window-body {
  top: 3.14285714em;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-actionWidget.oo-ui-iconElement > .oo-ui-buttonElement-button {
  min-height: 3.14285714em;
  padding-right: 3.14285714em;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-actionWidget.oo-ui-labelElement > .oo-ui-buttonElement-button {
  padding: 0.85714286em 12px;
}
.oo-ui-processDialog.oo-ui-isMobile .oo-ui-actionWidget.oo-ui-labelElement.oo-ui-iconElement > .oo-ui-buttonElement-button {
  padding-right: 2.42857143em;
}

.oo-ui-windowManager-modal > .oo-ui-dialog {
  position: fixed;
  width: 0;
  height: 0;
  overflow: hidden;
  z-index: 4;
}
.oo-ui-windowManager-modal > .oo-ui-dialog.oo-ui-window-active {
  width: auto;
  height: auto;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  padding: 1em;
}
.oo-ui-windowManager-modal > .oo-ui-dialog.oo-ui-window-active > .oo-ui-window-frame {
  position: absolute;
  left: 0;
  right: 0;
  margin: auto;
  max-width: 100%;
  max-height: 100%;
}
.oo-ui-windowManager-fullscreen > .oo-ui-dialog > .oo-ui-window-frame {
  width: 100%;
  height: 100%;
  top: 0;
  bottom: 0;
}
.oo-ui-windowManager-modal-active {
  overflow: hidden;
  position: relative;
}
.oo-ui-windowManager-ios-modal-ready {
  height: 100%;
  overflow: hidden;
}
.oo-ui-windowManager-modal > .oo-ui-dialog {
  background-color: rgba(255, 255, 255, 0.5);
  opacity: 0;
  -webkit-transition: opacity ease-out 250ms;
  -moz-transition: opacity ease-out 250ms;
  transition: opacity ease-out 250ms;
}
.oo-ui-windowManager-modal > .oo-ui-dialog > .oo-ui-window-frame {
  background-color: #fff;
  opacity: 0;
  -webkit-transform: scale(0.5);
  -moz-transform: scale(0.5);
  -ms-transform: scale(0.5);
  transform: scale(0.5);
  -webkit-transition: all ease-out 250ms;
  -moz-transition: all ease-out 250ms;
  transition: all ease-out 250ms;
}
.oo-ui-windowManager-modal > .oo-ui-dialog.oo-ui-window-setup {
  opacity: 1;
}
.oo-ui-windowManager-modal > .oo-ui-dialog.oo-ui-window-setup > .oo-ui-window-frame {
  opacity: 1;
  -webkit-transform: scale(1);
  -moz-transform: scale(1);
  -ms-transform: scale(1);
  transform: scale(1);
}
.oo-ui-windowManager-modal.oo-ui-windowManager-floating > .oo-ui-dialog > .oo-ui-window-frame {
  top: 1em;
  bottom: 1em;
  max-height: 100%;
  max-height: calc( 100% - 2em );
  border: 1px solid #a2a9b1;
  border-radius: 2px;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.25);
}

/*!
 * OOUI v0.39.3
 * https://www.mediawiki.org/wiki/OOUI
 *
 * Copyright 2011–2020 OOUI Team and other contributors.
 * Released under the MIT license
 * http://oojs.mit-license.org
 *
 * Date: 2020-07-10T06:32:25Z
 */
.oo-ui-icon-arrowNext {
  background-image: url('themes/wikimediaui/images/icons/arrowNext-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/arrowNext-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-arrowNext {
  background-image: url('themes/wikimediaui/images/icons/arrowNext-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/arrowNext-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-arrowNext {
  background-image: url('themes/wikimediaui/images/icons/arrowNext-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/arrowNext-rtl-progressive.svg');
}
.oo-ui-icon-arrowPrevious {
  background-image: url('themes/wikimediaui/images/icons/arrowPrevious-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/arrowPrevious-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-arrowPrevious {
  background-image: url('themes/wikimediaui/images/icons/arrowPrevious-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/arrowPrevious-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-arrowPrevious {
  background-image: url('themes/wikimediaui/images/icons/arrowPrevious-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/arrowPrevious-rtl-progressive.svg');
}
.oo-ui-icon-collapse {
  background-image: url('themes/wikimediaui/images/icons/collapse.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/collapse.svg');
}
.oo-ui-image-invert.oo-ui-icon-collapse {
  background-image: url('themes/wikimediaui/images/icons/collapse-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/collapse-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-collapse {
  background-image: url('themes/wikimediaui/images/icons/collapse-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/collapse-progressive.svg');
}
.oo-ui-icon-downTriangle {
  background-image: url('themes/wikimediaui/images/icons/downTriangle.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/downTriangle.svg');
}
.oo-ui-image-invert.oo-ui-icon-downTriangle {
  background-image: url('themes/wikimediaui/images/icons/downTriangle-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/downTriangle-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-downTriangle {
  background-image: url('themes/wikimediaui/images/icons/downTriangle-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/downTriangle-progressive.svg');
}
.oo-ui-icon-draggable {
  background-image: url('themes/wikimediaui/images/icons/draggable.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/draggable.svg');
}
.oo-ui-image-invert.oo-ui-icon-draggable {
  background-image: url('themes/wikimediaui/images/icons/draggable-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/draggable-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-draggable {
  background-image: url('themes/wikimediaui/images/icons/draggable-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/draggable-progressive.svg');
}
.oo-ui-icon-doubleChevronEnd {
  background-image: url('themes/wikimediaui/images/icons/doubleChevronEnd-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/doubleChevronEnd-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-doubleChevronEnd {
  background-image: url('themes/wikimediaui/images/icons/doubleChevronEnd-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/doubleChevronEnd-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-doubleChevronEnd {
  background-image: url('themes/wikimediaui/images/icons/doubleChevronEnd-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/doubleChevronEnd-rtl-progressive.svg');
}
.oo-ui-icon-doubleChevronStart {
  background-image: url('themes/wikimediaui/images/icons/doubleChevronStart-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/doubleChevronStart-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-doubleChevronStart {
  background-image: url('themes/wikimediaui/images/icons/doubleChevronStart-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/doubleChevronStart-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-doubleChevronStart {
  background-image: url('themes/wikimediaui/images/icons/doubleChevronStart-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/doubleChevronStart-rtl-progressive.svg');
}
.oo-ui-icon-expand {
  background-image: url('themes/wikimediaui/images/icons/expand.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/expand.svg');
}
.oo-ui-image-invert.oo-ui-icon-expand {
  background-image: url('themes/wikimediaui/images/icons/expand-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/expand-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-expand {
  background-image: url('themes/wikimediaui/images/icons/expand-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/expand-progressive.svg');
}
.oo-ui-icon-move {
  background-image: url('themes/wikimediaui/images/icons/move.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/move.svg');
}
.oo-ui-image-invert.oo-ui-icon-move {
  background-image: url('themes/wikimediaui/images/icons/move-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/move-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-move {
  background-image: url('themes/wikimediaui/images/icons/move-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/move-progressive.svg');
}
.oo-ui-icon-next {
  background-image: url('themes/wikimediaui/images/icons/next-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/next-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-next {
  background-image: url('themes/wikimediaui/images/icons/next-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/next-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-next {
  background-image: url('themes/wikimediaui/images/icons/next-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/next-rtl-progressive.svg');
}
.oo-ui-icon-previous {
  background-image: url('themes/wikimediaui/images/icons/previous-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/previous-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-previous {
  background-image: url('themes/wikimediaui/images/icons/previous-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/previous-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-previous {
  background-image: url('themes/wikimediaui/images/icons/previous-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/previous-rtl-progressive.svg');
}
.oo-ui-icon-last {
  background-image: url('themes/wikimediaui/images/icons/moveLast-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moveLast-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-last {
  background-image: url('themes/wikimediaui/images/icons/moveLast-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moveLast-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-last {
  background-image: url('themes/wikimediaui/images/icons/moveLast-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moveLast-rtl-progressive.svg');
}
.oo-ui-icon-first {
  background-image: url('themes/wikimediaui/images/icons/moveFirst-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moveFirst-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-first {
  background-image: url('themes/wikimediaui/images/icons/moveFirst-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moveFirst-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-first {
  background-image: url('themes/wikimediaui/images/icons/moveFirst-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moveFirst-rtl-progressive.svg');
}
.oo-ui-icon-upTriangle {
  background-image: url('themes/wikimediaui/images/icons/upTriangle.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upTriangle.svg');
}
.oo-ui-image-invert.oo-ui-icon-upTriangle {
  background-image: url('themes/wikimediaui/images/icons/upTriangle-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upTriangle-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-upTriangle {
  background-image: url('themes/wikimediaui/images/icons/upTriangle-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upTriangle-progressive.svg');
}

.oo-ui-icon-article {
  background-image: url('themes/wikimediaui/images/icons/article-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/article-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-article {
  background-image: url('themes/wikimediaui/images/icons/article-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/article-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-article {
  background-image: url('themes/wikimediaui/images/icons/article-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/article-rtl-progressive.svg');
}
.oo-ui-icon-articles {
  background-image: url('themes/wikimediaui/images/icons/articles-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articles-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-articles {
  background-image: url('themes/wikimediaui/images/icons/articles-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articles-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articles {
  background-image: url('themes/wikimediaui/images/icons/articles-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articles-rtl-progressive.svg');
}
.oo-ui-icon-articleAdd {
  background-image: url('themes/wikimediaui/images/icons/articleAdd.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleAdd.svg');
}
.oo-ui-image-invert.oo-ui-icon-articleAdd {
  background-image: url('themes/wikimediaui/images/icons/articleAdd-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleAdd-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articleAdd {
  background-image: url('themes/wikimediaui/images/icons/articleAdd-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleAdd-progressive.svg');
}
.oo-ui-icon-articleCheck {
  background-image: url('themes/wikimediaui/images/icons/articleCheck.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleCheck.svg');
}
.oo-ui-image-invert.oo-ui-icon-articleCheck {
  background-image: url('themes/wikimediaui/images/icons/articleCheck-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleCheck-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articleCheck {
  background-image: url('themes/wikimediaui/images/icons/articleCheck-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleCheck-progressive.svg');
}
.oo-ui-icon-articleDisambiguation {
  background-image: url('themes/wikimediaui/images/icons/articleDisambiguation-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleDisambiguation-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-articleDisambiguation {
  background-image: url('themes/wikimediaui/images/icons/articleDisambiguation-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleDisambiguation-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articleDisambiguation {
  background-image: url('themes/wikimediaui/images/icons/articleDisambiguation-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleDisambiguation-rtl-progressive.svg');
}
.oo-ui-icon-articleNotFound {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-rtl.svg');
}
/* @noflip */
.oo-ui-icon-articleNotFound:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr.svg');
}
/* @noflip */
.oo-ui-icon-articleNotFound:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-articleNotFound {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-rtl-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-articleNotFound:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-articleNotFound:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articleNotFound {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-rtl-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-articleNotFound:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-articleNotFound:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/articleNotFound-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleNotFound-ltr-progressive.svg');
}
.oo-ui-icon-articleSearch {
  background-image: url('themes/wikimediaui/images/icons/articleSearch.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleSearch.svg');
}
.oo-ui-image-invert.oo-ui-icon-articleSearch {
  background-image: url('themes/wikimediaui/images/icons/articleSearch-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleSearch-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articleSearch {
  background-image: url('themes/wikimediaui/images/icons/articleSearch-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleSearch-progressive.svg');
}
.oo-ui-icon-articlesSearch {
  background-image: url('themes/wikimediaui/images/icons/articlesSearch-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articlesSearch-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-articlesSearch {
  background-image: url('themes/wikimediaui/images/icons/articlesSearch-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articlesSearch-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articlesSearch {
  background-image: url('themes/wikimediaui/images/icons/articlesSearch-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articlesSearch-rtl-progressive.svg');
}
.oo-ui-icon-articleRedirect {
  background-image: url('themes/wikimediaui/images/icons/articleRedirect-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleRedirect-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-articleRedirect {
  background-image: url('themes/wikimediaui/images/icons/articleRedirect-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleRedirect-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-articleRedirect {
  background-image: url('themes/wikimediaui/images/icons/articleRedirect-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/articleRedirect-rtl-progressive.svg');
}
.oo-ui-icon-die {
  background-image: url('themes/wikimediaui/images/icons/die.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/die.svg');
}
.oo-ui-image-invert.oo-ui-icon-die {
  background-image: url('themes/wikimediaui/images/icons/die-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/die-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-die {
  background-image: url('themes/wikimediaui/images/icons/die-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/die-progressive.svg');
}
.oo-ui-icon-download {
  background-image: url('themes/wikimediaui/images/icons/download.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/download.svg');
}
.oo-ui-image-invert.oo-ui-icon-download {
  background-image: url('themes/wikimediaui/images/icons/download-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/download-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-download {
  background-image: url('themes/wikimediaui/images/icons/download-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/download-progressive.svg');
}
.oo-ui-icon-folderPlaceholder {
  background-image: url('themes/wikimediaui/images/icons/folderPlaceholder-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/folderPlaceholder-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-folderPlaceholder {
  background-image: url('themes/wikimediaui/images/icons/folderPlaceholder-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/folderPlaceholder-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-folderPlaceholder {
  background-image: url('themes/wikimediaui/images/icons/folderPlaceholder-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/folderPlaceholder-rtl-progressive.svg');
}
.oo-ui-icon-history {
  background-image: url('themes/wikimediaui/images/icons/history.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/history.svg');
}
.oo-ui-image-invert.oo-ui-icon-history {
  background-image: url('themes/wikimediaui/images/icons/history-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/history-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-history {
  background-image: url('themes/wikimediaui/images/icons/history-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/history-progressive.svg');
}
.oo-ui-icon-info {
  background-image: url('themes/wikimediaui/images/icons/info.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/info.svg');
}
/* @noflip */
.oo-ui-icon-info:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb.svg');
}
.oo-ui-image-invert.oo-ui-icon-info {
  background-image: url('themes/wikimediaui/images/icons/info-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/info-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-info:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-info {
  background-image: url('themes/wikimediaui/images/icons/info-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/info-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-info:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb-progressive.svg');
}
.oo-ui-icon-infoFilled {
  background-image: url('themes/wikimediaui/images/icons/infoFilled.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/infoFilled.svg');
}
/* @noflip */
.oo-ui-icon-infoFilled:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb.svg');
}
.oo-ui-image-invert.oo-ui-icon-infoFilled {
  background-image: url('themes/wikimediaui/images/icons/infoFilled-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/infoFilled-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-infoFilled:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-infoFilled {
  background-image: url('themes/wikimediaui/images/icons/infoFilled-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/infoFilled-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-infoFilled:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/lightbulb-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb-progressive.svg');
}
.oo-ui-icon-robot {
  background-image: url('themes/wikimediaui/images/icons/robot.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/robot.svg');
}
.oo-ui-image-invert.oo-ui-icon-robot {
  background-image: url('themes/wikimediaui/images/icons/robot-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/robot-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-robot {
  background-image: url('themes/wikimediaui/images/icons/robot-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/robot-progressive.svg');
}
.oo-ui-icon-specialPages {
  background-image: url('themes/wikimediaui/images/icons/specialPages-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialPages-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-specialPages {
  background-image: url('themes/wikimediaui/images/icons/specialPages-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialPages-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-specialPages {
  background-image: url('themes/wikimediaui/images/icons/specialPages-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialPages-rtl-progressive.svg');
}
.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-rtl.svg');
}
.oo-ui-image-destructive.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-rtl-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-rtl-destructive.svg');
}
.oo-ui-image-error.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-rtl-error.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-rtl-error.svg');
}
.oo-ui-image-warning.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-rtl-warning.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-rtl-warning.svg');
}
.oo-ui-image-invert.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tag {
  background-image: url('themes/wikimediaui/images/icons/tag-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tag-rtl-progressive.svg');
}
.oo-ui-icon-upload {
  background-image: url('themes/wikimediaui/images/icons/upload.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upload.svg');
}
.oo-ui-image-invert.oo-ui-icon-upload {
  background-image: url('themes/wikimediaui/images/icons/upload-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upload-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-upload {
  background-image: url('themes/wikimediaui/images/icons/upload-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/upload-progressive.svg');
}
.oo-ui-icon-window {
  background-image: url('themes/wikimediaui/images/icons/window.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/window.svg');
}
.oo-ui-image-invert.oo-ui-icon-window {
  background-image: url('themes/wikimediaui/images/icons/window-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/window-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-window {
  background-image: url('themes/wikimediaui/images/icons/window-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/window-progressive.svg');
}

.oo-ui-icon-alert {
  background-image: url('themes/wikimediaui/images/icons/alert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alert.svg');
}
.oo-ui-image-warning.oo-ui-icon-alert {
  background-image: url('themes/wikimediaui/images/icons/alert-warning.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alert-warning.svg');
}
.oo-ui-image-invert.oo-ui-icon-alert {
  background-image: url('themes/wikimediaui/images/icons/alert-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alert-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-alert {
  background-image: url('themes/wikimediaui/images/icons/alert-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alert-progressive.svg');
}
.oo-ui-icon-bell {
  background-image: url('themes/wikimediaui/images/icons/bell.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bell.svg');
}
.oo-ui-image-invert.oo-ui-icon-bell {
  background-image: url('themes/wikimediaui/images/icons/bell-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bell-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bell {
  background-image: url('themes/wikimediaui/images/icons/bell-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bell-progressive.svg');
}
.oo-ui-icon-bellOutline {
  background-image: url('themes/wikimediaui/images/icons/bellOutline.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bellOutline.svg');
}
.oo-ui-image-invert.oo-ui-icon-bellOutline {
  background-image: url('themes/wikimediaui/images/icons/bellOutline-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bellOutline-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bellOutline {
  background-image: url('themes/wikimediaui/images/icons/bellOutline-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bellOutline-progressive.svg');
}
.oo-ui-icon-error {
  background-image: url('themes/wikimediaui/images/icons/error.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/error.svg');
}
.oo-ui-image-error.oo-ui-icon-error {
  background-image: url('themes/wikimediaui/images/icons/error-error.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/error-error.svg');
}
.oo-ui-image-invert.oo-ui-icon-error {
  background-image: url('themes/wikimediaui/images/icons/error-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/error-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-error {
  background-image: url('themes/wikimediaui/images/icons/error-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/error-progressive.svg');
}
.oo-ui-icon-message {
  background-image: url('themes/wikimediaui/images/icons/message.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/message.svg');
}
.oo-ui-image-invert.oo-ui-icon-message {
  background-image: url('themes/wikimediaui/images/icons/message-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/message-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-message {
  background-image: url('themes/wikimediaui/images/icons/message-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/message-progressive.svg');
}
.oo-ui-icon-notice {
  background-image: url('themes/wikimediaui/images/icons/notice.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notice.svg');
}
.oo-ui-image-invert.oo-ui-icon-notice {
  background-image: url('themes/wikimediaui/images/icons/notice-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notice-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-notice {
  background-image: url('themes/wikimediaui/images/icons/notice-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notice-progressive.svg');
}
.oo-ui-icon-speechBubble {
  background-image: url('themes/wikimediaui/images/icons/speechBubble-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubble-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-speechBubble {
  background-image: url('themes/wikimediaui/images/icons/speechBubble-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubble-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-speechBubble {
  background-image: url('themes/wikimediaui/images/icons/speechBubble-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubble-rtl-progressive.svg');
}
.oo-ui-icon-speechBubbleAdd {
  background-image: url('themes/wikimediaui/images/icons/speechBubbleAdd-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubbleAdd-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-speechBubbleAdd {
  background-image: url('themes/wikimediaui/images/icons/speechBubbleAdd-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubbleAdd-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-speechBubbleAdd {
  background-image: url('themes/wikimediaui/images/icons/speechBubbleAdd-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubbleAdd-rtl-progressive.svg');
}
.oo-ui-icon-speechBubbles {
  background-image: url('themes/wikimediaui/images/icons/speechBubbles-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubbles-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-speechBubbles {
  background-image: url('themes/wikimediaui/images/icons/speechBubbles-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubbles-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-speechBubbles {
  background-image: url('themes/wikimediaui/images/icons/speechBubbles-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/speechBubbles-rtl-progressive.svg');
}
.oo-ui-icon-tray {
  background-image: url('themes/wikimediaui/images/icons/tray.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tray.svg');
}
.oo-ui-image-invert.oo-ui-icon-tray {
  background-image: url('themes/wikimediaui/images/icons/tray-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tray-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tray {
  background-image: url('themes/wikimediaui/images/icons/tray-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tray-progressive.svg');
}

.oo-ui-icon-add {
  background-image: url('themes/wikimediaui/images/icons/add.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/add.svg');
}
.oo-ui-image-invert.oo-ui-icon-add {
  background-image: url('themes/wikimediaui/images/icons/add-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/add-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-add {
  background-image: url('themes/wikimediaui/images/icons/add-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/add-progressive.svg');
}
.oo-ui-icon-browser {
  background-image: url('themes/wikimediaui/images/icons/browser-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/browser-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-browser {
  background-image: url('themes/wikimediaui/images/icons/browser-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/browser-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-browser {
  background-image: url('themes/wikimediaui/images/icons/browser-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/browser-rtl-progressive.svg');
}
.oo-ui-icon-cancel {
  background-image: url('themes/wikimediaui/images/icons/cancel.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/cancel.svg');
}
.oo-ui-image-destructive.oo-ui-icon-cancel {
  background-image: url('themes/wikimediaui/images/icons/cancel-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/cancel-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-cancel {
  background-image: url('themes/wikimediaui/images/icons/cancel-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/cancel-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-cancel {
  background-image: url('themes/wikimediaui/images/icons/cancel-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/cancel-progressive.svg');
}
.oo-ui-icon-check {
  background-image: url('themes/wikimediaui/images/icons/check.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/check.svg');
}
.oo-ui-image-destructive.oo-ui-icon-check {
  background-image: url('themes/wikimediaui/images/icons/check-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/check-destructive.svg');
}
.oo-ui-image-success.oo-ui-icon-check {
  background-image: url('themes/wikimediaui/images/icons/check-success.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/check-success.svg');
}
.oo-ui-image-invert.oo-ui-icon-check {
  background-image: url('themes/wikimediaui/images/icons/check-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/check-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-check {
  background-image: url('themes/wikimediaui/images/icons/check-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/check-progressive.svg');
}
.oo-ui-icon-checkAll {
  background-image: url('themes/wikimediaui/images/icons/checkAll.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/checkAll.svg');
}
.oo-ui-image-invert.oo-ui-icon-checkAll {
  background-image: url('themes/wikimediaui/images/icons/checkAll-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/checkAll-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-checkAll {
  background-image: url('themes/wikimediaui/images/icons/checkAll-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/checkAll-progressive.svg');
}
.oo-ui-icon-clear {
  background-image: url('themes/wikimediaui/images/icons/clear.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/clear.svg');
}
.oo-ui-image-invert.oo-ui-icon-clear {
  background-image: url('themes/wikimediaui/images/icons/clear-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/clear-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-clear {
  background-image: url('themes/wikimediaui/images/icons/clear-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/clear-progressive.svg');
}
.oo-ui-icon-clock {
  background-image: url('themes/wikimediaui/images/icons/clock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/clock.svg');
}
.oo-ui-image-invert.oo-ui-icon-clock {
  background-image: url('themes/wikimediaui/images/icons/clock-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/clock-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-clock {
  background-image: url('themes/wikimediaui/images/icons/clock-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/clock-progressive.svg');
}
.oo-ui-icon-close {
  background-image: url('themes/wikimediaui/images/icons/close.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/close.svg');
}
.oo-ui-image-invert.oo-ui-icon-close {
  background-image: url('themes/wikimediaui/images/icons/close-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/close-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-close {
  background-image: url('themes/wikimediaui/images/icons/close-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/close-progressive.svg');
}
.oo-ui-icon-ellipsis {
  background-image: url('themes/wikimediaui/images/icons/ellipsis.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ellipsis.svg');
}
.oo-ui-image-invert.oo-ui-icon-ellipsis {
  background-image: url('themes/wikimediaui/images/icons/ellipsis-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ellipsis-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-ellipsis {
  background-image: url('themes/wikimediaui/images/icons/ellipsis-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ellipsis-progressive.svg');
}
.oo-ui-icon-feedback {
  background-image: url('themes/wikimediaui/images/icons/feedback-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/feedback-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-feedback {
  background-image: url('themes/wikimediaui/images/icons/feedback-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/feedback-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-feedback {
  background-image: url('themes/wikimediaui/images/icons/feedback-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/feedback-rtl-progressive.svg');
}
.oo-ui-icon-funnel {
  background-image: url('themes/wikimediaui/images/icons/funnel-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/funnel-rtl.svg');
}
.oo-ui-image-destructive.oo-ui-icon-funnel {
  background-image: url('themes/wikimediaui/images/icons/funnel-rtl-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/funnel-rtl-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-funnel {
  background-image: url('themes/wikimediaui/images/icons/funnel-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/funnel-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-funnel {
  background-image: url('themes/wikimediaui/images/icons/funnel-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/funnel-rtl-progressive.svg');
}
.oo-ui-icon-heart {
  background-image: url('themes/wikimediaui/images/icons/heart.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/heart.svg');
}
.oo-ui-image-invert.oo-ui-icon-heart {
  background-image: url('themes/wikimediaui/images/icons/heart-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/heart-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-heart {
  background-image: url('themes/wikimediaui/images/icons/heart-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/heart-progressive.svg');
}
.oo-ui-icon-help {
  background-image: url('themes/wikimediaui/images/icons/help-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-rtl.svg');
}
/* @noflip */
.oo-ui-icon-help:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/help-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-ltr.svg');
}
/* @noflip */
.oo-ui-icon-help:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/help-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-help {
  background-image: url('themes/wikimediaui/images/icons/help-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-rtl-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-help:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/help-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-ltr-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-help:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/help-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-help {
  background-image: url('themes/wikimediaui/images/icons/help-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-rtl-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-help:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/help-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-ltr-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-help:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/help-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/help-ltr-progressive.svg');
}
.oo-ui-icon-helpNotice {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-rtl.svg');
}
/* @noflip */
.oo-ui-icon-helpNotice:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-ltr.svg');
}
/* @noflip */
.oo-ui-icon-helpNotice:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-ltr.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-ltr.svg');
}
.oo-ui-image-invert.oo-ui-icon-helpNotice {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-rtl-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-helpNotice:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-ltr-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-helpNotice:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-ltr-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-ltr-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-helpNotice {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-rtl-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-helpNotice:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-ltr-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-helpNotice:lang(yi) {
  background-image: url('themes/wikimediaui/images/icons/helpNotice-ltr-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/helpNotice-ltr-progressive.svg');
}
.oo-ui-icon-home {
  background-image: url('themes/wikimediaui/images/icons/home.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/home.svg');
}
.oo-ui-image-invert.oo-ui-icon-home {
  background-image: url('themes/wikimediaui/images/icons/home-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/home-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-home {
  background-image: url('themes/wikimediaui/images/icons/home-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/home-progressive.svg');
}
.oo-ui-icon-key {
  background-image: url('themes/wikimediaui/images/icons/key.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/key.svg');
}
.oo-ui-image-invert.oo-ui-icon-key {
  background-image: url('themes/wikimediaui/images/icons/key-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/key-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-key {
  background-image: url('themes/wikimediaui/images/icons/key-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/key-progressive.svg');
}
.oo-ui-icon-keyboard {
  background-image: url('themes/wikimediaui/images/icons/keyboard.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/keyboard.svg');
}
.oo-ui-image-invert.oo-ui-icon-keyboard {
  background-image: url('themes/wikimediaui/images/icons/keyboard-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/keyboard-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-keyboard {
  background-image: url('themes/wikimediaui/images/icons/keyboard-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/keyboard-progressive.svg');
}
.oo-ui-icon-lightbulb {
  background-image: url('themes/wikimediaui/images/icons/lightbulb.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb.svg');
}
.oo-ui-image-invert.oo-ui-icon-lightbulb {
  background-image: url('themes/wikimediaui/images/icons/lightbulb-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-lightbulb {
  background-image: url('themes/wikimediaui/images/icons/lightbulb-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lightbulb-progressive.svg');
}
.oo-ui-icon-logIn {
  background-image: url('themes/wikimediaui/images/icons/logIn-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logIn-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-logIn {
  background-image: url('themes/wikimediaui/images/icons/logIn-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logIn-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-logIn {
  background-image: url('themes/wikimediaui/images/icons/logIn-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logIn-rtl-progressive.svg');
}
.oo-ui-icon-logOut {
  background-image: url('themes/wikimediaui/images/icons/logOut-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logOut-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-logOut {
  background-image: url('themes/wikimediaui/images/icons/logOut-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logOut-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-logOut {
  background-image: url('themes/wikimediaui/images/icons/logOut-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logOut-rtl-progressive.svg');
}
.oo-ui-icon-newWindow {
  background-image: url('themes/wikimediaui/images/icons/newWindow-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newWindow-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-newWindow {
  background-image: url('themes/wikimediaui/images/icons/newWindow-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newWindow-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-newWindow {
  background-image: url('themes/wikimediaui/images/icons/newWindow-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newWindow-rtl-progressive.svg');
}
.oo-ui-icon-pageSettings {
  background-image: url('themes/wikimediaui/images/icons/pageSettings.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pageSettings.svg');
}
.oo-ui-image-invert.oo-ui-icon-pageSettings {
  background-image: url('themes/wikimediaui/images/icons/pageSettings-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pageSettings-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-pageSettings {
  background-image: url('themes/wikimediaui/images/icons/pageSettings-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pageSettings-progressive.svg');
}
.oo-ui-icon-printer {
  background-image: url('themes/wikimediaui/images/icons/printer.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/printer.svg');
}
.oo-ui-image-invert.oo-ui-icon-printer {
  background-image: url('themes/wikimediaui/images/icons/printer-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/printer-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-printer {
  background-image: url('themes/wikimediaui/images/icons/printer-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/printer-progressive.svg');
}
.oo-ui-icon-reload {
  background-image: url('themes/wikimediaui/images/icons/reload.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/reload.svg');
}
.oo-ui-image-invert.oo-ui-icon-reload {
  background-image: url('themes/wikimediaui/images/icons/reload-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/reload-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-reload {
  background-image: url('themes/wikimediaui/images/icons/reload-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/reload-progressive.svg');
}
.oo-ui-icon-search {
  background-image: url('themes/wikimediaui/images/icons/search.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/search.svg');
}
.oo-ui-image-invert.oo-ui-icon-search {
  background-image: url('themes/wikimediaui/images/icons/search-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/search-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-search {
  background-image: url('themes/wikimediaui/images/icons/search-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/search-progressive.svg');
}
.oo-ui-icon-settings {
  background-image: url('themes/wikimediaui/images/icons/settings.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/settings.svg');
}
.oo-ui-image-invert.oo-ui-icon-settings {
  background-image: url('themes/wikimediaui/images/icons/settings-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/settings-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-settings {
  background-image: url('themes/wikimediaui/images/icons/settings-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/settings-progressive.svg');
}
.oo-ui-icon-stopHand {
  background-image: url('themes/wikimediaui/images/icons/stopHand.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stopHand.svg');
}
.oo-ui-image-destructive.oo-ui-icon-stopHand {
  background-image: url('themes/wikimediaui/images/icons/stopHand-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stopHand-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-stopHand {
  background-image: url('themes/wikimediaui/images/icons/stopHand-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stopHand-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-stopHand {
  background-image: url('themes/wikimediaui/images/icons/stopHand-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stopHand-progressive.svg');
}
.oo-ui-icon-subtract {
  background-image: url('themes/wikimediaui/images/icons/subtract.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subtract.svg');
}
.oo-ui-image-destructive.oo-ui-icon-subtract {
  background-image: url('themes/wikimediaui/images/icons/subtract-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subtract-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-subtract {
  background-image: url('themes/wikimediaui/images/icons/subtract-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subtract-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-subtract {
  background-image: url('themes/wikimediaui/images/icons/subtract-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subtract-progressive.svg');
}

.oo-ui-icon-bookmarkOutline {
  background-image: url('themes/wikimediaui/images/icons/bookmarkOutline.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmarkOutline.svg');
}
.oo-ui-image-invert.oo-ui-icon-bookmarkOutline {
  background-image: url('themes/wikimediaui/images/icons/bookmarkOutline-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmarkOutline-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bookmarkOutline {
  background-image: url('themes/wikimediaui/images/icons/bookmarkOutline-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmarkOutline-progressive.svg');
}
.oo-ui-icon-bookmark {
  background-image: url('themes/wikimediaui/images/icons/bookmark.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmark.svg');
}
.oo-ui-image-invert.oo-ui-icon-bookmark {
  background-image: url('themes/wikimediaui/images/icons/bookmark-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmark-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bookmark {
  background-image: url('themes/wikimediaui/images/icons/bookmark-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bookmark-progressive.svg');
}
.oo-ui-icon-block {
  background-image: url('themes/wikimediaui/images/icons/block.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/block.svg');
}
.oo-ui-image-destructive.oo-ui-icon-block {
  background-image: url('themes/wikimediaui/images/icons/block-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/block-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-block {
  background-image: url('themes/wikimediaui/images/icons/block-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/block-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-block {
  background-image: url('themes/wikimediaui/images/icons/block-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/block-progressive.svg');
}
.oo-ui-icon-unBlock {
  background-image: url('themes/wikimediaui/images/icons/unBlock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unBlock.svg');
}
.oo-ui-image-invert.oo-ui-icon-unBlock {
  background-image: url('themes/wikimediaui/images/icons/unBlock-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unBlock-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-unBlock {
  background-image: url('themes/wikimediaui/images/icons/unBlock-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unBlock-progressive.svg');
}
.oo-ui-icon-flag {
  background-image: url('themes/wikimediaui/images/icons/flag-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/flag-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-flag {
  background-image: url('themes/wikimediaui/images/icons/flag-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/flag-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-flag {
  background-image: url('themes/wikimediaui/images/icons/flag-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/flag-rtl-progressive.svg');
}
.oo-ui-icon-unFlag {
  background-image: url('themes/wikimediaui/images/icons/unFlag-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unFlag-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-unFlag {
  background-image: url('themes/wikimediaui/images/icons/unFlag-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unFlag-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-unFlag {
  background-image: url('themes/wikimediaui/images/icons/unFlag-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unFlag-rtl-progressive.svg');
}
.oo-ui-icon-lock {
  background-image: url('themes/wikimediaui/images/icons/lock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lock.svg');
}
.oo-ui-image-destructive.oo-ui-icon-lock {
  background-image: url('themes/wikimediaui/images/icons/lock-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lock-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-lock {
  background-image: url('themes/wikimediaui/images/icons/lock-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lock-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-lock {
  background-image: url('themes/wikimediaui/images/icons/lock-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/lock-progressive.svg');
}
.oo-ui-icon-unLock {
  background-image: url('themes/wikimediaui/images/icons/unLock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLock.svg');
}
.oo-ui-image-destructive.oo-ui-icon-unLock {
  background-image: url('themes/wikimediaui/images/icons/unLock-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLock-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-unLock {
  background-image: url('themes/wikimediaui/images/icons/unLock-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLock-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-unLock {
  background-image: url('themes/wikimediaui/images/icons/unLock-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLock-progressive.svg');
}
.oo-ui-icon-restore {
  background-image: url('themes/wikimediaui/images/icons/restore.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/restore.svg');
}
.oo-ui-image-invert.oo-ui-icon-restore {
  background-image: url('themes/wikimediaui/images/icons/restore-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/restore-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-restore {
  background-image: url('themes/wikimediaui/images/icons/restore-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/restore-progressive.svg');
}
.oo-ui-icon-star {
  background-image: url('themes/wikimediaui/images/icons/star.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/star.svg');
}
.oo-ui-image-invert.oo-ui-icon-star {
  background-image: url('themes/wikimediaui/images/icons/star-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/star-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-star {
  background-image: url('themes/wikimediaui/images/icons/star-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/star-progressive.svg');
}
.oo-ui-icon-halfStar {
  background-image: url('themes/wikimediaui/images/icons/halfStar-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfStar-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-halfStar {
  background-image: url('themes/wikimediaui/images/icons/halfStar-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfStar-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-halfStar {
  background-image: url('themes/wikimediaui/images/icons/halfStar-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfStar-rtl-progressive.svg');
}
.oo-ui-icon-unStar {
  background-image: url('themes/wikimediaui/images/icons/unStar.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unStar.svg');
}
.oo-ui-image-invert.oo-ui-icon-unStar {
  background-image: url('themes/wikimediaui/images/icons/unStar-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unStar-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-unStar {
  background-image: url('themes/wikimediaui/images/icons/unStar-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unStar-progressive.svg');
}
.oo-ui-icon-trash {
  background-image: url('themes/wikimediaui/images/icons/trash.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/trash.svg');
}
.oo-ui-image-destructive.oo-ui-icon-trash {
  background-image: url('themes/wikimediaui/images/icons/trash-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/trash-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-trash {
  background-image: url('themes/wikimediaui/images/icons/trash-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/trash-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-trash {
  background-image: url('themes/wikimediaui/images/icons/trash-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/trash-progressive.svg');
}
.oo-ui-icon-pushPin {
  background-image: url('themes/wikimediaui/images/icons/pushPin.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pushPin.svg');
}
.oo-ui-image-invert.oo-ui-icon-pushPin {
  background-image: url('themes/wikimediaui/images/icons/pushPin-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pushPin-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-pushPin {
  background-image: url('themes/wikimediaui/images/icons/pushPin-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pushPin-progressive.svg');
}
.oo-ui-icon-ongoingConversation {
  background-image: url('themes/wikimediaui/images/icons/ongoingConversation-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ongoingConversation-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-ongoingConversation {
  background-image: url('themes/wikimediaui/images/icons/ongoingConversation-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ongoingConversation-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-ongoingConversation {
  background-image: url('themes/wikimediaui/images/icons/ongoingConversation-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/ongoingConversation-rtl-progressive.svg');
}

.oo-ui-icon-edit {
  background-image: url('themes/wikimediaui/images/icons/edit.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/edit.svg');
}
.oo-ui-image-invert.oo-ui-icon-edit {
  background-image: url('themes/wikimediaui/images/icons/edit-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/edit-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-edit {
  background-image: url('themes/wikimediaui/images/icons/edit-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/edit-progressive.svg');
}
.oo-ui-icon-editLock {
  background-image: url('themes/wikimediaui/images/icons/editLock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editLock.svg');
}
.oo-ui-image-invert.oo-ui-icon-editLock {
  background-image: url('themes/wikimediaui/images/icons/editLock-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editLock-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-editLock {
  background-image: url('themes/wikimediaui/images/icons/editLock-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editLock-progressive.svg');
}
.oo-ui-icon-editUndo {
  background-image: url('themes/wikimediaui/images/icons/editUndo-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editUndo-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-editUndo {
  background-image: url('themes/wikimediaui/images/icons/editUndo-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editUndo-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-editUndo {
  background-image: url('themes/wikimediaui/images/icons/editUndo-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/editUndo-rtl-progressive.svg');
}
.oo-ui-icon-link {
  background-image: url('themes/wikimediaui/images/icons/link.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/link.svg');
}
.oo-ui-image-invert.oo-ui-icon-link {
  background-image: url('themes/wikimediaui/images/icons/link-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/link-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-link {
  background-image: url('themes/wikimediaui/images/icons/link-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/link-progressive.svg');
}
.oo-ui-icon-unLink {
  background-image: url('themes/wikimediaui/images/icons/unLink.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLink.svg');
}
.oo-ui-image-destructive.oo-ui-icon-unLink {
  background-image: url('themes/wikimediaui/images/icons/unLink-destructive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLink-destructive.svg');
}
.oo-ui-image-invert.oo-ui-icon-unLink {
  background-image: url('themes/wikimediaui/images/icons/unLink-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLink-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-unLink {
  background-image: url('themes/wikimediaui/images/icons/unLink-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/unLink-progressive.svg');
}
.oo-ui-icon-linkExternal {
  background-image: url('themes/wikimediaui/images/icons/linkExternal-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkExternal-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-linkExternal {
  background-image: url('themes/wikimediaui/images/icons/linkExternal-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkExternal-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-linkExternal {
  background-image: url('themes/wikimediaui/images/icons/linkExternal-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkExternal-rtl-progressive.svg');
}
.oo-ui-icon-linkSecure {
  background-image: url('themes/wikimediaui/images/icons/linkSecure.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkSecure.svg');
}
.oo-ui-image-invert.oo-ui-icon-linkSecure {
  background-image: url('themes/wikimediaui/images/icons/linkSecure-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkSecure-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-linkSecure {
  background-image: url('themes/wikimediaui/images/icons/linkSecure-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/linkSecure-progressive.svg');
}
.oo-ui-icon-redo {
  background-image: url('themes/wikimediaui/images/icons/redo-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/redo-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-redo {
  background-image: url('themes/wikimediaui/images/icons/redo-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/redo-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-redo {
  background-image: url('themes/wikimediaui/images/icons/redo-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/redo-rtl-progressive.svg');
}
.oo-ui-icon-undo {
  background-image: url('themes/wikimediaui/images/icons/undo-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/undo-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-undo {
  background-image: url('themes/wikimediaui/images/icons/undo-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/undo-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-undo {
  background-image: url('themes/wikimediaui/images/icons/undo-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/undo-rtl-progressive.svg');
}

.oo-ui-icon-bigger {
  background-image: url('themes/wikimediaui/images/icons/bigger.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bigger.svg');
}
.oo-ui-image-invert.oo-ui-icon-bigger {
  background-image: url('themes/wikimediaui/images/icons/bigger-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bigger-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bigger {
  background-image: url('themes/wikimediaui/images/icons/bigger-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bigger-progressive.svg');
}
.oo-ui-icon-smaller {
  background-image: url('themes/wikimediaui/images/icons/smaller.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smaller.svg');
}
.oo-ui-image-invert.oo-ui-icon-smaller {
  background-image: url('themes/wikimediaui/images/icons/smaller-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smaller-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-smaller {
  background-image: url('themes/wikimediaui/images/icons/smaller-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smaller-progressive.svg');
}
.oo-ui-icon-subscript {
  background-image: url('themes/wikimediaui/images/icons/subscript-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subscript-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-subscript {
  background-image: url('themes/wikimediaui/images/icons/subscript-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subscript-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-subscript {
  background-image: url('themes/wikimediaui/images/icons/subscript-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/subscript-rtl-progressive.svg');
}
.oo-ui-icon-superscript {
  background-image: url('themes/wikimediaui/images/icons/superscript-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/superscript-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-superscript {
  background-image: url('themes/wikimediaui/images/icons/superscript-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/superscript-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-superscript {
  background-image: url('themes/wikimediaui/images/icons/superscript-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/superscript-rtl-progressive.svg');
}
.oo-ui-icon-bold {
  background-image: url('themes/wikimediaui/images/icons/bold-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-a.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-ain.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-ain.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-te.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-te.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-palochka.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-palochka.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/bold-b.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/bold-f.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/bold-n.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/bold-n.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/bold-n.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/bold-l.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/bold-l.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-dad.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-dad.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/bold-g.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/bold-g.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/bold-armn-to.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-armn-to.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/bold-geor-man.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-geor-man.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/bold-v.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-v.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-be.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-be.svg');
}
/* @noflip */
.oo-ui-icon-bold:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-jeem.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-jeem.svg');
}
.oo-ui-image-invert.oo-ui-icon-bold {
  background-image: url('themes/wikimediaui/images/icons/bold-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-ain-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-ain-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-te-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-te-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-palochka-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-palochka-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-dad-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-dad-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/bold-armn-to-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-armn-to-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/bold-geor-man-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-geor-man-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/bold-v-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-v-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-be-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-be-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-bold:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-jeem-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-jeem-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bold {
  background-image: url('themes/wikimediaui/images/icons/bold-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-ain-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-ain-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-te-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-te-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-palochka-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-palochka-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/bold-b-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-b-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/bold-f-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-f-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/bold-n-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-n-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/bold-l-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-l-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-dad-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-dad-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/bold-g-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-g-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/bold-armn-to-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-armn-to-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/bold-geor-man-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-geor-man-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-zhe-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/bold-v-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-v-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/bold-cyrl-be-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-cyrl-be-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-bold:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/bold-arab-jeem-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bold-arab-jeem-progressive.svg');
}
.oo-ui-icon-highlight {
  background-image: url('themes/wikimediaui/images/icons/highlight.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/highlight.svg');
}
.oo-ui-image-invert.oo-ui-icon-highlight {
  background-image: url('themes/wikimediaui/images/icons/highlight-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/highlight-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-highlight {
  background-image: url('themes/wikimediaui/images/icons/highlight-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/highlight-progressive.svg');
}
.oo-ui-icon-italic {
  background-image: url('themes/wikimediaui/images/icons/italic-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-a.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-meem.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-meem.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/italic-i.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/italic-k.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/italic-e.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-e.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/italic-d.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-d.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/italic-armn-sha.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-armn-sha.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/italic-s.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-s.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/italic-geor-kan.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-geor-kan.svg');
}
/* @noflip */
.oo-ui-icon-italic:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-teh.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-teh.svg');
}
.oo-ui-image-invert.oo-ui-icon-italic {
  background-image: url('themes/wikimediaui/images/icons/italic-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-meem-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-meem-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/italic-e-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-e-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/italic-d-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-d-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/italic-armn-sha-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-armn-sha-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/italic-s-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-s-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/italic-geor-kan-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-geor-kan-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-italic:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-teh-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-teh-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-italic {
  background-image: url('themes/wikimediaui/images/icons/italic-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ar) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-meem-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-meem-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(cs) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(fr) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(he) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ml) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(pl) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(pt) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(sco) {
  background-image: url('themes/wikimediaui/images/icons/italic-i-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-i-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(be) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ce) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(da) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ky) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(nn) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(no) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(os) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(sv) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ru) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(uk) {
  background-image: url('themes/wikimediaui/images/icons/italic-k-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-k-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(es) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(gl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(it) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(nl) {
  background-image: url('themes/wikimediaui/images/icons/italic-c-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-c-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(eu) {
  background-image: url('themes/wikimediaui/images/icons/italic-e-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-e-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(fa) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-keheh-jeem-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(hu) {
  background-image: url('themes/wikimediaui/images/icons/italic-d-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-d-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(hy) {
  background-image: url('themes/wikimediaui/images/icons/italic-armn-sha-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-armn-sha-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ksh) {
  background-image: url('themes/wikimediaui/images/icons/italic-s-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-s-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ka) {
  background-image: url('themes/wikimediaui/images/icons/italic-geor-kan-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-geor-kan-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-italic:lang(ur) {
  background-image: url('themes/wikimediaui/images/icons/italic-arab-teh-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/italic-arab-teh-progressive.svg');
}
.oo-ui-icon-strikethrough {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-a.svg');
}
/* @noflip */
.oo-ui-icon-strikethrough:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-s.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-s.svg');
}
/* @noflip */
.oo-ui-icon-strikethrough:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-y.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-y.svg');
}
.oo-ui-image-invert.oo-ui-icon-strikethrough {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-strikethrough:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-s-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-s-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-strikethrough:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-y-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-y-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-strikethrough {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-strikethrough:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-s-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-s-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-strikethrough:lang(fi) {
  background-image: url('themes/wikimediaui/images/icons/strikethrough-y-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/strikethrough-y-progressive.svg');
}
.oo-ui-icon-underline {
  background-image: url('themes/wikimediaui/images/icons/underline-a.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-a.svg');
}
/* @noflip */
.oo-ui-icon-underline:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/underline-u.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u.svg');
}
/* @noflip */
.oo-ui-icon-underline:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/underline-u.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u.svg');
}
.oo-ui-image-invert.oo-ui-icon-underline {
  background-image: url('themes/wikimediaui/images/icons/underline-a-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-a-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-underline:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-invert.svg');
}
/* @noflip */
.oo-ui-image-invert.oo-ui-icon-underline:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-underline {
  background-image: url('themes/wikimediaui/images/icons/underline-a-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-a-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-underline:lang(en) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-progressive.svg');
}
/* @noflip */
.oo-ui-image-progressive.oo-ui-icon-underline:lang(de) {
  background-image: url('themes/wikimediaui/images/icons/underline-u-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/underline-u-progressive.svg');
}
.oo-ui-icon-textLanguage {
  background-image: url('themes/wikimediaui/images/icons/language.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language.svg');
}
.oo-ui-image-invert.oo-ui-icon-textLanguage {
  background-image: url('themes/wikimediaui/images/icons/language-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textLanguage {
  background-image: url('themes/wikimediaui/images/icons/language-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-progressive.svg');
}
.oo-ui-icon-textDirLTR {
  background-image: url('themes/wikimediaui/images/icons/textDirLTR.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirLTR.svg');
}
.oo-ui-image-invert.oo-ui-icon-textDirLTR {
  background-image: url('themes/wikimediaui/images/icons/textDirLTR-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirLTR-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textDirLTR {
  background-image: url('themes/wikimediaui/images/icons/textDirLTR-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirLTR-progressive.svg');
}
.oo-ui-icon-textDirRTL {
  background-image: url('themes/wikimediaui/images/icons/textDirRTL.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirRTL.svg');
}
.oo-ui-image-invert.oo-ui-icon-textDirRTL {
  background-image: url('themes/wikimediaui/images/icons/textDirRTL-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirRTL-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textDirRTL {
  background-image: url('themes/wikimediaui/images/icons/textDirRTL-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textDirRTL-progressive.svg');
}
.oo-ui-icon-textStyle {
  background-image: url('themes/wikimediaui/images/icons/textStyle.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textStyle.svg');
}
.oo-ui-image-invert.oo-ui-icon-textStyle {
  background-image: url('themes/wikimediaui/images/icons/textStyle-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textStyle-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textStyle {
  background-image: url('themes/wikimediaui/images/icons/textStyle-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textStyle-progressive.svg');
}

.oo-ui-icon-indent {
  background-image: url('themes/wikimediaui/images/icons/indent-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/indent-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-indent {
  background-image: url('themes/wikimediaui/images/icons/indent-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/indent-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-indent {
  background-image: url('themes/wikimediaui/images/icons/indent-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/indent-rtl-progressive.svg');
}
.oo-ui-icon-listBullet {
  background-image: url('themes/wikimediaui/images/icons/listBullet-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/listBullet-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-listBullet {
  background-image: url('themes/wikimediaui/images/icons/listBullet-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/listBullet-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-listBullet {
  background-image: url('themes/wikimediaui/images/icons/listBullet-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/listBullet-rtl-progressive.svg');
}
.oo-ui-icon-listNumbered {
  background-image: url('themes/wikimediaui/images/icons/listNumbered-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/listNumbered-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-listNumbered {
  background-image: url('themes/wikimediaui/images/icons/listNumbered-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/listNumbered-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-listNumbered {
  background-image: url('themes/wikimediaui/images/icons/listNumbered-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/listNumbered-rtl-progressive.svg');
}
.oo-ui-icon-outdent {
  background-image: url('themes/wikimediaui/images/icons/outdent-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/outdent-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-outdent {
  background-image: url('themes/wikimediaui/images/icons/outdent-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/outdent-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-outdent {
  background-image: url('themes/wikimediaui/images/icons/outdent-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/outdent-rtl-progressive.svg');
}

.oo-ui-icon-alignCenter {
  background-image: url('themes/wikimediaui/images/icons/alignCenter.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignCenter.svg');
}
.oo-ui-image-invert.oo-ui-icon-alignCenter {
  background-image: url('themes/wikimediaui/images/icons/alignCenter-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignCenter-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-alignCenter {
  background-image: url('themes/wikimediaui/images/icons/alignCenter-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignCenter-progressive.svg');
}
.oo-ui-icon-alignLeft {
  background-image: url('themes/wikimediaui/images/icons/alignLeft.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignLeft.svg');
}
.oo-ui-image-invert.oo-ui-icon-alignLeft {
  background-image: url('themes/wikimediaui/images/icons/alignLeft-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignLeft-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-alignLeft {
  background-image: url('themes/wikimediaui/images/icons/alignLeft-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignLeft-progressive.svg');
}
.oo-ui-icon-alignRight {
  background-image: url('themes/wikimediaui/images/icons/alignRight.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignRight.svg');
}
.oo-ui-image-invert.oo-ui-icon-alignRight {
  background-image: url('themes/wikimediaui/images/icons/alignRight-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignRight-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-alignRight {
  background-image: url('themes/wikimediaui/images/icons/alignRight-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/alignRight-progressive.svg');
}
.oo-ui-icon-attachment {
  background-image: url('themes/wikimediaui/images/icons/attachment.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/attachment.svg');
}
.oo-ui-image-invert.oo-ui-icon-attachment {
  background-image: url('themes/wikimediaui/images/icons/attachment-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/attachment-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-attachment {
  background-image: url('themes/wikimediaui/images/icons/attachment-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/attachment-progressive.svg');
}
.oo-ui-icon-calendar {
  background-image: url('themes/wikimediaui/images/icons/calendar.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/calendar.svg');
}
.oo-ui-image-invert.oo-ui-icon-calendar {
  background-image: url('themes/wikimediaui/images/icons/calendar-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/calendar-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-calendar {
  background-image: url('themes/wikimediaui/images/icons/calendar-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/calendar-progressive.svg');
}
.oo-ui-icon-code {
  background-image: url('themes/wikimediaui/images/icons/code.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/code.svg');
}
.oo-ui-image-invert.oo-ui-icon-code {
  background-image: url('themes/wikimediaui/images/icons/code-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/code-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-code {
  background-image: url('themes/wikimediaui/images/icons/code-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/code-progressive.svg');
}
.oo-ui-icon-hieroglyph {
  background-image: url('themes/wikimediaui/images/icons/hieroglyph.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/hieroglyph.svg');
}
.oo-ui-image-invert.oo-ui-icon-hieroglyph {
  background-image: url('themes/wikimediaui/images/icons/hieroglyph-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/hieroglyph-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-hieroglyph {
  background-image: url('themes/wikimediaui/images/icons/hieroglyph-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/hieroglyph-progressive.svg');
}
.oo-ui-icon-imageLayoutBasic {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutBasic.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutBasic.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageLayoutBasic {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutBasic-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutBasic-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageLayoutBasic {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutBasic-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutBasic-progressive.svg');
}
.oo-ui-icon-imageLayoutFrame {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutFrame.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutFrame.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageLayoutFrame {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutFrame-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutFrame-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageLayoutFrame {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutFrame-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutFrame-progressive.svg');
}
.oo-ui-icon-imageLayoutFrameless {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutFrameless.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutFrameless.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageLayoutFrameless {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutFrameless-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutFrameless-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageLayoutFrameless {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutFrameless-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutFrameless-progressive.svg');
}
.oo-ui-icon-imageLayoutThumbnail {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutThumbnail.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutThumbnail.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageLayoutThumbnail {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutThumbnail-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutThumbnail-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageLayoutThumbnail {
  background-image: url('themes/wikimediaui/images/icons/imageLayoutThumbnail-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLayoutThumbnail-progressive.svg');
}
.oo-ui-icon-labFlask {
  background-image: url('themes/wikimediaui/images/icons/labFlask.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/labFlask.svg');
}
.oo-ui-image-invert.oo-ui-icon-labFlask {
  background-image: url('themes/wikimediaui/images/icons/labFlask-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/labFlask-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-labFlask {
  background-image: url('themes/wikimediaui/images/icons/labFlask-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/labFlask-progressive.svg');
}
.oo-ui-icon-language {
  background-image: url('themes/wikimediaui/images/icons/language.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language.svg');
}
.oo-ui-image-invert.oo-ui-icon-language {
  background-image: url('themes/wikimediaui/images/icons/language-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-language {
  background-image: url('themes/wikimediaui/images/icons/language-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-progressive.svg');
}
.oo-ui-icon-layout {
  background-image: url('themes/wikimediaui/images/icons/layout-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/layout-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-layout {
  background-image: url('themes/wikimediaui/images/icons/layout-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/layout-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-layout {
  background-image: url('themes/wikimediaui/images/icons/layout-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/layout-rtl-progressive.svg');
}
.oo-ui-icon-markup {
  background-image: url('themes/wikimediaui/images/icons/markup.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/markup.svg');
}
.oo-ui-image-invert.oo-ui-icon-markup {
  background-image: url('themes/wikimediaui/images/icons/markup-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/markup-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-markup {
  background-image: url('themes/wikimediaui/images/icons/markup-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/markup-progressive.svg');
}
.oo-ui-icon-mathematics {
  background-image: url('themes/wikimediaui/images/icons/mathematics.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematics.svg');
}
.oo-ui-image-invert.oo-ui-icon-mathematics {
  background-image: url('themes/wikimediaui/images/icons/mathematics-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematics-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mathematics {
  background-image: url('themes/wikimediaui/images/icons/mathematics-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematics-progressive.svg');
}
.oo-ui-icon-mathematicsDisplayBlock {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayBlock.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayBlock.svg');
}
.oo-ui-image-invert.oo-ui-icon-mathematicsDisplayBlock {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayBlock-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayBlock-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mathematicsDisplayBlock {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayBlock-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayBlock-progressive.svg');
}
.oo-ui-icon-mathematicsDisplayDefault {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayDefault.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayDefault.svg');
}
.oo-ui-image-invert.oo-ui-icon-mathematicsDisplayDefault {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayDefault-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayDefault-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mathematicsDisplayDefault {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayDefault-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayDefault-progressive.svg');
}
.oo-ui-icon-mathematicsDisplayInline {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayInline.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayInline.svg');
}
.oo-ui-image-invert.oo-ui-icon-mathematicsDisplayInline {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayInline-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayInline-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mathematicsDisplayInline {
  background-image: url('themes/wikimediaui/images/icons/mathematicsDisplayInline-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mathematicsDisplayInline-progressive.svg');
}
.oo-ui-icon-newline {
  background-image: url('themes/wikimediaui/images/icons/newline-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newline-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-newline {
  background-image: url('themes/wikimediaui/images/icons/newline-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newline-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-newline {
  background-image: url('themes/wikimediaui/images/icons/newline-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newline-rtl-progressive.svg');
}
.oo-ui-icon-noWikiText {
  background-image: url('themes/wikimediaui/images/icons/noWikiText.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/noWikiText.svg');
}
.oo-ui-image-invert.oo-ui-icon-noWikiText {
  background-image: url('themes/wikimediaui/images/icons/noWikiText-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/noWikiText-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-noWikiText {
  background-image: url('themes/wikimediaui/images/icons/noWikiText-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/noWikiText-progressive.svg');
}
.oo-ui-icon-outline {
  background-image: url('themes/wikimediaui/images/icons/outline-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/outline-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-outline {
  background-image: url('themes/wikimediaui/images/icons/outline-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/outline-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-outline {
  background-image: url('themes/wikimediaui/images/icons/outline-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/outline-rtl-progressive.svg');
}
.oo-ui-icon-puzzle {
  background-image: url('themes/wikimediaui/images/icons/puzzle-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/puzzle-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-puzzle {
  background-image: url('themes/wikimediaui/images/icons/puzzle-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/puzzle-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-puzzle {
  background-image: url('themes/wikimediaui/images/icons/puzzle-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/puzzle-rtl-progressive.svg');
}
.oo-ui-icon-quotes {
  background-image: url('themes/wikimediaui/images/icons/quotes-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/quotes-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-quotes {
  background-image: url('themes/wikimediaui/images/icons/quotes-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/quotes-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-quotes {
  background-image: url('themes/wikimediaui/images/icons/quotes-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/quotes-rtl-progressive.svg');
}
.oo-ui-icon-searchCaseSensitive {
  background-image: url('themes/wikimediaui/images/icons/searchCaseSensitive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchCaseSensitive.svg');
}
.oo-ui-image-invert.oo-ui-icon-searchCaseSensitive {
  background-image: url('themes/wikimediaui/images/icons/searchCaseSensitive-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchCaseSensitive-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-searchCaseSensitive {
  background-image: url('themes/wikimediaui/images/icons/searchCaseSensitive-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchCaseSensitive-progressive.svg');
}
.oo-ui-icon-searchDiacritics {
  background-image: url('themes/wikimediaui/images/icons/searchDiacritics.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchDiacritics.svg');
}
.oo-ui-image-invert.oo-ui-icon-searchDiacritics {
  background-image: url('themes/wikimediaui/images/icons/searchDiacritics-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchDiacritics-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-searchDiacritics {
  background-image: url('themes/wikimediaui/images/icons/searchDiacritics-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchDiacritics-progressive.svg');
}
.oo-ui-icon-searchRegularExpression {
  background-image: url('themes/wikimediaui/images/icons/searchRegularExpression.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchRegularExpression.svg');
}
.oo-ui-image-invert.oo-ui-icon-searchRegularExpression {
  background-image: url('themes/wikimediaui/images/icons/searchRegularExpression-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchRegularExpression-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-searchRegularExpression {
  background-image: url('themes/wikimediaui/images/icons/searchRegularExpression-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/searchRegularExpression-progressive.svg');
}
.oo-ui-icon-signature {
  background-image: url('themes/wikimediaui/images/icons/signature-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/signature-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-signature {
  background-image: url('themes/wikimediaui/images/icons/signature-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/signature-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-signature {
  background-image: url('themes/wikimediaui/images/icons/signature-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/signature-rtl-progressive.svg');
}
.oo-ui-icon-specialCharacter {
  background-image: url('themes/wikimediaui/images/icons/specialCharacter.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialCharacter.svg');
}
.oo-ui-image-invert.oo-ui-icon-specialCharacter {
  background-image: url('themes/wikimediaui/images/icons/specialCharacter-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialCharacter-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-specialCharacter {
  background-image: url('themes/wikimediaui/images/icons/specialCharacter-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/specialCharacter-progressive.svg');
}
.oo-ui-icon-table {
  background-image: url('themes/wikimediaui/images/icons/table.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/table.svg');
}
.oo-ui-image-invert.oo-ui-icon-table {
  background-image: url('themes/wikimediaui/images/icons/table-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/table-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-table {
  background-image: url('themes/wikimediaui/images/icons/table-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/table-progressive.svg');
}
.oo-ui-icon-tableAddColumnAfter {
  background-image: url('themes/wikimediaui/images/icons/tableAddColumnAfter-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddColumnAfter-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableAddColumnAfter {
  background-image: url('themes/wikimediaui/images/icons/tableAddColumnAfter-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddColumnAfter-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableAddColumnAfter {
  background-image: url('themes/wikimediaui/images/icons/tableAddColumnAfter-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddColumnAfter-rtl-progressive.svg');
}
.oo-ui-icon-tableAddColumnBefore {
  background-image: url('themes/wikimediaui/images/icons/tableAddColumnBefore-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddColumnBefore-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableAddColumnBefore {
  background-image: url('themes/wikimediaui/images/icons/tableAddColumnBefore-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddColumnBefore-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableAddColumnBefore {
  background-image: url('themes/wikimediaui/images/icons/tableAddColumnBefore-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddColumnBefore-rtl-progressive.svg');
}
.oo-ui-icon-tableAddRowAfter {
  background-image: url('themes/wikimediaui/images/icons/tableAddRowAfter.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddRowAfter.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableAddRowAfter {
  background-image: url('themes/wikimediaui/images/icons/tableAddRowAfter-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddRowAfter-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableAddRowAfter {
  background-image: url('themes/wikimediaui/images/icons/tableAddRowAfter-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddRowAfter-progressive.svg');
}
.oo-ui-icon-tableAddRowBefore {
  background-image: url('themes/wikimediaui/images/icons/tableAddRowBefore.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddRowBefore.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableAddRowBefore {
  background-image: url('themes/wikimediaui/images/icons/tableAddRowBefore-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddRowBefore-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableAddRowBefore {
  background-image: url('themes/wikimediaui/images/icons/tableAddRowBefore-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableAddRowBefore-progressive.svg');
}
.oo-ui-icon-tableCaption {
  background-image: url('themes/wikimediaui/images/icons/tableCaption.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableCaption.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableCaption {
  background-image: url('themes/wikimediaui/images/icons/tableCaption-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableCaption-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableCaption {
  background-image: url('themes/wikimediaui/images/icons/tableCaption-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableCaption-progressive.svg');
}
.oo-ui-icon-tableMergeCells {
  background-image: url('themes/wikimediaui/images/icons/tableMergeCells.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMergeCells.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableMergeCells {
  background-image: url('themes/wikimediaui/images/icons/tableMergeCells-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMergeCells-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableMergeCells {
  background-image: url('themes/wikimediaui/images/icons/tableMergeCells-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMergeCells-progressive.svg');
}
.oo-ui-icon-tableMoveColumnAfter {
  background-image: url('themes/wikimediaui/images/icons/tableMoveColumnAfter-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveColumnAfter-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableMoveColumnAfter {
  background-image: url('themes/wikimediaui/images/icons/tableMoveColumnAfter-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveColumnAfter-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableMoveColumnAfter {
  background-image: url('themes/wikimediaui/images/icons/tableMoveColumnAfter-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveColumnAfter-rtl-progressive.svg');
}
.oo-ui-icon-tableMoveColumnBefore {
  background-image: url('themes/wikimediaui/images/icons/tableMoveColumnBefore-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveColumnBefore-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableMoveColumnBefore {
  background-image: url('themes/wikimediaui/images/icons/tableMoveColumnBefore-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveColumnBefore-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableMoveColumnBefore {
  background-image: url('themes/wikimediaui/images/icons/tableMoveColumnBefore-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveColumnBefore-rtl-progressive.svg');
}
.oo-ui-icon-tableMoveRowAfter {
  background-image: url('themes/wikimediaui/images/icons/tableMoveRowAfter.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveRowAfter.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableMoveRowAfter {
  background-image: url('themes/wikimediaui/images/icons/tableMoveRowAfter-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveRowAfter-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableMoveRowAfter {
  background-image: url('themes/wikimediaui/images/icons/tableMoveRowAfter-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveRowAfter-progressive.svg');
}
.oo-ui-icon-tableMoveRowBefore {
  background-image: url('themes/wikimediaui/images/icons/tableMoveRowBefore.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveRowBefore.svg');
}
.oo-ui-image-invert.oo-ui-icon-tableMoveRowBefore {
  background-image: url('themes/wikimediaui/images/icons/tableMoveRowBefore-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveRowBefore-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-tableMoveRowBefore {
  background-image: url('themes/wikimediaui/images/icons/tableMoveRowBefore-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/tableMoveRowBefore-progressive.svg');
}
.oo-ui-icon-templateAdd {
  background-image: url('themes/wikimediaui/images/icons/templateAdd-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/templateAdd-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-templateAdd {
  background-image: url('themes/wikimediaui/images/icons/templateAdd-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/templateAdd-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-templateAdd {
  background-image: url('themes/wikimediaui/images/icons/templateAdd-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/templateAdd-rtl-progressive.svg');
}
.oo-ui-icon-translation {
  background-image: url('themes/wikimediaui/images/icons/language.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language.svg');
}
.oo-ui-image-invert.oo-ui-icon-translation {
  background-image: url('themes/wikimediaui/images/icons/language-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-translation {
  background-image: url('themes/wikimediaui/images/icons/language-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/language-progressive.svg');
}
.oo-ui-icon-wikiText {
  background-image: url('themes/wikimediaui/images/icons/wikiText.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/wikiText.svg');
}
.oo-ui-image-invert.oo-ui-icon-wikiText {
  background-image: url('themes/wikimediaui/images/icons/wikiText-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/wikiText-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-wikiText {
  background-image: url('themes/wikimediaui/images/icons/wikiText-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/wikiText-progressive.svg');
}

.oo-ui-icon-book {
  background-image: url('themes/wikimediaui/images/icons/book-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/book-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-book {
  background-image: url('themes/wikimediaui/images/icons/book-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/book-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-book {
  background-image: url('themes/wikimediaui/images/icons/book-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/book-rtl-progressive.svg');
}
.oo-ui-icon-journal {
  background-image: url('themes/wikimediaui/images/icons/journal-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/journal-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-journal {
  background-image: url('themes/wikimediaui/images/icons/journal-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/journal-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-journal {
  background-image: url('themes/wikimediaui/images/icons/journal-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/journal-rtl-progressive.svg');
}
.oo-ui-icon-newspaper {
  background-image: url('themes/wikimediaui/images/icons/newspaper-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newspaper-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-newspaper {
  background-image: url('themes/wikimediaui/images/icons/newspaper-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newspaper-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-newspaper {
  background-image: url('themes/wikimediaui/images/icons/newspaper-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/newspaper-rtl-progressive.svg');
}
.oo-ui-icon-reference {
  background-image: url('themes/wikimediaui/images/icons/reference.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/reference.svg');
}
.oo-ui-image-invert.oo-ui-icon-reference {
  background-image: url('themes/wikimediaui/images/icons/reference-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/reference-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-reference {
  background-image: url('themes/wikimediaui/images/icons/reference-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/reference-progressive.svg');
}
.oo-ui-icon-referenceExisting {
  background-image: url('themes/wikimediaui/images/icons/referenceExisting-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/referenceExisting-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-referenceExisting {
  background-image: url('themes/wikimediaui/images/icons/referenceExisting-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/referenceExisting-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-referenceExisting {
  background-image: url('themes/wikimediaui/images/icons/referenceExisting-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/referenceExisting-rtl-progressive.svg');
}
.oo-ui-icon-references {
  background-image: url('themes/wikimediaui/images/icons/references-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/references-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-references {
  background-image: url('themes/wikimediaui/images/icons/references-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/references-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-references {
  background-image: url('themes/wikimediaui/images/icons/references-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/references-rtl-progressive.svg');
}

.oo-ui-icon-camera {
  background-image: url('themes/wikimediaui/images/icons/camera.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/camera.svg');
}
.oo-ui-image-invert.oo-ui-icon-camera {
  background-image: url('themes/wikimediaui/images/icons/camera-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/camera-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-camera {
  background-image: url('themes/wikimediaui/images/icons/camera-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/camera-progressive.svg');
}
.oo-ui-icon-chart {
  background-image: url('themes/wikimediaui/images/icons/chart.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/chart.svg');
}
.oo-ui-image-invert.oo-ui-icon-chart {
  background-image: url('themes/wikimediaui/images/icons/chart-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/chart-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-chart {
  background-image: url('themes/wikimediaui/images/icons/chart-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/chart-progressive.svg');
}
.oo-ui-icon-exitFullscreen {
  background-image: url('themes/wikimediaui/images/icons/exitFullscreen.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/exitFullscreen.svg');
}
.oo-ui-image-invert.oo-ui-icon-exitFullscreen {
  background-image: url('themes/wikimediaui/images/icons/exitFullscreen-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/exitFullscreen-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-exitFullscreen {
  background-image: url('themes/wikimediaui/images/icons/exitFullscreen-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/exitFullscreen-progressive.svg');
}
.oo-ui-icon-fullScreen {
  background-image: url('themes/wikimediaui/images/icons/fullScreen.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/fullScreen.svg');
}
.oo-ui-image-invert.oo-ui-icon-fullScreen {
  background-image: url('themes/wikimediaui/images/icons/fullScreen-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/fullScreen-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-fullScreen {
  background-image: url('themes/wikimediaui/images/icons/fullScreen-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/fullScreen-progressive.svg');
}
.oo-ui-icon-image {
  background-image: url('themes/wikimediaui/images/icons/image.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/image.svg');
}
.oo-ui-image-invert.oo-ui-icon-image {
  background-image: url('themes/wikimediaui/images/icons/image-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/image-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-image {
  background-image: url('themes/wikimediaui/images/icons/image-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/image-progressive.svg');
}
.oo-ui-icon-imageAdd {
  background-image: url('themes/wikimediaui/images/icons/imageAdd-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageAdd-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageAdd {
  background-image: url('themes/wikimediaui/images/icons/imageAdd-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageAdd-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageAdd {
  background-image: url('themes/wikimediaui/images/icons/imageAdd-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageAdd-rtl-progressive.svg');
}
.oo-ui-icon-imageBroken {
  background-image: url('themes/wikimediaui/images/icons/imageBroken.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageBroken.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageBroken {
  background-image: url('themes/wikimediaui/images/icons/imageBroken-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageBroken-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageBroken {
  background-image: url('themes/wikimediaui/images/icons/imageBroken-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageBroken-progressive.svg');
}
.oo-ui-icon-imageLock {
  background-image: url('themes/wikimediaui/images/icons/imageLock-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLock-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageLock {
  background-image: url('themes/wikimediaui/images/icons/imageLock-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLock-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageLock {
  background-image: url('themes/wikimediaui/images/icons/imageLock-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageLock-rtl-progressive.svg');
}
.oo-ui-icon-imageGallery {
  background-image: url('themes/wikimediaui/images/icons/imageGallery.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageGallery.svg');
}
.oo-ui-image-invert.oo-ui-icon-imageGallery {
  background-image: url('themes/wikimediaui/images/icons/imageGallery-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageGallery-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-imageGallery {
  background-image: url('themes/wikimediaui/images/icons/imageGallery-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/imageGallery-progressive.svg');
}
.oo-ui-icon-musicalScore {
  background-image: url('themes/wikimediaui/images/icons/musicalScore.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/musicalScore.svg');
}
.oo-ui-image-invert.oo-ui-icon-musicalScore {
  background-image: url('themes/wikimediaui/images/icons/musicalScore-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/musicalScore-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-musicalScore {
  background-image: url('themes/wikimediaui/images/icons/musicalScore-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/musicalScore-progressive.svg');
}
.oo-ui-icon-pause {
  background-image: url('themes/wikimediaui/images/icons/pause.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pause.svg');
}
.oo-ui-image-invert.oo-ui-icon-pause {
  background-image: url('themes/wikimediaui/images/icons/pause-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pause-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-pause {
  background-image: url('themes/wikimediaui/images/icons/pause-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/pause-progressive.svg');
}
.oo-ui-icon-play {
  background-image: url('themes/wikimediaui/images/icons/play.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/play.svg');
}
.oo-ui-image-invert.oo-ui-icon-play {
  background-image: url('themes/wikimediaui/images/icons/play-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/play-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-play {
  background-image: url('themes/wikimediaui/images/icons/play-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/play-progressive.svg');
}
.oo-ui-icon-stop {
  background-image: url('themes/wikimediaui/images/icons/stop.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stop.svg');
}
.oo-ui-image-invert.oo-ui-icon-stop {
  background-image: url('themes/wikimediaui/images/icons/stop-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stop-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-stop {
  background-image: url('themes/wikimediaui/images/icons/stop-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/stop-progressive.svg');
}
.oo-ui-icon-zoomIn {
  background-image: url('themes/wikimediaui/images/icons/zoomIn.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomIn.svg');
}
.oo-ui-image-invert.oo-ui-icon-zoomIn {
  background-image: url('themes/wikimediaui/images/icons/zoomIn-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomIn-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-zoomIn {
  background-image: url('themes/wikimediaui/images/icons/zoomIn-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomIn-progressive.svg');
}
.oo-ui-icon-zoomOut {
  background-image: url('themes/wikimediaui/images/icons/zoomOut.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomOut.svg');
}
.oo-ui-image-invert.oo-ui-icon-zoomOut {
  background-image: url('themes/wikimediaui/images/icons/zoomOut-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomOut-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-zoomOut {
  background-image: url('themes/wikimediaui/images/icons/zoomOut-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/zoomOut-progressive.svg');
}

.oo-ui-icon-globe {
  background-image: url('themes/wikimediaui/images/icons/globe.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/globe.svg');
}
.oo-ui-image-invert.oo-ui-icon-globe {
  background-image: url('themes/wikimediaui/images/icons/globe-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/globe-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-globe {
  background-image: url('themes/wikimediaui/images/icons/globe-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/globe-progressive.svg');
}
.oo-ui-icon-map {
  background-image: url('themes/wikimediaui/images/icons/map-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/map-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-map {
  background-image: url('themes/wikimediaui/images/icons/map-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/map-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-map {
  background-image: url('themes/wikimediaui/images/icons/map-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/map-rtl-progressive.svg');
}
.oo-ui-icon-mapPin {
  background-image: url('themes/wikimediaui/images/icons/mapPin.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapPin.svg');
}
.oo-ui-image-invert.oo-ui-icon-mapPin {
  background-image: url('themes/wikimediaui/images/icons/mapPin-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapPin-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mapPin {
  background-image: url('themes/wikimediaui/images/icons/mapPin-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapPin-progressive.svg');
}
.oo-ui-icon-mapPinAdd {
  background-image: url('themes/wikimediaui/images/icons/mapPinAdd.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapPinAdd.svg');
}
.oo-ui-image-invert.oo-ui-icon-mapPinAdd {
  background-image: url('themes/wikimediaui/images/icons/mapPinAdd-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapPinAdd-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mapPinAdd {
  background-image: url('themes/wikimediaui/images/icons/mapPinAdd-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapPinAdd-progressive.svg');
}
.oo-ui-icon-mapTrail {
  background-image: url('themes/wikimediaui/images/icons/mapTrail.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapTrail.svg');
}
.oo-ui-image-invert.oo-ui-icon-mapTrail {
  background-image: url('themes/wikimediaui/images/icons/mapTrail-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapTrail-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-mapTrail {
  background-image: url('themes/wikimediaui/images/icons/mapTrail-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/mapTrail-progressive.svg');
}

.oo-ui-icon-userAdd {
  background-image: url('themes/wikimediaui/images/icons/userAdd-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAdd-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAdd {
  background-image: url('themes/wikimediaui/images/icons/userAdd-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAdd-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAdd {
  background-image: url('themes/wikimediaui/images/icons/userAdd-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAdd-rtl-progressive.svg');
}
.oo-ui-icon-userAnonymous {
  background-image: url('themes/wikimediaui/images/icons/userAnonymous.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAnonymous.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAnonymous {
  background-image: url('themes/wikimediaui/images/icons/userAnonymous-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAnonymous-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAnonymous {
  background-image: url('themes/wikimediaui/images/icons/userAnonymous-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAnonymous-progressive.svg');
}
.oo-ui-icon-userAvatar {
  background-image: url('themes/wikimediaui/images/icons/userAvatar.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatar.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAvatar {
  background-image: url('themes/wikimediaui/images/icons/userAvatar-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatar-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAvatar {
  background-image: url('themes/wikimediaui/images/icons/userAvatar-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatar-progressive.svg');
}
.oo-ui-icon-userAvatarOutline {
  background-image: url('themes/wikimediaui/images/icons/userAvatarOutline.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatarOutline.svg');
}
.oo-ui-image-invert.oo-ui-icon-userAvatarOutline {
  background-image: url('themes/wikimediaui/images/icons/userAvatarOutline-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatarOutline-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userAvatarOutline {
  background-image: url('themes/wikimediaui/images/icons/userAvatarOutline-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userAvatarOutline-progressive.svg');
}
.oo-ui-icon-userContributions {
  background-image: url('themes/wikimediaui/images/icons/userContributions-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userContributions-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-userContributions {
  background-image: url('themes/wikimediaui/images/icons/userContributions-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userContributions-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userContributions {
  background-image: url('themes/wikimediaui/images/icons/userContributions-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userContributions-rtl-progressive.svg');
}
.oo-ui-icon-userGroup {
  background-image: url('themes/wikimediaui/images/icons/userGroup-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userGroup-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-userGroup {
  background-image: url('themes/wikimediaui/images/icons/userGroup-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userGroup-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userGroup {
  background-image: url('themes/wikimediaui/images/icons/userGroup-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userGroup-rtl-progressive.svg');
}
.oo-ui-icon-userTalk {
  background-image: url('themes/wikimediaui/images/icons/userTalk-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userTalk-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-userTalk {
  background-image: url('themes/wikimediaui/images/icons/userTalk-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userTalk-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-userTalk {
  background-image: url('themes/wikimediaui/images/icons/userTalk-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/userTalk-rtl-progressive.svg');
}

.oo-ui-icon-menu {
  background-image: url('themes/wikimediaui/images/icons/menu.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/menu.svg');
}
.oo-ui-image-invert.oo-ui-icon-menu {
  background-image: url('themes/wikimediaui/images/icons/menu-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/menu-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-menu {
  background-image: url('themes/wikimediaui/images/icons/menu-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/menu-progressive.svg');
}
.oo-ui-icon-recentChanges {
  background-image: url('themes/wikimediaui/images/icons/recentChanges-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/recentChanges-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-recentChanges {
  background-image: url('themes/wikimediaui/images/icons/recentChanges-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/recentChanges-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-recentChanges {
  background-image: url('themes/wikimediaui/images/icons/recentChanges-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/recentChanges-rtl-progressive.svg');
}
.oo-ui-icon-textFlow {
  background-image: url('themes/wikimediaui/images/icons/textFlow-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textFlow-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-textFlow {
  background-image: url('themes/wikimediaui/images/icons/textFlow-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textFlow-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textFlow {
  background-image: url('themes/wikimediaui/images/icons/textFlow-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textFlow-rtl-progressive.svg');
}
.oo-ui-icon-textSummary {
  background-image: url('themes/wikimediaui/images/icons/textSummary-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textSummary-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-textSummary {
  background-image: url('themes/wikimediaui/images/icons/textSummary-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textSummary-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-textSummary {
  background-image: url('themes/wikimediaui/images/icons/textSummary-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/textSummary-rtl-progressive.svg');
}
.oo-ui-icon-viewCompact {
  background-image: url('themes/wikimediaui/images/icons/viewCompact.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewCompact.svg');
}
.oo-ui-image-invert.oo-ui-icon-viewCompact {
  background-image: url('themes/wikimediaui/images/icons/viewCompact-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewCompact-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-viewCompact {
  background-image: url('themes/wikimediaui/images/icons/viewCompact-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewCompact-progressive.svg');
}
.oo-ui-icon-viewDetails {
  background-image: url('themes/wikimediaui/images/icons/viewDetails-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewDetails-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-viewDetails {
  background-image: url('themes/wikimediaui/images/icons/viewDetails-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewDetails-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-viewDetails {
  background-image: url('themes/wikimediaui/images/icons/viewDetails-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/viewDetails-rtl-progressive.svg');
}

.oo-ui-icon-bright {
  background-image: url('themes/wikimediaui/images/icons/bright.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bright.svg');
}
.oo-ui-image-invert.oo-ui-icon-bright {
  background-image: url('themes/wikimediaui/images/icons/bright-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bright-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-bright {
  background-image: url('themes/wikimediaui/images/icons/bright-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/bright-progressive.svg');
}
.oo-ui-icon-halfBright {
  background-image: url('themes/wikimediaui/images/icons/halfBright-rtl.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfBright-rtl.svg');
}
.oo-ui-image-invert.oo-ui-icon-halfBright {
  background-image: url('themes/wikimediaui/images/icons/halfBright-rtl-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfBright-rtl-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-halfBright {
  background-image: url('themes/wikimediaui/images/icons/halfBright-rtl-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/halfBright-rtl-progressive.svg');
}
.oo-ui-icon-notBright {
  background-image: url('themes/wikimediaui/images/icons/notBright.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notBright.svg');
}
.oo-ui-image-invert.oo-ui-icon-notBright {
  background-image: url('themes/wikimediaui/images/icons/notBright-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notBright-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-notBright {
  background-image: url('themes/wikimediaui/images/icons/notBright-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/notBright-progressive.svg');
}
.oo-ui-icon-eye {
  background-image: url('themes/wikimediaui/images/icons/eye.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eye.svg');
}
.oo-ui-image-invert.oo-ui-icon-eye {
  background-image: url('themes/wikimediaui/images/icons/eye-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eye-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-eye {
  background-image: url('themes/wikimediaui/images/icons/eye-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eye-progressive.svg');
}
.oo-ui-icon-eyeClosed {
  background-image: url('themes/wikimediaui/images/icons/eyeClosed.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eyeClosed.svg');
}
.oo-ui-image-invert.oo-ui-icon-eyeClosed {
  background-image: url('themes/wikimediaui/images/icons/eyeClosed-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eyeClosed-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-eyeClosed {
  background-image: url('themes/wikimediaui/images/icons/eyeClosed-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/eyeClosed-progressive.svg');
}
.oo-ui-icon-moon {
  background-image: url('themes/wikimediaui/images/icons/moon.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moon.svg');
}
.oo-ui-image-invert.oo-ui-icon-moon {
  background-image: url('themes/wikimediaui/images/icons/moon-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moon-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-moon {
  background-image: url('themes/wikimediaui/images/icons/moon-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/moon-progressive.svg');
}
.oo-ui-icon-largerText {
  background-image: url('themes/wikimediaui/images/icons/largerText.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/largerText.svg');
}
.oo-ui-image-invert.oo-ui-icon-largerText {
  background-image: url('themes/wikimediaui/images/icons/largerText-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/largerText-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-largerText {
  background-image: url('themes/wikimediaui/images/icons/largerText-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/largerText-progressive.svg');
}
.oo-ui-icon-smallerText {
  background-image: url('themes/wikimediaui/images/icons/smallerText.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smallerText.svg');
}
.oo-ui-image-invert.oo-ui-icon-smallerText {
  background-image: url('themes/wikimediaui/images/icons/smallerText-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smallerText-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-smallerText {
  background-image: url('themes/wikimediaui/images/icons/smallerText-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/smallerText-progressive.svg');
}
.oo-ui-icon-visionSimulator {
  background-image: url('themes/wikimediaui/images/icons/visionSimulator.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/visionSimulator.svg');
}
.oo-ui-image-invert.oo-ui-icon-visionSimulator {
  background-image: url('themes/wikimediaui/images/icons/visionSimulator-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/visionSimulator-invert.svg');
}
.oo-ui-image-progressive.oo-ui-icon-visionSimulator {
  background-image: url('themes/wikimediaui/images/icons/visionSimulator-progressive.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/visionSimulator-progressive.svg');
}

.oo-ui-icon-logoCC {
  background-image: url('themes/wikimediaui/images/icons/logo-CC.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-CC.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoCC {
  background-image: url('themes/wikimediaui/images/icons/logo-CC-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-CC-invert.svg');
}
.oo-ui-icon-logoWikidata {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikidata.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikidata.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikidata {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikidata-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikidata-invert.svg');
}
.oo-ui-icon-logoWikimedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikimedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-invert.svg');
}
.oo-ui-icon-logoWikimediaCommons {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikimediaCommons {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Commons-invert.svg');
}
.oo-ui-icon-logoWikimediaDiscovery {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikimediaDiscovery {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikimedia-Discovery-invert.svg');
}
.oo-ui-icon-logoWikipedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikipedia.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikipedia.svg');
}
.oo-ui-image-invert.oo-ui-icon-logoWikipedia {
  background-image: url('themes/wikimediaui/images/icons/logo-Wikipedia-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/icons/logo-Wikipedia-invert.svg');
}

.oo-ui-indicator-clear {
  background-image: url('themes/wikimediaui/images/indicators/clear.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/clear.svg');
}
.oo-ui-image-invert.oo-ui-indicator-clear {
  background-image: url('themes/wikimediaui/images/indicators/clear-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/clear-invert.svg');
}
.oo-ui-indicator-up {
  background-image: url('themes/wikimediaui/images/indicators/up.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/up.svg');
}
.oo-ui-image-invert.oo-ui-indicator-up {
  background-image: url('themes/wikimediaui/images/indicators/up-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/up-invert.svg');
}
.oo-ui-indicator-down {
  background-image: url('themes/wikimediaui/images/indicators/down.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/down.svg');
}
.oo-ui-image-invert.oo-ui-indicator-down {
  background-image: url('themes/wikimediaui/images/indicators/down-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/down-invert.svg');
}
.oo-ui-indicator-required {
  background-image: url('themes/wikimediaui/images/indicators/required.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/required.svg');
}
.oo-ui-image-invert.oo-ui-indicator-required {
  background-image: url('themes/wikimediaui/images/indicators/required-invert.png');
  background-image: linear-gradient(transparent, transparent), /* @embed */ url('themes/wikimediaui/images/indicators/required-invert.svg');
}
